//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this library; see the file COPYING.LIB.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//
//#define _KVI_DEBUG_CLASS_NAME_ "KviChangers"
#include "kvi_app.h"

#include <kfontdialog.h>

KviColorChanger::KviColorChanger(QTabDialog *lpDlg,QWidget *parent,
									int x,int y,int w,int h,
									const char *szText,QColor *lpColor)
			:QWidget(parent,szText){
	m_lpColorBtn=new KColorButton((*lpColor),this);
	m_lpColor=lpColor;
	m_lpLabel=new QLabel(szText,this);
	setGeometry(x,y,w,h);
	m_lpColorBtn->setGeometry(w-70,0,70,h);
	m_lpLabel->setGeometry(0,0,w-70,h);
	connect(lpDlg,SIGNAL(applyButtonPressed()),this,SLOT(applySlot()));
}
KviColorChanger::~KviColorChanger(){
}

void KviColorChanger::applySlot(){
	(*m_lpColor)=m_lpColorBtn->color();
}
KviBoolChanger::KviBoolChanger(QTabDialog *lpDlg,QWidget *parent,
									int x,int y,int w,int h,
									const char *szText,bool *lpBool)
			:QCheckBox(szText,parent){
	m_lpBool=lpBool;
	setGeometry(x,y,w,h);
	setChecked((*m_lpBool));
	connect(lpDlg,SIGNAL(applyButtonPressed()),this,SLOT(applySlot()));
}
KviBoolChanger::~KviBoolChanger(){
}

void KviBoolChanger::applySlot(){
	(*m_lpBool)=isChecked();
}

//============ KviBackgroundChanger ============//

KviBackgroundChanger::KviBackgroundChanger(QTabDialog *lpDlg,QWidget *parent,int x,int y,int w,int h,
							const char *szTitle,QString *pixFileName,QPixmap *lpPixmap,
							QColor *lpColor)
		: QGroupBox(szTitle,parent)
{
	_debug_entertrace("KviBackgroundChanger");
	setGeometry(x,y,w,h);
	lpPixChanger = new KviPixmapChanger(lpDlg,this,15,50,width()-30,height()-70,
					"Background image\n\nPlease be careful to select\n"\
					"a 'real' image.\nA 'buggy' image\ncan hang kvirc.\n"\
					"If it happens,\nand kvirc refuses to\n"\
					"restart ,please delete\nthe kvirc file in the\n"\
					"$KDEDIR\\share\\config\n(or your $HOME\\.kde\\\nshare\\config\\)\n directory",
					pixFileName,lpPixmap); 
	lpClrChanger = new KviColorChanger(lpDlg,this,15,15,width()-30,30,
								"Background color",lpColor);
	_debug_leavetrace("KviBackgroundChanger");
}

//============ ~KviBackgroundChanger ============//

KviBackgroundChanger::~KviBackgroundChanger()
{
	_debug_entertrace("~KviBackgroundChanger");
	delete lpPixChanger;
	delete lpClrChanger;
	_debug_leavetrace("~KviBackgroundChanger");
}


//============ KviCaptionColorChanger ============//

KviCaptionColorChanger::KviCaptionColorChanger(QTabDialog *lpDlg,QWidget *parent,int x,int y,int w,int h,
							const char *szTitle,QString *pixFileName,QPixmap *lpPixmap,
							QColor *lpBkAct,QColor *lpBkInact,QColor *lpFrAct,
							QColor *lpFrInact,QFont *lpFont)
		: QGroupBox(szTitle,parent)
{
	_debug_entertrace("KviCaptionColorChanger");
	setGeometry(x,y,w,h);
	lpBkActChanger	=	new KviColorChanger(lpDlg,this,15,15,width()-30,30,
											"Active background color",lpBkAct);
	lpBkInactChanger=	new KviColorChanger(lpDlg,this,15,50,width()-30,30,
											"Inactive background color",lpBkInact);
	lpFrActChanger	=	new KviColorChanger(lpDlg,this,15,85,width()-30,30,
											"Active foreground color",lpFrAct);
	lpFrInactChanger=	new KviColorChanger(lpDlg,this,15,120,width()-30,30,
											"Inactive foreground color",lpFrInact);
	lpFontChanger	=	new KviFontChanger(lpDlg,this,15,155,width()-30,30,
											"Font",lpFont);
	lpPixChanger = new KviPixmapChanger(lpDlg,this,15,190,width()-30,height()-210,
										"Background image",pixFileName,lpPixmap); 
	_debug_leavetrace("KviCaptionColorChanger");
}

//============ ~KviBackgroundChanger ============//

KviCaptionColorChanger::~KviCaptionColorChanger()
{
	_debug_entertrace("~KviCaptionColorChanger");
	delete lpPixChanger;
	delete lpBkActChanger;
	delete lpBkInactChanger;
	delete lpFrActChanger;
	delete lpFrInactChanger;
	delete lpFontChanger;
	_debug_leavetrace("~KviCaptionColorChanger");
}


///////////////////////////////////////
KviFontChanger::KviFontChanger(QTabDialog *lpDlg,QWidget *parent,int x,int y,int w,int h,
							const char *szText,QFont *lpFont)
			:QWidget(parent,szText){
	m_lpButton=new QPushButton("Sample",this);
	m_lpButton->setFont(*lpFont);
	m_lpButton->repaint();
	connect(m_lpButton,SIGNAL(clicked()),this,SLOT(buttonPressed()));
	m_lpFont=lpFont;
	m_lpLabel=new QLabel(szText,this);
	setGeometry(x,y,w,h);
	m_lpButton->setGeometry(w-70,0,70,h);
	m_lpLabel->setGeometry(0,0,w-70,h);
	connect(lpDlg,SIGNAL(applyButtonPressed()),this,SLOT(applySlot()));
}
KviFontChanger::~KviFontChanger(){
}

void KviFontChanger::applySlot(){
	(*m_lpFont)=m_lpButton->font();
}
void KviFontChanger::buttonPressed(){
	KFontDialog aDialog(this,"KviFontChanger",true);
	aDialog.setFont(m_lpButton->font());
	if(aDialog.exec()==QDialog::Accepted){
		m_lpButton->setFont(aDialog.font());
		m_lpButton->repaint();
	}
}
///////////////////////////////////////
KviPixmapChanger::KviPixmapChanger(QTabDialog *lpDlg,QWidget *parent,
										int x,int y,int w,int h,
										const char *szText,QString *lpFileName,QPixmap *lpPixmap)
			:QWidget(parent,szText){
	m_lpPixmap=lpPixmap;

	m_lpFileName=lpFileName;
	m_lpBox=new QListBox(this);
	m_lpLabel=new QLabel(szText,this);
	m_lpPreview= new QLabel(this);
	m_lpPreview->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	setGeometry(x,y,w,h);
	m_lpPreview->setGeometry(w-70,0,70,h);
	m_lpBox->setGeometry(w-200,0,125,h);
	m_lpLabel->setGeometry(0,0,w-200,h);
	m_lpIL=_macro_kviIconLoader;
	QStrList *lpDirList=m_lpIL->getDirList();
	const char *szDir=0;
	uint idx_last=0;
	m_lpBox->insertItem("[__none__]");
	m_lpBox->setCurrentItem(0);
	int index=1;
	for(szDir=lpDirList->first();szDir;szDir=lpDirList->next()){
		QDir theDir(szDir,"*.gif");
		int cnt=theDir.count();
		for(int i=0;i<cnt;i++){
			m_lpBox->insertItem(theDir[i]);	
			if(strcmp(theDir[i],lpFileName->data())==0)idx_last=index;
			index++;
		}
	}
	for(szDir=lpDirList->first();szDir;szDir=lpDirList->next()){
		QDir theDir(szDir,"*.xpm");
		int cnt=theDir.count();
		for(int i=0;i<cnt;i++){
			m_lpBox->insertItem(theDir[i]);	
			if(strcmp(theDir[i],lpFileName->data())==0)idx_last=index;
			index++;
		}
	}
//	for(szDir=lpDirList->first();szDir;szDir=lpDirList->next()){
//		QDir theDir(szDir,"*.jpg");
//		int cnt=theDir.count();
//		for(int i=0;i<cnt;i++){
//			m_lpBox->insertItem(theDir[i]);	
//			if(strcmp(theDir[i],lpFileName->data())==0)idx_last=index;
//			index++;
//		}
//	}
	for(szDir=lpDirList->first();szDir;szDir=lpDirList->next()){
		QDir theDir(szDir,"*.ppm");
		int cnt=theDir.count();
		for(int i=0;i<cnt;i++){
			m_lpBox->insertItem(theDir[i]);	
			if(strcmp(theDir[i],lpFileName->data())==0)idx_last=index;
			index++;
		}
	}
	m_lpBox->setCurrentItem(idx_last);
	if(!lpPixmap->isNull())m_lpPreview->setBackgroundPixmap(*lpPixmap);
	connect(lpDlg,SIGNAL(applyButtonPressed()),this,SLOT(applySlot()));
	connect(m_lpBox,SIGNAL(highlighted(int)),this,SLOT(selectionChanged(int)));
}
KviPixmapChanger::~KviPixmapChanger(){
}

void KviPixmapChanger::applySlot(){
	const char *szFile=m_lpBox->text(m_lpBox->currentItem());
	if(strcmp(szFile,"[__none__]")==0){
		m_lpPixmap->resize(0,0);
		(*m_lpFileName)="[__none__]";
	} else {
		QPixmap pix=m_lpIL->loadIcon(szFile);
		if(!pix.isNull()){
			(*m_lpPixmap)=pix;
			(*m_lpFileName)=szFile;
		} else {
			m_lpPixmap->resize(0,0);
			(*m_lpFileName="[__none__]");
		}
	}
}

void KviPixmapChanger::selectionChanged(int idx){
	const char *szFile=m_lpBox->text(idx);
	if(strcmp(szFile,"[__none__]")==0){
		m_lpPreview->setBackgroundColor(QColor(80,80,80));
	} else {
		QPixmap pix=m_lpIL->loadIcon(szFile);
		if(!pix.isNull()){
			m_lpPreview->setBackgroundPixmap(pix);
		}
	}
	m_lpPreview->repaint();
}
#include "m_kvi_chng.moc"

