//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this library; see the file COPYING.LIB.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//
#include "kvi_app.h"


KviEventDialog::KviEventDialog(KviFrame *lpFrame):QDialog(lpFrame,"KviEventDialog",true)
{
	m_lpFrm=lpFrame;
	m_lpEventManager=lpFrame->m_lpEventManager;

	setCaption("Events");
	setMinimumWidth(400);
	setMinimumHeight(400);

	m_lpEdit    = new KviMultiLineEdit(this,lpFrame);
	m_lpDone    = new QPushButton("Done",this);
	m_lpBox     = new QComboBox(false,this);
	m_lpCheck   = new QCheckBox("Enabled",this);
	m_lpSyntax  = new QLabel(this);
	m_lpSyntax->setFrameStyle(QFrame::Sunken | QFrame::Panel);

	connect(m_lpDone,SIGNAL(clicked()),this,SLOT(accept()));
	connect(m_lpBox,SIGNAL(activated(int)),this,SLOT(selectionChanged(int)));
	connect(m_lpCheck,SIGNAL(toggled(bool)),this,SLOT(activeChanged(bool)));

	m_iLastSelection=-1;
	fillBox();
	resize(400,400);
}
KviEventDialog::~KviEventDialog()
{
	selectionChanged(-1);
	m_lpEventManager->saveToFile();
	delete m_lpEdit;
	delete m_lpDone;
	delete m_lpBox;
	delete m_lpSyntax;
}
void KviEventDialog::resizeEvent(QResizeEvent *)
{
	m_lpDone->setGeometry(width()-100,height()-30,95,25);
	m_lpBox->setGeometry(5,5,width()-10,25);
	m_lpCheck->setGeometry(5,40,width()/4,25);
	m_lpSyntax->setGeometry(10+(width()/4),40,width()-(15+(width()/4)),25);
	m_lpEdit->setGeometry(5,70,width()-10,height()-105);
}
void KviEventDialog::fillBox()
{
	for(int i=0;i<KVI_EVENTS_NUMBER;i++){
		m_lpBox->insertItem(m_lpEventManager->lpEvent[i]->szName.data());
	}
	m_lpBox->setCurrentItem(0);
	selectionChanged(0);
}
void KviEventDialog::selectionChanged(int idx)
{
	if(m_iLastSelection>=0){
		m_lpEventManager->lpEvent[m_iLastSelection]->szBuffer=m_lpEdit->text();
		m_lpEventManager->lpEvent[m_iLastSelection]->bEnabled=m_lpCheck->isChecked();
	}
	m_iLastSelection=idx;
	if(m_iLastSelection>=0){
		m_lpSyntax->setText(m_lpEventManager->appStr[(idx*2)+1]);
		m_lpEdit->setText(m_lpEventManager->lpEvent[m_iLastSelection]->szBuffer.data());
		m_lpCheck->setChecked(m_lpEventManager->lpEvent[m_iLastSelection]->bEnabled);
		m_lpEdit->setEnabled(m_lpEventManager->lpEvent[m_iLastSelection]->bEnabled);
		m_lpSyntax->setEnabled(m_lpEventManager->lpEvent[m_iLastSelection]->bEnabled);
		m_lpEdit->setFocus();
	}
}

//============ activeChanged ============//

void KviEventDialog::activeChanged(bool bOn)
{
	_debug_entertrace("activeChanged");
	int curSel=m_lpBox->currentItem();
	if(curSel != -1)m_lpEventManager->lpEvent[curSel]->bEnabled=bOn;
	m_lpEdit->setEnabled(bOn);
	if(bOn)m_lpEdit->setFocus();
	m_lpSyntax->setEnabled(bOn);
	_debug_leavetrace("activeChanged");
}



#include "m_kvi_evedlg.moc"
