//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this library; see the file COPYING.LIB.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//
#include "kvi_app.h"

//============ KviInterfaceDialog ============//

KviInterfaceDialog::KviInterfaceDialog(KviFrame *lpParent,KviInterface *lpInt)
						:QTabDialog(lpParent,"KviInterfaceDialogClass",true)
{
	_debug_entertrace("KviInterfaceDialog");
	m_lpInt=lpInt;
	setFixedWidth(400);
	setFixedHeight(400);
	setCaption("User interface settings");
	lpMdiTab		= new QWidget(this,"KviInterfaceDialogTab1");
	lpMdiChng		= new KviBackgroundChanger(this,lpMdiTab,10,10,360,300,
								"Main window background",&(m_lpInt->str_bk_mdi),
								&(m_lpInt->pix_bk_mdi),&(m_lpInt->clr_bk_mdi));
	lpCaptionTab	= new QWidget(this,"KviInterfaceDialogTab2");
	lpCaptionChng	= new KviCaptionColorChanger(this,lpCaptionTab,10,10,360,300,
								"Child caption options...",&(m_lpInt->str_bk_caption),
								&(m_lpInt->pix_bk_caption),&(m_lpInt->clr_bk_act_caption),
								&(m_lpInt->clr_bk_ina_caption),&(m_lpInt->clr_fr_act_caption),
								&(m_lpInt->clr_fr_ina_caption),&(m_lpInt->fnt_caption));
	lpInputTab		= new QWidget(this,"KviInterfaceDialogTab3");
	lpInputGroup	= new QGroupBox("Input box options...",lpInputTab);
	lpInputGroup->setGeometry(10,10,360,300);
	lpInBk			= new KviColorChanger(this,lpInputGroup,10,15,340,26,"Background color",&(m_lpInt->clr_bk_input));
	lpInFr			= new KviColorChanger(this,lpInputGroup,10,45,340,26,"Foreground color",&(m_lpInt->clr_fr_input));
	lpInSelBk		= new KviColorChanger(this,lpInputGroup,10,75,340,26,"Selection background",&(m_lpInt->clr_bk_sel_input));
	lpInSelFr		= new KviColorChanger(this,lpInputGroup,10,105,340,26,"Selection foreground",&(m_lpInt->clr_fr_sel_input));
	lpInCur			= new KviColorChanger(this,lpInputGroup,10,135,340,26,"Cursor & control marks",&(m_lpInt->clr_cursor_input));
	lpInFnt			= new KviFontChanger(this,lpInputGroup,10,165,340,26,"Font",&(m_lpInt->fnt_input));
	lpInPix			= new KviPixmapChanger(this,lpInputGroup,10,195,340,60,"Background image",&(m_lpInt->str_bk_input),(&m_lpInt->pix_bk_input));

	lpOutputTab		= new QWidget(this,"KviInterfaceDialogTab4");
	lpOutputGroup	= new QGroupBox("Output box options...",lpOutputTab);
	lpOutputGroup->setGeometry(10,10,360,300);
	lpOutBk			= new KviColorChanger(this,lpOutputGroup,10,15,340,26,"Background color",&(m_lpInt->clr_bk_output));
	lpOutSelBk		= new KviColorChanger(this,lpOutputGroup,10,45,340,26,"Selection background",&(m_lpInt->clr_bk_sel_output));
	lpOutSelFr		= new KviColorChanger(this,lpOutputGroup,10,75,340,26,"Selection foreground",&(m_lpInt->clr_fr_sel_output));
	lpOutFnt		= new KviFontChanger(this,lpOutputGroup,10,105,340,26,"Font",&(m_lpInt->fnt_output));
	lpOutPix		= new KviPixmapChanger(this,lpOutputGroup,10,135,340,60,"Background image",&(m_lpInt->str_bk_output),&(m_lpInt->pix_bk_output));

	lpListboxTab	= new QWidget(this,"KviInterfaceDialogTab5");
	lpListboxGroup	= new QGroupBox("Channel users list options...",lpListboxTab);
	lpListboxGroup->setGeometry(10,10,360,300);
	lpLBBk			= new KviColorChanger(this,lpListboxGroup,10,15,340,26,"Background color",&(m_lpInt->clr_bk_listbox));
	lpLBSelBk		= new KviColorChanger(this,lpListboxGroup,10,45,340,26,"Selection background",&(m_lpInt->clr_bk_sel_listbox));
	lpLBSelFr		= new KviColorChanger(this,lpListboxGroup,10,75,340,26,"Selection foreground",&(m_lpInt->clr_fr_sel_listbox));
	lpLBNorFr		= new KviColorChanger(this,lpListboxGroup,10,105,340,26,"Normal foreground",&(m_lpInt->clr_fr_normal_listbox));
	lpLBOpFr		= new KviColorChanger(this,lpListboxGroup,10,135,340,26,"Op foreground",&(m_lpInt->clr_fr_op_listbox));
	lpLBVoiFr		= new KviColorChanger(this,lpListboxGroup,10,165,340,26,"Voice foreground",&(m_lpInt->clr_fr_voice_listbox));
	lpLBFnt			= new KviFontChanger(this,lpListboxGroup,10,195,340,26,"Font",&(m_lpInt->fnt_listbox));
	lpLBPix			= new KviPixmapChanger(this,lpListboxGroup,10,225,340,60,"Background image",&(m_lpInt->str_bk_listbox),&(m_lpInt->pix_bk_listbox));

	lpLabelTab		= new QWidget(this,"KviInterfaceDialogTab6");
	lpLabelGroup	= new QGroupBox("Channel labels options...",lpLabelTab);
	lpLabelGroup->setGeometry(10,10,360,300);
	lpLablBk		= new KviColorChanger(this,lpLabelGroup,10,15,340,26,"Background color",&(m_lpInt->clr_bk_chanlabels));
	lpLablFr		= new KviColorChanger(this,lpLabelGroup,10,45,340,26,"Disabled foreground color",&(m_lpInt->clr_fr_chanlabels));
	lpLablActFr     = new KviColorChanger(this,lpLabelGroup,10,75,340,26,"Enabled foreground color",&(m_lpInt->clr_fr_act_chanlabels));
	lpLablFnt		= new KviFontChanger(this,lpLabelGroup,10,105,340,26,"Font",&(m_lpInt->fnt_chanlabels));
	lpLablPix		= new KviPixmapChanger(this,lpLabelGroup,10,135,340,90,"Backgrdound image",&(m_lpInt->str_bk_chanlabels),&(m_lpInt->pix_bk_chanlabels));

	addTab(lpMdiTab,"&Main");
	addTab(lpCaptionTab,"&Captions");
	addTab(lpInputTab,"&Input");
	addTab(lpOutputTab,"&Output");
	addTab(lpListboxTab,"&User list");
	addTab(lpLabelTab,"&Labels");
	//connect(this,SIGNAL(applyButtonPressed()),this,SLOT(apply()));
	setCancelButton();
	_debug_leavetrace("KviInterfaceDialog");
}

//============ ~KviInterfaceDialog ============//

KviInterfaceDialog::~KviInterfaceDialog()
{
	_debug_entertrace("~KviInterfaceDialog");
	delete lpLablPix;
	delete lpLablFnt;
	delete lpLablFr;
	delete lpLablActFr;
	delete lpLablBk;
	delete lpLabelGroup;
	delete lpLabelTab;
	delete lpLBPix;
	delete lpLBFnt;
	delete lpLBVoiFr;
	delete lpLBOpFr;
	delete lpLBNorFr;
	delete lpLBSelFr;
	delete lpLBSelBk;
	delete lpLBBk;
	delete lpListboxGroup;
	delete lpListboxTab;
	delete lpMdiChng;
	delete lpMdiTab;
	delete lpCaptionChng;
	delete lpCaptionTab;
	delete lpInBk;
	delete lpInFr;
	delete lpInSelBk;
	delete lpInSelFr;
	delete lpInCur;
	delete lpInFnt;
	delete lpInPix;
	delete lpInputGroup;
	delete lpInputTab;
	delete lpOutBk;
	delete lpOutSelBk;
	delete lpOutSelFr;
	delete lpOutFnt;
	delete lpOutPix;
	delete lpOutputGroup;
	delete lpOutputTab;
	_debug_leavetrace("~KviInterfaceDialog");
}

#include "m_kvi_intdlg.moc"
