//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CLASS_NAME_ "KviLinksDialog"

#include "kvi_app.h"

//============ KviLinksDialog ============//

KviLinksDialog::KviLinksDialog(KviFrame *lpFrm) : QDialog()
{
	_debug_entertrace("KviLinksDialog");
	m_lpFrm=lpFrm;
	setCaption("Server links");
	m_lpView = new QListView(this);
	m_lpView->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_lpView->addColumn("Host");
	m_lpView->addColumn("Link");
	m_lpView->addColumn("Description");
	QListViewItem *lpI=new QListViewItem(m_lpView,"No information available");
	lpI->setOpen(true);
//	m_lpView->insertItem(lpI);
	m_lpClose=new QPushButton("Close",this);
	m_lpGet=new QPushButton("Get links",this);
	connect(m_lpClose,SIGNAL(clicked()),this,SLOT(closeSlot()));
	connect(m_lpGet,SIGNAL(clicked()),this,SLOT(getSlot()));
	resize(300,300);
	show();
	m_bDone=false;
	_debug_leavetrace("KviLinksDialog");
}
//============ ~KviLinksDialog ============//
KviLinksDialog::~KviLinksDialog()
{
	_debug_entertrace("~KviLinksDialog");
	delete m_lpView;
	delete m_lpGet;
	delete m_lpClose;
	_debug_leavetrace("~KviLinksDialog");
}
//============ getChildItem ============//
QListViewItem *KviLinksDialog::getChildItem(QListViewItem *parent,const char *parenthost)
{
	_debug_entertrace("getChildItem");
	if(!strcasecmp(parent->text(0),parenthost))return parent;
	else {
		QListViewItem *lpI=parent->firstChild();
		if(lpI){
			QListViewItem *lpX=getChildItem(lpI,parenthost);
			if(lpX)return lpX;
		}
		lpI=parent->nextSibling();
		if(lpI){
			QListViewItem *lpX=getChildItem(lpI,parenthost);
			if(lpX)return lpX;
		}
	}
	return 0;
	_debug_leavetrace("getChildItem");
}
//============ getItemByHost ============//
QListViewItem *KviLinksDialog::getItemByHost(const char *parenthost)
{
	_debug_entertrace("getItemByHost");
	QListViewItem *lpI=m_lpView->firstChild();
	if(!lpI)return 0;
	return getChildItem(lpI,parenthost);
	_debug_leavetrace("getItemByHost");
}
//============ insertLink ============//
void KviLinksDialog::insertLink(const char *host,const char *parenthost,const char *description)
{
	_debug_entertrace("insertLink");
	if(!m_lpView->firstChild()){
		//first link
		QListViewItem *lpI=new QListViewItem(m_lpView,host,"Root server",description);
		lpI->setOpen(true);
		return;
	}
	QListViewItem *lpP=getItemByHost(parenthost);
	if(!lpP){
		debug("The server misordered something in the links....ouh! ignoring link...");
		return;
	}
	QString szLnk=parenthost;
	szLnk+="-";
	szLnk+=host;
	QListViewItem *lpI=new QListViewItem(lpP,host,szLnk.data(),description);
	lpI->setOpen(true);
	_debug_leavetrace("insertLink");
}
//============ clearList ============//
void KviLinksDialog::clearList()
{
	_debug_entertrace("clearList");
	m_lpView->clear();
	_debug_leavetrace("clearList");
}
//============ closeSlot ============//
void KviLinksDialog::closeSlot()
{
	_debug_entertrace("closeSlot");
	close();
	_debug_leavetrace("closeSlot");
}
//============ endOfLinks ============//
void KviLinksDialog::endOfLinks()
{
	_debug_entertrace("endOfLinks");
	m_lpGet->setEnabled(true);
	m_bDone=true;
	_debug_leavetrace("endOfLinks");
}
//============ getSlot ============//
void KviLinksDialog::getSlot()
{
	_debug_entertrace("getSlot");
	if(!m_lpFrm->m_lpSock->m_bConnected)return;
	m_lpGet->setEnabled(false);
	clearList();
	m_bDone=false;
	if(m_lpFrm->m_lpSock->sendData("LINKS")){
		m_lpGet->setEnabled(true);
		m_bDone=true;
	}
	_debug_leavetrace("getSlot");
}
//============ resizeEvent ============//
void KviLinksDialog::resizeEvent(QResizeEvent *)
{
	_debug_entertrace("resizeEvent");
	m_lpView->setGeometry(10,10,width()-20,height()-55);
	m_lpGet->setGeometry(10,height()-40,(width()-25)/2,30);
	m_lpClose->setGeometry(((width()-20)/2)+10,height()-40,(width()-25)/2,30);
	_debug_leavetrace("resizeEvent");
}
//============ closeEvent ============//
void KviLinksDialog::closeEvent(QCloseEvent *e)
{
	_debug_entertrace("closeEvent");
	e->accept();
	m_lpFrm->linksDialogClosed();
	delete this;
	_debug_leavetrace("closeEvent");
}
#include "m_kvi_links.moc"
