//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CLASS_NAME_ "KviMdiManager"
#include "kvi_app.h"

//============ KviMdiManager ============//
KviMdiManager::KviMdiManager(KviFrame *parent) : QFrame(parent,"KviMdiManagerClass")
{
	_debug_entertrace("KviMdiManager");
	m_lpFrm=parent;
	m_bMinimizingAll=false;
	m_lpInt	=	parent->m_lpInt;
	setFrameStyle(WinPanel | Sunken);
	m_lpChildList = new QList<KviMdiChild>; //In fact a stack...
	m_lpChildList->setAutoDelete(false);
	m_lpTaskBar = new KviTaskBar(parent,this);
	m_lpFrm->addToolBar(m_lpTaskBar);
	m_lpTaskBar->setBarPos(KToolBar::Bottom);
	linkChildren(m_lpTaskBar,this);
	m_lpWinListPopup = new QPopupMenu();
	setFocusPolicy(QWidget::StrongFocus);
	setFocusProxy(0);
	connect(m_lpTaskBar,SIGNAL(itemActivated(int)),this,SLOT(taskbarButtonClicked(int)));
	connect(m_lpTaskBar,SIGNAL(taskBarPopupRequest(QPoint,int)),this,SLOT(taskbarPopupRequest(QPoint,int)));
	connect(m_lpWinListPopup,SIGNAL(activated(int)),this,SLOT(menuButtonClicked(int)));
	_debug_leavetrace("KviMdiManager");
}
//================= linkChildren ==============//
void KviMdiManager::linkChildren(QWidget *widget,QWidget *focusedOne)
{
	_debug_entertrace_huge("linkChildren");
	QList<QObject> *list = (QList<QObject> *)(widget->children());
	if(list){
		for (unsigned int i=0; i< list->count(); i++){
			QWidget *w = (QWidget *)list->at(i);
			linkChildren(w,focusedOne);
		}
	}
	widget->installEventFilter(this);
	if(widget != focusedOne){
		widget->setFocusPolicy(NoFocus);
		widget->setFocusProxy(focusedOne);
	}
	_debug_leavetrace_huge("linkChildren");
}
//============ ~KviMdiManager ============//
KviMdiManager::~KviMdiManager()
{
	_debug_entertrace("~KviMdiManager");
	while(!m_lpChildList->isEmpty()){
		KviMdiChild *lpC=m_lpChildList->last();
		m_lpChildList->removeLast();
		lpC->killWindow();
	}
	delete m_lpChildList;
	delete m_lpTaskBar;
	delete m_lpWinListPopup;
	_debug_leavetrace("~KviMdiManager");
}
//============ applyOptions ============//
void KviMdiManager::applyOptions()
{
	_debug_entertrace("applyOptions");
	if(!m_lpInt->pix_bk_mdi.isNull())setBackgroundPixmap(m_lpInt->pix_bk_mdi);
	else setBackgroundColor(m_lpInt->clr_bk_mdi);
	KviMdiChild *lpC=0;
	KviMdiChild *lastCh=0;
	for(lpC=m_lpChildList->first();lpC;lpC=m_lpChildList->next()){
		lpC->applyOptionsInternal();
		lpC->applyOptions();
		lastCh=lpC;
	}
	if(lastCh)lastCh->setFocus();
	_debug_leavetrace("applyOptions");
}
//============ focusTopChild ============//
void KviMdiManager::focusTopChild()
{
	_debug_entertrace("focusTopChild");
	//work around a qt(?) bug...right clicking
	//in the window when the application is not active
	//the main widget gets focused, but for some strange
	//reason it is not raised to the top of the windows stack...
	//Window wnd=(Window)topLevelWidget()->winId();
	Window wnd=(Window)m_lpFrm->winId();
	if(KWM::desktop(wnd)==KWM::currentDesktop()){
		if(!KWM::isActive(wnd))KWM::raise(wnd);
		KviMdiChild *lpC=topChildInZOrder();
		if(lpC){
			lpC->raise();
			lpC->setFocus();
			if(!lpC->m_bLabelRaised)lpC->raiseLabel(true);
		}
	}
	_debug_leavetrace("focusTopChild");
}
//============ focusInEvent ============//
void KviMdiManager::focusInEvent(QFocusEvent *)
{
	_debug_entertrace("focusInEvent");
	focusTopChild();
	_debug_leavetrace("focusInEvent");
}
//============ eventFiter ============//
bool KviMdiManager::eventFilter(QObject *,QEvent *e)
{
	_debug_entertrace_huge("eventFiter");
	if(e->type()==Event_FocusIn){
		focusTopChild();
		return true;
	}
	if(e->type()==Event_FocusOut){
		return true;
	}
	_debug_leavetrace_huge("eventFiter");
	return false;
}
//============ mousePressEvent ============//
void KviMdiManager::mousePressEvent(QMouseEvent *e)
{
	_debug_entertrace("mousePressEvent");
	if(e->button() & RightButton)m_lpWinListPopup->popup(mapToGlobal(e->pos()));
	_debug_leavetrace("mousePressEvent");
}
//============ addChild ============//
void KviMdiManager::addChild(KviMdiChild *lpC,const char *szIconName,bool setGeom)
{
	_debug_entertrace("addChild");
	KviMdiChild *lpChild=0;
	m_lpChildList->append(lpC);
	m_lpTaskBar->addButton(lpC->id(),lpC->name(),szIconName,(lpC->type()==KVI_WND_TYPE_SEND));
	_macro_kviApplication->processEvents();
	//position the new child...(like cascade...)
	int chx=0;
	int chy=0;
	if(setGeom){
			for(lpChild=m_lpChildList->first();lpChild;lpChild=m_lpChildList->next()){
				if(lpChild==lpC && (lpChild->id()!=0))
					lpChild->setGeometry(chx,chy,KVI_MDI_CHILD_CASCADE_WIDTH,KVI_MDI_CHILD_CASCADE_HEIGHT);
				chx+=KVI_MDI_CHILD_CASCADE_STEP; 			chy+=KVI_MDI_CHILD_CASCADE_STEP;
				if(chy>(height()-(KVI_MDI_CHILD_CASCADE_STEP*2))){
					chy=0;
					chx+=KVI_MDI_CHILD_CASCADE_JUMPX;
				}
			}
	}
	lpC->setIcon(szIconName);
	lpC->show();
	lpC->raise();
	lpC->setFocus();
	lpC->raiseLabel(true);
	connect(lpC,SIGNAL(windowMaximized(KviMdiChild *)),this,SLOT(childMaximized(KviMdiChild *)));
	connect(lpC,SIGNAL(windowMinimized(KviMdiChild *)),this,SLOT(childMinimized(KviMdiChild *)));
	fillWinListPopup();
	_debug_leavetrace("addChild");
}
//============ addChildMinimized ============//
void KviMdiManager::addChildMinimized(KviMdiChild *lpC,const char *szIconName,bool setGeom)
{
	_debug_entertrace("addChildMinimized");
	m_lpChildList->insert(0,lpC);
	m_lpTaskBar->addButton(lpC->id(),lpC->name(),szIconName,(lpC->type()==KVI_WND_TYPE_SEND));
	_macro_kviApplication->processEvents();
	//position the new child...(like cascade...)
	int chx=0;
	int chy=0;
	KviMdiChild *lpChild=0;
	if(setGeom){
		for(lpChild=m_lpChildList->first();lpChild;lpChild=m_lpChildList->next()){
			if(lpChild==lpC && (lpChild->id()!=0))lpChild->setGeometry(chx,chy,KVI_MDI_CHILD_CASCADE_WIDTH,KVI_MDI_CHILD_CASCADE_HEIGHT);
			chx+=KVI_MDI_CHILD_CASCADE_STEP;
			chy+=KVI_MDI_CHILD_CASCADE_STEP;
			if(chy>(height()-(KVI_MDI_CHILD_CASCADE_STEP*2))){
				chy=0;
				chx+=KVI_MDI_CHILD_CASCADE_JUMPX;
			}
		}
	}
	lpC->setIcon(szIconName);
	lpC->m_bMinimized=true;
	lpC->lower();
	lpC->hide();
	connect(lpC,SIGNAL(windowMaximized(KviMdiChild *)),this,SLOT(childMaximized(KviMdiChild *)));
	connect(lpC,SIGNAL(windowMinimized(KviMdiChild *)),this,SLOT(childMinimized(KviMdiChild *)));
	fillWinListPopup();
	focusTopChild();
	_debug_leavetrace("addChildMinimized");
}
//============ topChildInZOrder ============//
KviMdiChild * KviMdiManager::topChildInZOrder()
{
	_debug_entertrace("topChildInZOrder");
	return (m_lpChildList->isEmpty() ? 0 : m_lpChildList->last());
	_debug_leavetrace("topChildInZOrder");
}
//============ childFocusIn ============//
void KviMdiManager::childFocusIn(KviMdiChild *lpC)
{
	_debug_entertrace("childFocusIn");
	KviMdiChild *lpFoc=0;
	for(lpFoc=m_lpChildList->first();lpFoc;lpFoc=m_lpChildList->next()){
		if(lpFoc != lpC){
			lpFoc->clearFocus();
			if(lpFoc->m_bLabelRaised)lpFoc->raiseLabel(false);
		}
	}
	//tricky maximized management
	lpFoc=m_lpChildList->last();
	if(m_lpChildList->removeRef(lpC)){
		m_lpChildList->append(lpC);
		m_lpTaskBar->setActiveButton(lpC->id());
	}
//	if(lpFoc && (lpFoc != lpC)){
//		if(lpFoc->m_bMaximized && (!lpFoc->m_bMinimized)){
//			if(!lpC->m_bMaximized){
//				lpC->maximizeWindow(false);
//				lpC->raise();
//			} else lpC->raise();
//			lpFoc->maximizeWindow(false);
//		} else lpC->raise();
//	} else lpC->raise();
	lpC->raise();
	_debug_leavetrace("childFocusIn");
}
//============ childFocusOut ============//
void KviMdiManager::childFocusOut(KviMdiChild *)
{
	_debug_entertrace("childFocusOut");
	//do Nothing 'til now...
	_debug_leavetrace("childFocusOut");
}
KviMdiChild * KviMdiManager::findWindow(const char *szName){
	_debug_entertrace("findWindow");
	if(m_lpChildList->isEmpty())return 0;
	KviMdiChild *lpC=0;
	for(lpC=m_lpChildList->first();lpC;lpC=m_lpChildList->next()){
		if(!strcasecmp(lpC->name(),szName)){
			_debug_leavetrace("findWindow...found.");
			return lpC;
		}
	}
	debug("Unable to find child window %s",szName);
	return 0;
}
KviMdiChild * KviMdiManager::findWindow(const QString &szName){
	_debug_entertrace("findWindow");
	if(m_lpChildList->isEmpty())return 0;
	KviMdiChild *lpC=0;
	for(lpC=m_lpChildList->first();lpC;lpC=m_lpChildList->next()){
		QString szC(lpC->name());
		if(szName.lower()==szC.lower()){
			_debug_leavetrace("findWindow...found.");
			return lpC;
		}
	}
	debug("Unable to find child window %s",szName.data());
	return 0;
}
KviMdiChild * KviMdiManager::findWindow(int winId){
	_debug_entertrace("findWindow");
	if(m_lpChildList->isEmpty())return 0;
	KviMdiChild *lpC=0;
	for(lpC=m_lpChildList->first();lpC;lpC=m_lpChildList->next()){
		if(winId==lpC->id()){
			_debug_leavetrace("findWindow....found.");
			return lpC;
		}
	}
	debug("Unable to find child window %d",winId);
	return 0;
}
//============ removeWindow ============//
void KviMdiManager::removeWindow(int winId)
{
	_debug_entertrace("removeWindow");
	KviMdiChild *lpC=findWindow(winId);
	if(lpC)removeWindow(lpC);
	_debug_leavetrace("removeWindow");
}
//============ removeWindow ============//
void KviMdiManager::removeWindow(const char *szName)
{
	_debug_entertrace("removeWindow");
	KviMdiChild *lpC=findWindow(szName);
	if(lpC)removeWindow(lpC);
	_debug_leavetrace("removeWindow");
}
//============ removeWindow ============//
void KviMdiManager::removeWindow(KviMdiChild *lpC)
{
	_debug_entertrace("removeWindow");
//	if(lpC==m_lpMaximizedChild)m_lpMaximizedChild=0;
	m_lpTaskBar->removeButton(lpC->id());
	lpC->hide();
	m_lpChildList->setAutoDelete(false);
	m_lpChildList->removeRef(lpC);
	lpC->killWindow();
	focusTopChild();
	fillWinListPopup();
	_debug_leavetrace("removeWindow");
}
void KviMdiManager::taskbarButtonClicked(int nId){
	_debug_entertrace("taskbarButtonClicked");
	KviMdiChild *lpC=findWindow(nId);
	if(lpC){
		if((!lpC->hasFocus())||(lpC->m_bMinimized)){
			if(lpC->m_bMinimized)lpC->minimizeWindow(); 
			lpC->setFocus();
		}
	}
	_debug_leavetrace("taskbarButtonClicked");
}
void KviMdiManager::menuButtonClicked(int nId){
	_debug_entertrace("menuButtonClicked");
	if(nId>=0){
		KviMdiChild *lpC=findWindow(nId);
		if(lpC){
			if(lpC->m_bMinimized)lpC->minimizeWindow(); //restore :)
			lpC->setFocus();
		}
	}
	_debug_leavetrace("menuButtonClicked");
}
void KviMdiManager::taskbarPopupRequest(QPoint pnt,int nId){
	_debug_entertrace("taskbarPopupRequest");
	KviMdiChild *lpC=findWindow(nId);
	if(lpC)lpC->showSystemMenu(pnt);
	_debug_leavetrace("taskbarPopupRequest");
}
void KviMdiManager::childMaximized(KviMdiChild *lpC){
	_debug_entertrace("childMaximized");
	_debug_leavetrace("childMaximized");
}
void KviMdiManager::childMinimized(KviMdiChild *lpC){
	_debug_entertrace("childMinimized");
	m_lpTaskBar->unpressAllButtons();
	if(!m_bMinimizingAll)focusNextChild();
	_debug_leavetrace("childMinimized");
}
void KviMdiManager::focusNextChild(){
	_debug_entertrace("focusNextChild");
	KviMdiChild *lpC;
	lpC = m_lpChildList->last(); //top of the stack...
	if(!lpC)return; //can't focus nothing....
	while(lpC->m_bMinimized||(!lpC->isVisible())){
		lpC=m_lpChildList->prev();
		if(!lpC){
			_debug_warning("NULL : There should be no windows visible...");
			return;
		}
	}
	lpC->raise();
	lpC->setFocus();
	_debug_leavetrace("focusNextChild");
}
void KviMdiManager::resizeEvent(QResizeEvent *)
{
	_debug_entertrace("resizeEvent");
	KviMdiChild *lpC=0;
	for(lpC=m_lpChildList->first();lpC;lpC=m_lpChildList->next()){
		if(lpC->m_bMaximized)lpC->setGeometry(rect());
	}
	_debug_leavetrace("resizeEvent");
}

void KviMdiManager::cascadeWindows()
{
	_debug_entertrace("cascadeWindows");
	if(!m_lpChildList->isEmpty()){
		KviMdiChild *lpChild=0;
		int chx=0;
		int chy=0;
		KviMdiChild *prevChild=0;
		for(lpChild=m_lpChildList->first();lpChild;lpChild=m_lpChildList->next()){
			if(lpChild->m_bMaximized)lpChild->maximizeWindow(); //restore windows if any
			if(!lpChild->m_bMinimized){ //skip the minimized windows
				lpChild->setGeometry(chx,chy,KVI_MDI_CHILD_CASCADE_WIDTH,KVI_MDI_CHILD_CASCADE_HEIGHT);
				chx+=KVI_MDI_CHILD_CASCADE_STEP;
				chy+=KVI_MDI_CHILD_CASCADE_STEP;
				if(chy>(height()-(KVI_MDI_CHILD_CASCADE_STEP*2))){
					chy=0;
					chx+=KVI_MDI_CHILD_CASCADE_JUMPX;
				}
				if(prevChild)prevChild->clearFocus();
				prevChild=lpChild;
			}
		}
		if(prevChild){
			prevChild->raise();
			prevChild->setFocus();
		}
	}
	_debug_leavetrace("cascadeWindows");
}
void KviMdiManager::restoreAllWindows()
{
	_debug_entertrace("restoreAllWindows");
	if(!m_lpChildList->isEmpty()){
		KviMdiChild *lpChild=0;
		KviMdiChild *lastChild=0;
		QList<KviMdiChild> list(*m_lpChildList);
		for(lpChild=list.first();lpChild;lpChild=list.next()){
			if(lpChild->m_bMaximized)lpChild->maximizeWindow(); //restore windows if any
			if(lpChild->m_bMinimized)lpChild->minimizeWindow();
			if(lastChild)lastChild->clearFocus();
			if(lpChild)lastChild=lpChild;
		}
		if(lastChild){
			lastChild->setFocus();
		}
	}
	_debug_leavetrace("restoreAllWindows");
}
void KviMdiManager::minimizeAllWindows()
{
	_debug_entertrace("minimizeAllWindows");
	if(!m_lpChildList->isEmpty()){
		KviMdiChild *lpChild;
		m_bMinimizingAll=true;
		for(lpChild=m_lpChildList->first();lpChild;lpChild=m_lpChildList->next()){
			if(!lpChild->m_bMinimized){
				lpChild->minimizeWindow();
			}
		}
		m_bMinimizingAll=false;
		m_lpTaskBar->unpressAllButtons();
//		m_lpMaximizedChild=0;
	}
	_debug_leavetrace("minimizeAllWindows");
}
void KviMdiManager::highlightWindow(int nID)
{
	_debug_entertrace("highlightWindow");
	m_lpTaskBar->highlightButton(nID);
	_debug_leavetrace("highlightWindow");
}
void KviMdiManager::fillWinListPopup(){
	_debug_entertrace("fillWinListPopup");
	KviMdiChild *lpChild;
	KIconLoader *lpIL=_macro_kviIconLoader;
	m_lpWinListPopup->clear();
	m_lpWinListPopup->insertItem(lpIL->loadIcon("kvii_casc.gif"),"Cascade",this,SLOT(cascadeWindows()));
	m_lpWinListPopup->insertItem(lpIL->loadIcon("kvii_rstr.gif"),"Restore all",this,SLOT(restoreAllWindows()));
	m_lpWinListPopup->insertItem(lpIL->loadIcon("kvii_mnmz.gif"),"Minimize all",this,SLOT(minimizeAllWindows()));
	m_lpWinListPopup->insertItem(lpIL->loadIcon("kvii_horz.gif"),"Tile horizontally",this,SLOT(tileHorz()));
	m_lpWinListPopup->insertItem(lpIL->loadIcon("kvii_vert.gif"),"Tile vertically",this,SLOT(tileVert()));
	m_lpWinListPopup->insertSeparator();
	if(!m_lpChildList->isEmpty()){
		for(lpChild=m_lpChildList->first();lpChild;lpChild=m_lpChildList->next()){
			//debug("inserting : %s",lpChild->name());
			m_lpWinListPopup->insertItem((*lpChild->getIcon()),lpChild->name(),lpChild->id());
		}
	}
	_debug_leavetrace("fillWinListPopup");
}
void KviMdiManager::tileVert(){
	_debug_entertrace("tileVert");
	int cnt=0;
	KviMdiChild *lpChild;
	if(!m_lpChildList->isEmpty()){
		for(lpChild=m_lpChildList->first();lpChild;lpChild=m_lpChildList->next()){
			if(!lpChild->m_bMinimized){
				cnt++;
				if(lpChild->m_bMaximized)lpChild->maximizeWindow(); //restore;
			}
		}
	} else return;
	if(cnt){
		int wdt=(width()/cnt);
		int ax=0;
		for(lpChild=m_lpChildList->first();lpChild;lpChild=m_lpChildList->next()){
			if(!lpChild->m_bMinimized){
				lpChild->move(ax,0);
				lpChild->resize(wdt,height());
				ax+=wdt;
			}
		}
	}
	_debug_leavetrace("tileVert");
}
void KviMdiManager::tileHorz(){
	_debug_entertrace("tileHorz");
	int cnt=0;
	KviMdiChild *lpChild;
	if(!m_lpChildList->isEmpty()){
		for(lpChild=m_lpChildList->first();lpChild;lpChild=m_lpChildList->next()){
			if(!lpChild->m_bMinimized){cnt++;
				if(lpChild->m_bMaximized)lpChild->maximizeWindow(); //restore;
			}
		}
	} else return;
	if(cnt){
		int hgh=(height()/cnt)-1;
		int ay=0;
		for(lpChild=m_lpChildList->first();lpChild;lpChild=m_lpChildList->next()){
			if(!lpChild->m_bMinimized){
				lpChild->move(0,ay);
				lpChild->resize(width(),hgh);
				ay+=hgh;
			}
		}
	}
	_debug_leavetrace("tileVert");
}

#include "m_kvi_mdi.moc"
