//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//
#include "kvi_app.h"

KviNick::KviNick(){
	szNick=KVI_STR_NULL;
	szUserName=KVI_STR_NULL;
	szAddress=KVI_STR_NULL;
	bOp=false;
	bVoice=false;
}
KviNick::KviNick(QString &szMask){
	fromString(szMask);
}

//============ fromString ============//

void KviNick::fromString(QString &szString)
{
	_debug_entertrace("fromString");
	register const char *pC=szString.data();
	ASSERT(pC);
	while((*pC)&&(isspace(*pC)))pC++;
	const char *pApp=pC;
	if(*pC){
		while((*pC) && (*pC != '!'))pC++;
		if(*pC){ //Complete mask
			bOp=false;
			bVoice=false;
			szNick=QString(pApp,(pC-pApp)+1);
			if(pC==pApp)szNick=KVI_STR_NULL;
			pC++;
			if(*pC){ //Nick!something
				pApp=pC;
				while((*pC) && (*pC != '@'))pC++;
				szUserName=QString(pApp,(pC-pApp)+1);
				if(pC==pApp)szUserName=KVI_STR_NULL;
				if(*pC){ //complete mask
					pC++;
					if(*pC){ //complete mask
						//pApp=pC;
						//while(*pC)pC++;
						szAddress=QString(pC);
						if(szAddress.isEmpty())szAddress=KVI_STR_NULL;
						return;
					} else { //Nick!username@
						szAddress=KVI_STR_NULL;
						return;
					}
				} else { //nick!username
					szAddress=KVI_STR_NULL;
					return;
				}
			} else { //Nick!
				szAddress=KVI_STR_NULL;
				szUserName=KVI_STR_NULL;
				return;
			}
		} else { //Only a nick
			pC=szString.data();
			if(*pC=='@'){
				bOp=true;
				pC++;
			} else bOp=false;
			if(*pC=='+'){
				bVoice=true;
				pC++;
			} else bVoice=false;
			if(*pC){ //OK copy the nick
				szNick=QString(pC);
				while((*pC) && (*pC != '.'))pC++;
				if(*pC=='.'){
					szUserName="ircd"; //irc daemon :)
					szAddress=szNick.copy();  //  irc.dal.net (ircd@irc.dal.net)"
				} else {
					szUserName=KVI_STR_NULL;
					szAddress=KVI_STR_NULL;
				}
				return;			
			} else { //Huh??? anly a '@' or '+' ??????
				debug("Corrupted nickmask : %s",szString.data());
				ASSERT(pC);
			}
		}
	} else {
		szNick=KVI_STR_NULL;
		szAddress=KVI_STR_NULL;
		szUserName=KVI_STR_NULL;
	}
	_debug_leavetrace("fromString");
}

KviNick::KviNick(const KviNick &kNick){
	szNick=kNick.szNick.copy();
	szUserName=kNick.szUserName.copy();
	szAddress=kNick.szAddress.copy();
	bOp=kNick.bOp;
	bVoice=kNick.bVoice;
}
KviNick::~KviNick(){
}

void KviNick::mask(QString &szMask,int type){
	if(szNick.isEmpty())szNick=KVI_STR_NULL;
	if(szUserName.isEmpty())szUserName=KVI_STR_NULL;
	if(szAddress.isEmpty())szAddress=KVI_STR_NULL;
	switch(type){
		case 0: //Nick!User@foo.foobar.com
			szMask.sprintf("%s!%s@%s",szNick.data(),szUserName.data(),szAddress.data());
			break;
		case 1: //User@foo.fobar.com
			szMask.sprintf("%s@%s",szUserName.data(),szAddress.data());
			break;
		case 2: //Nick!*@foo.foobar.com
			szMask.sprintf("%s!*@%s",szNick.data(),szAddress.data());
			break;
		case 3: //*!User@foo.foobar.com
			szMask.sprintf("*!%s@%s",szUserName.data(),szAddress.data());
			break;
		case 4: //Nick!*@*
			szMask.sprintf("%s!*@*",szNick.data());
			break;
		case 5: //*!*@*.foobar.com or *!*@255.255.255.*
			szMask="*!*@";
			{
				const char *pC=szAddress.data();
				bool bIp=true;
				while(*pC && bIp){
					if(*pC=='.' || ((*pC>='0')&&(*pC<='9')) )pC++;
					else bIp=false;
				}
				if(bIp){
					int idx=szAddress.findRev('.');
					szMask+=szAddress.left(idx);
					szMask+=".*";
				} else {
					szMask+="*";
					pC=szAddress.data();
					while((*pC) && (!(*pC=='.')))pC++;
					szMask+=QString(pC);
				}
			}
			break;
		case 6: //*!user@*.foobar.com or *!user@255.255.255.*
			szMask="*!";
			szMask+=szUserName.data();
			szMask+='@';
			{
				const char *pC=szAddress.data();
				bool bIp=true;
				while(*pC && bIp){
					if(*pC=='.' || ((*pC>='0')&&(*pC<='9')) )pC++;
					else bIp=false;
				}
				if(bIp){
					int idx=szAddress.findRev('.');
					szMask+=szAddress.left(idx);
					szMask+=".*";
				} else {
					szMask+="*";
					pC=szAddress.data();
					while((*pC) && (!(*pC=='.')))pC++;
					szMask+=QString(pC);
				}
			}
			break;
		case 7: //nick!user@*.foobar.com or nick!user@255.255.255.*
			szMask=szNick.data();
			szMask+='!';
			szMask+=szUserName.data();
			szMask+='@';
			{
				const char *pC=szAddress.data();
				bool bIp=true;
				while(*pC && bIp){
					if(*pC=='.' || ((*pC>='0')&&(*pC<='9')) )pC++;
					else bIp=false;
				}
				if(bIp){
					int idx=szAddress.findRev('.');
					szMask+=szAddress.left(idx);
					szMask+=".*";
				} else {
					szMask+="*";
					pC=szAddress.data();
					while((*pC) && (!(*pC=='.')))pC++;
					szMask+=QString(pC);
				}
			}
			break;	
		case 8: //nick!*@*.foobar.com or nick!*@255.255.255.*
			szMask=szNick.data();
			szMask+="!*@";
			{
				const char *pC=szAddress.data();
				bool bIp=true;
				while(*pC && bIp){
					if(*pC=='.' || ((*pC>='0')&&(*pC<='9')) )pC++;
					else bIp=false;
				}
				if(bIp){
					int idx=szAddress.findRev('.');
					szMask+=szAddress.left(idx);
					szMask+=".*";
				} else {
					szMask+="*";
					pC=szAddress.data();
					while((*pC) && (!(*pC=='.')))pC++;
					szMask+=QString(pC);
				}
			}
			break;		
		default:
			szMask.sprintf("%s!%s@%s",szNick.data(),szUserName.data(),szAddress.data());
			break;	
	}
}
bool KviNick::isOp(){
	return bOp;
}
bool KviNick::isVoice(){
	return bVoice;
}
bool KviNick::matches(QString &szMask){
	int idx=szMask.find('!');
	if(idx == -1){ //it's only a nick...
		QString szM=szMask.stripWhiteSpace();
		if(szM[0]=='@' || szM[0]=='+')szM.remove(0,1);
		QString szm=szMask.lower();
		QString szn=szNick.lower();
		return (szm==szn);
	}
	QRegExp rMask((szMask.lower()).data(),false,true);
	QString szTmpM;
	mask(szTmpM);
	QRegExp myMask((szTmpM.lower()).data(),false,true);
	if(rMask.match(myMask.pattern()) != -1)return true;
	return (myMask.match(rMask.pattern()) != -1);
	
}
bool KviNick::matches(const char *szMask){
	QString szTheMask(szMask);
	return matches(szTheMask);
}
