//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this library; see the file COPYING.LIB.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CLASS_NAME_ "KviSendWnd"

#include "kvi_app.h"

KviSendWnd::KviSendWnd(KviMdiManager *parent,KviFrame *frame,const char *aname,int aid):KviMdiChild(parent,frame,aname,aid,KVI_WND_TYPE_SEND)
{
	//GUI
	m_lpOutput=new KviView(this,frame,"KviViewClass");
	m_lpProgress=new KProgress(0,100,0,KProgress::Horizontal,this,"KviProgressClass");
	m_lpLabel=new QLabel("No connection active.",this);
	m_lpLabel->setFrameStyle(QFrame::Sunken | QFrame::Panel);
	m_szFileName="";
	m_lpFile=0;
	m_lpListeningSocket=0;
	m_Sock=0;
	m_iPort=0;
	m_bConnected=false;
	m_bSending=false;
	m_lpReadNotifier=0;
	m_lpWriteNotifier=0;
	m_lpTimer=0;
	m_iNumAck=0;
	setFocusOwner(m_lpProgress);
	applyOptions();
	connect(this,SIGNAL(closeButtonPressed()),this,SLOT(closeSlot()));
}

KviSendWnd::~KviSendWnd()
{
	m_lpFrm->m_lpUserParser->m_szLastSendTarget="";
	destroyFile();
	killSock();
	delete m_lpLabel;
	delete m_lpProgress;
	delete m_lpOutput;
}
//============ closeSlot ============//
void KviSendWnd::closeSlot()
{
	_debug_entertrace("closeSlot");
	killSock();
	hide();
	_macro_kviApplication->processEvents();
	blockSignals(true);
	m_lpMdi->m_lpTaskBar->removeButton(id());
	m_lpMdi->m_lpChildList->setAutoDelete(false);
	m_lpMdi->m_lpChildList->removeRef(this);
	m_lpMdi->focusTopChild();
	m_lpMdi->fillWinListPopup();
	m_lpFrm->queryExternalDestroy(this);
	_debug_leavetrace("closeSlot");
}

void KviSendWnd::applyOptions(){
	m_lpOutput->setFont(m_lpInt->fnt_output);
	m_lpLabel->setFont(m_lpInt->fnt_chanlabels);
	QPalette labelPal=m_lpLabel->palette();
	QColorGroup aCG(labelPal.normal());
	QColorGroup nCG(aCG.foreground(),aCG.background(),aCG.light(),
			 	aCG.dark(), aCG.mid(),m_lpInt->clr_fr_chanlabels, aCG.base()); 
	labelPal.setNormal(nCG);
	m_lpOutput->m_bTimestamp=m_lpFrm->m_lpOpt->bTimestamp;
	m_lpOutput->m_bShowPixmaps=m_lpFrm->m_lpOpt->bShowPixmaps;
	m_lpLabel->setPalette(labelPal);
	m_lpLabel->setBackgroundColor(m_lpInt->clr_bk_chanlabels);
	m_lpLabel->update();
}

void KviSendWnd::resizeEvent(QResizeEvent *){
	_debug_entertrace("resizeEvent");
	updateRects();
	QRect rct=viewSizeHint();
	QFontMetrics fm=m_lpLabel->fontMetrics();
	int labl_hght=fm.lineSpacing()+6;
	m_lpOutput->setGeometry(rct.left(),
							rct.top()+labl_hght+KVI_PROGRESS_HEIGHT+(2*KVI_MDI_VIEW_SEP),rct.width(),
							rct.height()-(KVI_PROGRESS_HEIGHT+(2*KVI_MDI_VIEW_SEP)+labl_hght));
	m_lpProgress->setGeometry(rct.left(),rct.top(),rct.width(),KVI_PROGRESS_HEIGHT);
	m_lpLabel->setGeometry(rct.left(),rct.top()+KVI_PROGRESS_HEIGHT+KVI_MDI_VIEW_SEP,rct.width(),labl_hght);
	_debug_leavetrace("resizeEvent");
}
void KviSendWnd::doFmtOutput(int nType,const char *szFmt,...){
	char szText[600]; //It should be big enough... I hope...
	va_list list;
	va_start(list,szFmt);
	vsprintf(szText,szFmt,list);
	va_end(list);
	m_lpMdi->highlightWindow(m_iId);
	m_lpOutput->appendText(nType,(const char *) &szText);
}
void KviSendWnd::doOutput(int nType,const char *szText){
	m_lpMdi->highlightWindow(m_iId);
	m_lpOutput->appendText(nType,szText);
}
////////////////////////////////////////


//============ initiateResumeGet ============//

void KviSendWnd::initiateResumeGet()
{
	_debug_entertrace("initiateResumeGet");
	QString szRealIp=QString(inet_ntoa(m_inAddress));
	doFmtOutput(KVI_OUT_INTERNAL,"RESUME ACCEPTED : Connecting to %s on port %s",szRealIp.data(),m_szPort.data());
	struct sockaddr_in sockAddrIn;
	m_lpFrm->m_lpSock->fillSockAddr(&sockAddrIn,m_inAddress,m_iPort);
	m_Sock = ::socket(PF_INET,SOCK_STREAM,0);
	if (m_Sock < 0){
		doOutput(KVI_OUT_ERROR,"Unable to create STREAM socket for connection");
		m_lpFrm->setStatusText("DCC GET failed");
		return;
	}
	if(fcntl(m_Sock, F_SETFL, O_NONBLOCK) < 0){
		::close(m_Sock);
		doOutput(KVI_OUT_ERROR,"Unable to create STREAM socket for connection (fcntl failed)");
		m_lpFrm->setStatusText("DCC GET failed");
		return;
	};
	m_lpTimer=new QTimer();
	QObject::connect(m_lpTimer,SIGNAL(timeout()),this,SLOT(connectTimedOut()));
	m_lpTimer->start(m_lpFrm->m_lpOpt->iDCCCHATTimeout * 1000);
	if(::connect( m_Sock, (struct sockaddr*)(&sockAddrIn), sizeof(sockAddrIn) )<0){
		if(errno != EINPROGRESS){
			killSock();
			int errore=m_lpFrm->m_lpSock->translateFromSystemError(errno);
			QString szError;
			m_lpFrm->m_lpSock->getErr(errore,szError);
			doFmtOutput(KVI_OUT_ERROR,"Unable to connect to %s : %s",szRealIp.data(),szError.data());
			return;
		}
	}
	m_lpReadNotifier = new QSocketNotifier(m_Sock, QSocketNotifier::Read,this);
	m_lpWriteNotifier= new QSocketNotifier(m_Sock, QSocketNotifier::Write,this);
	QObject::connect(m_lpReadNotifier, SIGNAL(activated(int)), this, SLOT(slotDataInBuffer(int)));
	QObject::connect(m_lpWriteNotifier, SIGNAL(activated(int)), this, SLOT(slotConnected(int)));
	_debug_leavetrace("initiateResumeGet");
}


//============ acceptSend ============//

bool KviSendWnd::acceptSend(const char *szNick,QString &szPort,QString &szIp,const char *szFileName,QString szSize,bool bResume)
{
	_debug_entertrace("acceptSend");
	//
	//  here we accept a DCC send request from another user
	//  If bResume is true...that means that we have another file with that name on the HD
	//
	QString szStat;
	//  set the status text
	szStat.sprintf("Accepting DCC SEND connection from %s",szNick);
	m_lpFrm->setStatusText(szStat.data());
	m_szRemoteNick=QString(szNick);
	//setup the file
	bool bOK=false;
	m_iFileLength=szSize.toUInt(&bOK);
	if(!bOK)m_iFileLength=0;

	m_szFileName=szFileName;
	if(m_lpFile||m_bConnected)_debug_fatal("Can not use this window two times");
	//create the file
	m_lpFile=new QFile(szFileName);
	if(bResume){
		if(!m_lpFile->exists()){
			delete m_lpFile;
			m_lpFile=0;
			doFmtOutput(KVI_OUT_ERROR,"Ops...file doesn't exist..cant start a RESUME session\n"\
											"Can not open file %s for appending",szFileName);
			m_lpFrm->setStatusText("DCC RESUME GET failed");
			return false;	
		}
		if(!m_lpFile->open(IO_WriteOnly | IO_Append)){
			delete m_lpFile;
			m_lpFile=0;
			doFmtOutput(KVI_OUT_ERROR,"Can not open file %s for appending",szFileName);
			m_lpFrm->setStatusText("DCC RESUME GET failed");
			return false;
		}
	} else {
		if(!m_lpFile->open(IO_WriteOnly)){
			delete m_lpFile;
			m_lpFile=0;
			doFmtOutput(KVI_OUT_ERROR,"Can not open file %s for writing",szFileName);
			m_lpFrm->setStatusText("DCC GET failed");
			return false;
		}
	}
	m_iPort=(unsigned short int)szPort.toUInt(&bOK);
	if(!bOK){
		doOutput(KVI_OUT_ERROR,"Can't retrieve the port to connect. DCC GET failed.");
		m_lpFrm->setStatusText("DCC GET failed");
		return false;	
	}
	szIp=szIp.stripWhiteSpace();
	unsigned long iAddr=szIp.toULong(&bOK);
	if(!bOK){
		doFmtOutput(KVI_OUT_ERROR,"Can't retrieve the address to connect. DCC GET failed (%s).",szIp.data());
		m_lpFrm->setStatusText("DCC GET failed");
		return false;	
	}
	//struct in_addr inAddress;
	m_inAddress.s_addr=ntohl(iAddr);

	if(bResume){
		QFileInfo aFileInfo(*m_lpFile);
		m_iFileRead=aFileInfo.size();
		m_lpFrm->m_lpSock->sendFmtData("PRIVMSG %s :%cDCC RESUME %s %s %u%c",szNick,0x01,
				aFileInfo.fileName().data(),szPort.data(),m_iFileRead,0x01);
		doFmtOutput(KVI_OUT_INTERNAL,"Sent DCC RESUME [%s:%u of %u total] request to %s...waiting for reply",
			aFileInfo.fileName().data(),m_iFileRead,m_iFileLength,szNick);
		m_szPort=szPort.data();
		return true;
	}

	QString szRealIp=QString(inet_ntoa(m_inAddress));
	doFmtOutput(KVI_OUT_INTERNAL,"Connecting to %s on port %s",szRealIp.data(),szPort.data());
	struct sockaddr_in sockAddrIn;
	m_lpFrm->m_lpSock->fillSockAddr(&sockAddrIn,m_inAddress,m_iPort);
	m_Sock = ::socket(PF_INET,SOCK_STREAM,0);
	if (m_Sock < 0){
		doOutput(KVI_OUT_ERROR,"Unable to create STREAM socket for connection");
		m_lpFrm->setStatusText("DCC GET failed");
		return false;
	}
	if(fcntl(m_Sock, F_SETFL, O_NONBLOCK) < 0){
		::close(m_Sock);
		doOutput(KVI_OUT_ERROR,"Unable to create STREAM socket for connection (fcntl failed)");
		m_lpFrm->setStatusText("DCC GET failed");
		return false;
	};
	m_lpTimer=new QTimer();
	QObject::connect(m_lpTimer,SIGNAL(timeout()),this,SLOT(connectTimedOut()));
	m_lpTimer->start(m_lpFrm->m_lpOpt->iDCCCHATTimeout * 1000);
	if(::connect( m_Sock, (struct sockaddr*)(&sockAddrIn), sizeof(sockAddrIn) )<0){
		if(errno != EINPROGRESS){
			killSock();
			int errore=m_lpFrm->m_lpSock->translateFromSystemError(errno);
			QString szError;
			m_lpFrm->m_lpSock->getErr(errore,szError);
			doFmtOutput(KVI_OUT_ERROR,"Unable to connect to %s : %s",szRealIp.data(),szError.data());
			return false;
		}
	}
	m_iFileRead=0;
	m_lpReadNotifier = new QSocketNotifier(m_Sock, QSocketNotifier::Read,this);
	m_lpWriteNotifier= new QSocketNotifier(m_Sock, QSocketNotifier::Write,this);
	QObject::connect(m_lpReadNotifier, SIGNAL(activated(int)), this, SLOT(slotDataInBuffer(int)));
	QObject::connect(m_lpWriteNotifier, SIGNAL(activated(int)), this, SLOT(slotConnected(int)));
	_debug_leavetrace("acceptSend");
	return true;
}
//============ slotConnected ============//

void KviSendWnd::slotConnected(int)
{
	_debug_entertrace("slotConnected");
	if(m_lpWriteNotifier){
		delete m_lpWriteNotifier;
		m_lpWriteNotifier=0;
	}
	destroyTimer();
	m_bConnected=true;
	m_startTime=0;
	m_bSending=false;
	doOutput(KVI_OUT_INTERNAL,"Connected.");
	_debug_leavetrace("slotConnected");
}

//============ requestSend ============//

bool KviSendWnd::requestSend(const char *szNick,const char *szFileName)
{
	_debug_entertrace("requestSend");
	QString szStat;
	szStat.sprintf("Requesting DCC SEND connection to %s",szNick);
	m_lpFrm->setStatusText(szStat.data());
	m_szFileName=szFileName;
	if(m_lpFile||m_bConnected)_debug_fatal("Can not use this window two times");
	m_lpFile=new QFile(szFileName);
	if(!m_lpFile->open(IO_ReadOnly)){
		delete m_lpFile;
		m_lpFile=0;
		doFmtOutput(KVI_OUT_ERROR,"Can not open file %s for reading",szFileName);
		m_lpFrm->setStatusText("DCC SEND Request failed");
		return false;
	}
	QFileInfo aFileInfo(*m_lpFile);
	m_iFileLength=aFileInfo.size();
	m_szRemoteNick=QString(szNick);
	if(!m_lpFrm->m_lpSock->m_bConnected){
		doOutput(KVI_OUT_ERROR,"DCC SEND Requires an active IRC connection");
		m_lpFrm->setStatusText("DCC SEND Request failed");
		return false;
	}
	unsigned long myAddr=m_lpFrm->m_lpSock->getSockAddress();
	if(!myAddr){
		doOutput(KVI_OUT_ERROR,"Unable to resolve localhost, can't request DCC SEND");
		m_lpFrm->setStatusText("DCC SEND Request failed");
		return false;
	}
	m_lpListeningSocket=new KviListeningSocket();
	if(!m_lpListeningSocket->listenForConnection(0,&m_Sock,m_lpFrm)){
		doOutput(KVI_OUT_ERROR,"Unable to setup a listening socket, can't request a DCC SEND");
		m_lpFrm->setStatusText("DCC SEND Request failed");
		killSock();
		return false;
	}
	QObject::connect(m_lpListeningSocket,SIGNAL(gotConnection()),this,SLOT(connectedSlot()));
	QObject::connect(m_lpListeningSocket,SIGNAL(connectTimedOut()),this,SLOT(connectTimedOut()));
	QString szAddr;
	QString szPort;
	szAddr.setNum(myAddr);
	szPort.setNum(m_lpListeningSocket->m_iPort);
	m_iPort=m_lpListeningSocket->m_iPort;
	m_lpFrm->m_lpSock->sendFmtData("PRIVMSG %s :%cDCC SEND %s %s %s %u%c",szNick,0x01,
				aFileInfo.fileName().data(),szAddr.data(),szPort.data(),m_iFileLength,0x01);
	doFmtOutput(KVI_OUT_INTERNAL,"Sent DCC SEND [%s:%u] request to %s...waiting for reply",aFileInfo.fileName().data(),m_iFileLength,szNick);
	m_iFileSent=0;
	m_bSending=true;
	_debug_leavetrace("requestSend");
	return true;
}

//============ dccResumeForCurrentSend ============//

bool KviSendWnd::dccResumeForCurrentSend(uint iPosition)
{
	_debug_entertrace("dccResumeForCurrentSend");
	// iPosition is the file size already sent (say 100 bytes)
	m_iFileSent=iPosition; //so sent 100 bytes
	doFmtOutput(KVI_OUT_INTERNAL,"%s requested a DCC RESUME for %s (%u bytes)",m_szRemoteNick.data(),m_szFileName.data(),m_iFileLength);
	if(m_iFileLength<=iPosition){
		doFmtOutput(KVI_OUT_INTERNAL,"WARNING : File position requested is bigger that file size.Resuming NORMAL dcc send!");
		m_iFileSent=0;
		return false;
	}
	m_lpFile->at(iPosition); //and the 101st byte is at position 100 in the file (0 based index)
	doFmtOutput(KVI_OUT_INTERNAL,"Acknowledged DCC RESUME : Transfer will start from position %u",iPosition);
	_debug_leavetrace("dccResumeForCurrentSend");
	return true;
}

//============ destoryTimer ============//

void KviSendWnd::destroyTimer()
{
	_debug_entertrace("destoryTimer");
	if(m_lpTimer){
		if(m_lpTimer->isActive())m_lpTimer->stop();
		delete m_lpTimer;
		m_lpTimer=0;
	}
	_debug_leavetrace("destoryTimer");
}


//============ destroyReadNotifier ============//

void KviSendWnd::destroyReadNotifier()
{
	_debug_entertrace("destroyReadNotifier");
	if(m_lpReadNotifier){
		if(m_lpReadNotifier->isEnabled())m_lpReadNotifier->setEnabled(false);
		delete m_lpReadNotifier;
		m_lpReadNotifier=0;
	}
	_debug_leavetrace("destroyReadNotifier");
}


//============ destroyWriteNotifier ============//

void KviSendWnd::destroyWriteNotifier()
{
	_debug_entertrace("destroyWriteNotifier");
	if(m_lpWriteNotifier){
		if(m_lpWriteNotifier->isEnabled())m_lpWriteNotifier->setEnabled(false);
		delete m_lpWriteNotifier;
		m_lpWriteNotifier=0;
	}
	_debug_leavetrace("destroyWriteNotifier");
}


//============ killSock ============//

void KviSendWnd::killSock()
{
	_debug_entertrace("killSock");
	destroyListeningSocket();
	destroyWriteNotifier();
	destroyReadNotifier();
	destroyTimer();
	destroyFile();
	if(m_Sock != -1)::close(m_Sock);
	m_bConnected=false;
	_debug_leavetrace("killSock");
}
//============ connectedSlot ============//
void KviSendWnd::connectedSlot()
{
	_debug_entertrace("connectedSlot");
	m_bSending=true;
	QString szName=QString(inet_ntoa(m_lpListeningSocket->connectedAddr.sin_addr));
	doFmtOutput(KVI_OUT_INTERNAL,"Connected to %s on port %u",szName.data(),m_iPort);
	m_lpReadNotifier=new QSocketNotifier(m_Sock,QSocketNotifier::Read,this);
	QObject::connect(m_lpReadNotifier, SIGNAL(activated(int)), this, SLOT(slotDataInBuffer(int)));
	m_bConnected=true;
	m_startTime=time(0);
	sendFileBuffer();
	_debug_leavetrace("connectedSlot");
}
//============ connectTimedOut ============//
void KviSendWnd::connectTimedOut()
{
	_debug_entertrace("connectTimedOut");
	terminateByError("SEND","by a socket timeout");
	_debug_leavetrace("connectTimedOut");
}

//============ terminateByError ============//

void KviSendWnd::terminateByError(const char *szType,const char *szErr)
{
	_debug_entertrace("terminateByError");
	QString szStat;
	szStat.sprintf("DCC %s session with %s terminated",szType,m_szRemoteNick.data());
	m_lpFrm->setStatusText(szStat.data());
	doFmtOutput(KVI_OUT_ERROR,"DCC %s session with %s terminated %s",szType,m_szRemoteNick.data(),szErr);
	if(m_lpListeningSocket)m_lpListeningSocket->stopListening();
	destroyReadNotifier();
	destroyWriteNotifier();
	destroyTimer();
	destroyFile();
	if(m_Sock != -1)::close(m_Sock);
	m_bConnected=false;
	_debug_leavetrace("terminateByError");
}


//============ terminateBySuccess ============//

void KviSendWnd::terminateBySuccess(const char *szType)
{
	_debug_entertrace("terminateBySuccess");
	QString szStat;
	szStat.sprintf("DCC %s with %s terminated",szType,m_szRemoteNick.data());
	m_lpFrm->setStatusText(szStat.data());
	doFmtOutput(KVI_OUT_INTERNAL,"DCC %s with %s succesfully terminated",szType,m_szRemoteNick.data(),errno);
	killSock();
	if(m_lpFrm->m_lpOpt->bAutoCloseDCCChatOnSuccess)emit closeButtonPressed();
	_debug_leavetrace("terminateBySuccess");
}


//============ destroyFile ============//

void KviSendWnd::destroyFile()
{
	_debug_entertrace("destroyFile");
	if(m_lpFile){
		m_lpFile->close();
		delete m_lpFile;
		m_lpFile=0;
	}
	_debug_leavetrace("destroyFile");
}


//============ destroyListeningSocket ============//

void KviSendWnd::destroyListeningSocket()
{
	_debug_entertrace("destroyListeningSocket");
	if(m_lpListeningSocket){
		delete m_lpListeningSocket;
		m_lpListeningSocket=0;
	}
	_debug_leavetrace("destroyListeningSocket");
}


//============ slotDataInBuffer ============//

void KviSendWnd::slotDataInBuffer(int)
{
	_debug_entertrace("slotDataInBuffer");
	destroyListeningSocket();
	if(m_bSending){	
		uint ack;
		int readLength = read(m_Sock,&ack,sizeof(int));
		uint numAck=ntohl(ack);
		if(readLength<=0){
			if(readLength==0)terminateByError("SEND","(remote end closed connection)");
			else if((errno != EINTR)&&(errno != EAGAIN))terminateByError("SEND","by a transmission error");
		} else {
			if(numAck>=m_iFileLength){
				destroyFile();
				setLabelInfo(m_iNumAck,m_iFileSent);
				if(m_iFileLength != 0)m_lpProgress->setValue((numAck * 100)/m_iFileLength);
				m_lpMdi->m_lpTaskBar->setButtonProgress(id(),(numAck * 100)/m_iFileLength);
				terminateBySuccess("SEND");
				return;
			} else sendFileBuffer();
			m_iNumAck=numAck;
			setLabelInfo(m_iNumAck,m_iFileSent);
			if(m_iFileLength != 0)m_lpProgress->setValue((numAck * 100)/m_iFileLength);
			m_lpMdi->m_lpTaskBar->setButtonProgress(id(),(numAck * 100)/m_iFileLength);
		}
	} else {
		char szIncomingData[2048];
   		bzero(szIncomingData, 2048);
		int readLength = read(m_Sock,szIncomingData, 2048);
		if (readLength <= 0){
			if(readLength==0){
				if(m_iFileLength == m_iFileRead)terminateBySuccess("GET");
				else terminateByError("GET","(remote end closed connection)");
			} else if((errno != EINTR)&&(errno != EAGAIN))terminateByError("GET","by a transmission error");
  		} else {
			m_iFileRead+=readLength;
			setLabelInfo(m_iFileRead,0);
			uint toAckBytes=htonl(m_iFileRead);
			if(m_lpFile->writeBlock(szIncomingData,readLength)<0){
				terminateByError("GET","by a file I/O Error");
				return;
			}
			::write(m_Sock,&toAckBytes,sizeof(int));
			if(m_iFileLength != 0){
				m_lpProgress->setValue((m_iFileRead * 100)/m_iFileLength);
				m_lpMdi->m_lpTaskBar->setButtonProgress(id(),(m_iFileRead * 100)/m_iFileLength);	
				if(m_iFileLength==m_iFileRead)doFmtOutput(KVI_OUT_INTERNAL,"File transfer complete");
			}
		}
	}
	_debug_leavetrace("slotDataInBuffer");
}

//============ setLabelInfo ============//

void KviSendWnd::setLabelInfo(uint iBytesLeft,uint iBytesRight)
{
	_debug_entertrace("setLabelInfo");
	uint dur=(time(0)-m_startTime);
	if(dur==0)dur=1;
	uint bps=iBytesLeft/dur;
	QString szTxt;
	if(m_iFileLength){
		if(m_bSending){
			szTxt.sprintf("[File: %s (%u bytes)][Ack: %u bytes][Sent: %u bytes][Time: %u sec.][%u bytes/sec]",
						m_szFileName.data(),m_iFileLength,iBytesLeft,iBytesRight,dur,bps);
		} else {
			szTxt.sprintf("[File: %s (%u bytes)][Received: %u bytes][Time: %u sec][%u bytes/sec]",
						m_szFileName.data(),m_iFileLength,iBytesLeft,dur,bps); 
		}
	} else {
		if(m_bSending){
			szTxt.sprintf("[File: %s (unknown size)][Ack: %u bytes][Sent: %u bytes][Time: %u sec.][%u bytes/sec]",
						m_szFileName.data(),iBytesLeft,iBytesRight,dur,bps);
		} else {
			szTxt.sprintf("[File: %s (unknown size)][Received: %u bytes][Time: %u sec][%u bytes/sec]",
						m_szFileName.data(),iBytesLeft,dur,bps); 
		}
	}
	m_lpLabel->setText(szTxt.data());
	_debug_leavetrace("setLabelInfo");
}


//============ sendFileBuffer ============//

void KviSendWnd::sendFileBuffer()
{
	_debug_entertrace("sendFileBuffer");
	int sizeToRead=m_iFileLength-m_iFileSent;
	if(sizeToRead<=0)return;
	if(sizeToRead>m_lpFrm->m_lpOpt->iDCCSendPacketSize)sizeToRead=m_lpFrm->m_lpOpt->iDCCSendPacketSize;
	char szBlock[4096];
	if(m_lpFile->readBlock(szBlock,sizeToRead)<0){
		terminateByError("SEND","by a file I/O Error");
		return;
	}
	if(::write(m_Sock,szBlock,sizeToRead)<sizeToRead){
		terminateByError("SEND","by a socket I/O Error");
		return;
	}
	m_iFileSent+=sizeToRead;
	setLabelInfo(m_iNumAck,m_iFileSent);
	_debug_leavetrace("sendFileBuffer");
}

#include "m_kvi_send.moc"
