//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CLASS_NAME_ "KviSetup"

#include "kvi_app.h"

#include "kvi_setup.h"


//============ KviSetup ============//

KviSetup::KviSetup()
{
	_debug_entertrace("KviSetup");
	_debug_leavetrace("KviSetup");
}

//============ ~KviSetup ============//

KviSetup::~KviSetup()
{
	_debug_entertrace("~KviSetup");
	_debug_leavetrace("~KviSetup");
}

//============ setup ============//

void KviSetup::setup()
{
	_debug_entertrace("setup");
	//setup our icon loader
	QString szDir=getKVIrcHomeDirectory("Scripts/Images");
	_macro_kviIconLoader->insertDirectory(0,szDir);
	szDir=getKVIrcHomeDirectory("Images");
	_macro_kviIconLoader->insertDirectory(0,szDir);
	//check if $KDE/share/apps/kvirc exists.
	szDir=getGlobalDataDirectory(0);
	if(!directoryExists(szDir.data())){
		KMsgBox::message(0,"WARNING","Can not find the global application share directory.\n" \
													"You forgot make install or your KDE is misconfigured.\n");
		return;
	}
	//check for $HOME/kvirc.
	szDir=getKVIrcHomeDirectory(0);
	if(!directoryExists(szDir.data()))executeSetup(false);
	else {
		bool bNeedSetup=true;
		szDir=getKVIrcHomeDirectory("Audio");
		bNeedSetup=(!directoryExists(szDir.data()));
		szDir=getKVIrcHomeDirectory("Ascii");
		bNeedSetup=(!directoryExists(szDir.data()));
		szDir=getKVIrcHomeDirectory("Incoming");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Config");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Logs");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Images");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Scripts");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Scripts/Audio");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Scripts/Images");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Scripts/Misc");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Scripts/Ascii");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Translation");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		szDir=getKVIrcHomeDirectory("Misc");
		bNeedSetup=((!directoryExists(szDir.data())) || bNeedSetup);
		if(bNeedSetup)executeSetup(true);
	}
	_debug_leavetrace("setup");
}


//============ executeSetup ============//

void KviSetup::executeSetup(bool bRepeat)
{
	_debug_entertrace("executeSetup");
	if(!bRepeat)KMsgBox::message(0,"KVirc auto-setup","WELCOME to the KVirc auto-setup.\n\n"\
														"A 'kvirc' directory tree will be created in your home directory,\n"\
														"and some default configuration files will be stored there.");
	else KMsgBox::message(0,"KVirc auto-setup","KVirc detected a misconfiguraion in the $HOME/kvirc directory tree.\n"\
														"The setup will be now repeated to rebuild the correct kvirc dependancies.");

	QString szDirsCreated;
	szDirsCreated+=createHomeDirIfNotExists(0);
	szDirsCreated+=createHomeDirIfNotExists("Audio");
	szDirsCreated+=createHomeDirIfNotExists("Ascii");
	szDirsCreated+=createHomeDirIfNotExists("Incoming");
	szDirsCreated+=createHomeDirIfNotExists("Config");
	szDirsCreated+=createHomeDirIfNotExists("Logs");
	szDirsCreated+=createHomeDirIfNotExists("Images");
	szDirsCreated+=createHomeDirIfNotExists("Scripts");
	szDirsCreated+=createHomeDirIfNotExists("Scripts/Audio");
	szDirsCreated+=createHomeDirIfNotExists("Scripts/Misc");
	szDirsCreated+=createHomeDirIfNotExists("Scripts/Images");
	szDirsCreated+=createHomeDirIfNotExists("Scripts/Ascii");
	szDirsCreated+=createHomeDirIfNotExists("Translation");
	szDirsCreated+=createHomeDirIfNotExists("Misc");
	if(!szDirsCreated.isEmpty()){
		szDirsCreated.prepend("The following directories were created :\n");
		KMsgBox::message(0,"KVirc auto-setup",szDirsCreated.data());
	}
	szDirsCreated="";
	szDirsCreated+=copyAllFrom(0);
	szDirsCreated+=copyAllFrom("Audio");
	szDirsCreated+=copyAllFrom("Ascii");
	szDirsCreated+=copyAllFrom("Incoming");
	szDirsCreated+=copyAllFrom("Config");
	szDirsCreated+=copyAllFrom("Logs");
	szDirsCreated+=copyAllFrom("Images");
	szDirsCreated+=copyAllFrom("Scripts");
	szDirsCreated+=copyAllFrom("Scripts/Audio");
	szDirsCreated+=copyAllFrom("Scripts/Images");
	szDirsCreated+=copyAllFrom("Scripts/Misc");
	szDirsCreated+=copyAllFrom("Scripts/Ascii");
	szDirsCreated+=copyAllFrom("Translation");
	szDirsCreated+=copyAllFrom("Misc");
	if(!szDirsCreated.isEmpty()){
		szDirsCreated.prepend("The following files were created :\n");
		if(szDirsCreated.length()>512){
			szDirsCreated.truncate(512);
			szDirsCreated.append("...\n...\n...more files");
		}
		KMsgBox::message(0,"KVirc auto-setup",szDirsCreated.data());
	}

	KMsgBox::message(0,"KVirc auto-setup","Setup terminated.");
	_debug_leavetrace("executeSetup");
}


//============ copyAllFrom ============//

QString KviSetup::copyAllFrom(const char *subdir)
{
	_debug_entertrace("copyAllFrom");
	QString szFilesCopied="";
	QString szDir=getGlobalDataDirectory(subdir);
	QDir theDir(szDir.data());
	const QFileInfoList * lpList=theDir.entryInfoList();
	if(!lpList)return szFilesCopied;
	QFileInfoListIterator it(*lpList);
	for( ; it.current() ; ++it){
		QFileInfo *lpInfo=it.current();
		if(lpInfo->isFile()){
			QString szFileName=lpInfo->absFilePath();
			if(!lpInfo->isReadable()){
				QString szMsg="Unable to read file ";
				szMsg+=szFileName;
				szMsg+="\nIt's possible that you don't have the right permissions to do it.\n";
				szMsg+="Since this is a default configuration file,KVirc will work without it\n";
				szMsg+="But you may want to ask your system administrator to copy that file for you\n";
				szMsg+="or give you read access to that file.";
				KMsgBox::message(0,"KVirc auto-setup",szMsg.data());
			} else {
				uint fSize=lpInfo->size();
				if(fSize){
					QString szDirectory=getKVIrcHomeDirectory(subdir);
					szDirectory+='/';
					szDirectory+=lpInfo->fileName();
					QFile newOne(szDirectory.data());
					QFile oldOne(szFileName.data());
					if(!newOne.exists()){
						if(oldOne.exists()){
							if(newOne.open(IO_WriteOnly)){
								if(oldOne.open(IO_ReadOnly)){
									char block[4096];
									int readBlock=4096;
									while(fSize){
										if(fSize<4096)readBlock=fSize;
										fSize-=readBlock;
										if(oldOne.readBlock(block,readBlock)==readBlock){
											if(newOne.writeBlock(block,readBlock)<readBlock){
												QString szMsg="File I/O error while writing ";
												szMsg+=szDirectory.data();
												KMsgBox::message(0,"KVirc auto-setup",szMsg.data());
												fSize=0;
											}					
										} else {
											QString szMsg="File I/O error while reading ";
											szMsg+=szFileName.data();
											KMsgBox::message(0,"KVirc auto-setup",szMsg.data());
											fSize=0;
										}
									}
									szFilesCopied+=szDirectory;
									szFilesCopied+='\n';
									oldOne.close();
									newOne.close();
								} else {
									QString szMsg="Unable to open file ";
									szMsg+=szFileName.data();
									szMsg+=" for reading.";
									KMsgBox::message(0,"KVirc auto-setup",szMsg.data());
								}			
							} else {
								QString szMsg="unable to open file ";
								szMsg+=szDirectory.data();
								szMsg+=" for writing.";
								KMsgBox::message(0,"KVirc auto-setup",szMsg.data());
							}
						}
					}		
				}
			}
		}
	}
	return szFilesCopied;
	_debug_leavetrace("copyAllFrom");
}


//============ createHomeDirIfNotExists ============//

QString KviSetup::createHomeDirIfNotExists(const char *subdir)
{
	_debug_entertrace("createHomeDirIfNotExists");
	QString szHome=getKVIrcHomeDirectory(subdir);
	QDir theDir(szHome.data());
	if(!theDir.exists()){
		if(!theDir.mkdir(szHome.data())){
			QString szMsg="Unable to create ";
			szMsg+=szHome;
			szMsg+=" directory.\n";
			KMsgBox::message(0,"KVirc auto-setup",szMsg.data());
		}
	}
	szHome+='\n';
	return szHome;
	_debug_leavetrace("createHomeDirIfNotExists");
}


//============ getGlobalDataDirectory ============//

QString KviSetup::getGlobalDataDirectory(const char *subdir)
{
	_debug_entertrace("getGlobalDataDirectory");
	QString szDir=KApplication::kde_datadir().data();
	if(szDir.isEmpty()){
		KMsgBox::message(0,"WARNING","Your KDE einviroinment is not configured correctly.\n"\
													"Unable to get the $KDEDIR/share/apps global directory.");
		szDir=KApplication::localkdedir();
		if(szDir[szDir.length()-1]!='/')szDir+='/';
		szDir+="share/apps";
	}
	if(szDir[szDir.length()-1]!='/')szDir+='/';
	szDir+="kvirc";
	if(subdir){
		szDir+='/';
		szDir+=subdir;
	}
	return szDir;
	_debug_leavetrace("getGlobalDataDirectory");
}

//============ directoryExists ============//

bool KviSetup::directoryExists(const char *directory)
{
	_debug_entertrace("directoryExists");
	QDir theDir(directory);
	return theDir.exists();
	_debug_leavetrace("directoryExists");
}


