//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CLASS_NAME_ "KviServerParser"

#include "kvi_app.h"
#include "kvi_numeric.h"
#include "kvi_token.h"
#include "kvi_ctcp.h"

handleCmdTable KviServerParser::cmdTab[]={
	{ "PRIVMSG" , &KviServerParser::handlePrivmsg },
	{ "PING"    , &KviServerParser::handlePing    },
	{ "JOIN"    , &KviServerParser::handleJoin    },
	{ "PART"    , &KviServerParser::handlePart    },
	{ "MODE"    , &KviServerParser::handleMode    },
	{ "KICK"    , &KviServerParser::handleKick    },
	{ "TOPIC"   , &KviServerParser::handleTopic   },
	{ "NICK"    , &KviServerParser::handleNick    },
	{ "QUIT"    , &KviServerParser::handleQuit    },
	{ "NOTICE"  , &KviServerParser::handleNotice  },
	{ "ERROR"   , &KviServerParser::handleError   },
	{ 0         , 0                              }
	//Manca ERROR,KILL; ???
};
//============ KviServerParser ============//
KviServerParser::KviServerParser(KviFrame *lpFrame)
{
	_debug_entertrace("KviServerParser");
	m_lpFrm   = lpFrame;
	m_lpCons  = lpFrame->m_lpConsole;
	m_lpMdi   = lpFrame->m_lpMdi;
	m_lpUsr   = lpFrame->m_lpUsr;
	m_lpOpt   = lpFrame->m_lpOpt;
	m_bActive = false;
	_debug_leavetrace("KviServerParser");
}
//============ ~KviServerParser ============//
KviServerParser::~KviServerParser()
{
	_debug_entertrace("~KviServerParser");
	if(!m_bActive){ //Just a build & design time control
		debug("Activate was NOT called...");
		ASSERT(false);
	}
	delete m_lpCTCP;
	_debug_leavetrace("~KviServerParser");
}
//============ setStatus ============//
void KviServerParser::setStatus(const char *szFmt,const char *szData)
{
	_debug_entertrace("setStatus");
	QString szStat;
	szStat.sprintf(szFmt,szData);
	m_lpFrm->setStatusText(szStat.data());
	_debug_leavetrace("setStatus");
}
//============ activate ============//
void KviServerParser::activate(KviIrcSocket *lpSock)
{
	_debug_entertrace("activate");
	m_lpSock  = lpSock;
	m_lpEventManager=m_lpFrm->m_lpEventManager;
	m_lpUserParser=m_lpFrm->m_lpUserParser;
	m_lpCTCP=new KviCTCP(m_lpFrm,this);
	m_bActive = true;
	_debug_leavetrace("activate");
}
//============ parseMessage ============//
void KviServerParser::parseMessage(QString &szMessage)
{
	_debug_entertrace("parseMessage");
	//Split the string in three parts...
	register const char *pC=szMessage.data();
	int len=szMessage.length();
	QString szPrefix(len); //It should be enough
	QString szCommand(len); //I Know that this is too long but... we want to be sure....
	QString szParams(len);
	char *pPnt=szPrefix.data();
	const char *pStart=0;
	if(!(pC))return;
	while(((*pC)==' '))pC++; //skip the spaces... (is it really necessary?)
	if((*pC == '\0')) return;
	//Ok we point to a non space...
	if((*pC)==':'){
		pC++;
		//Point to the first letter (or null)
		pStart=pC;
		while((*pC) && (*pC != ' '))*pPnt++ = *pC++; //copy the first word (prefix);
		if(!(*pC))return; //Fucked message...(only a senseless prefix)...
		//debug("prefix copied");
		//We point to a space...
		//:prefix command
		// |     |
		//pStart pC
		++(*pPnt)='\0';
		szPrefix.resize((int)((pC-pStart)+1));
		while(((*pC)==' '))pC++; //skip the spaces...
		if((*pC == '\0')) return; //Fucked message.
	}
	//Ok we point to the first letter of the command...
	pStart=pC;
	pPnt=szCommand.data();
	while((*pC) && (*pC != ' '))*pPnt++ = *pC++; //copy the second(?) word (command);
	//We point to a something after the command...
	//:prefix command???????????????.....
	//        |      |
	//       pStart  pC
	++(*pPnt)='\0';
	szCommand.resize((int)((pC-pStart)+1));
	while(((*pC)==' '))pC++; //skip the spaces...
	if((*pC)){
		//debug("params too...");
		//we have params too;
		pStart=pC;
		pPnt=szParams.data();
		while((*pC))*pPnt++ = *pC++; //copy the rest...
		++(*pPnt)='\0';
		szParams.resize((int) ((pC-pStart)+1));
	} else {
		//no params
		szParams="";
	}
	///////////////////////// Splitting DONE
	//Check if it is a numeric msg
	bool bOk=false;
	int nCommand=szCommand.toInt(&bOk);
	if(bOk){
		if((nCommand>399) && (nCommand<600))parseNumericError(szPrefix,nCommand,szParams);
		else parseNumericReply(szPrefix,nCommand,szParams);
	} else parseLiteral(szPrefix,szCommand,szParams);
	//m_lpCons->doFmtOutput(KVI_OUT_SERVER,"%s from [%s] : %s",szCommand.data(),szPrefix.data(),szParams.data());
	_debug_leavetrace("parseMessage");
}
//============ topWin ============//
KviMdiChild * KviServerParser::topWin()
{
	_debug_entertrace("topWin");
	KviMdiChild *lpC=m_lpMdi->topChildInZOrder();
	return ( lpC ? lpC : m_lpCons );
	_debug_leavetrace("topWin");
}
//============ findWin ============//
KviMdiChild * KviServerParser::findWin(const char *szName)
{
	_debug_entertrace("findWin");
	KviMdiChild *lpC=m_lpMdi->findWindow(szName);
	return ( lpC ? lpC : m_lpCons );
	_debug_leavetrace("findWin");
}
//============ getChanWnd ============//
KviChanWnd * KviServerParser::getChanWnd(const char *szName)
{
	_debug_entertrace("getChanWnd");
	KviMdiChild *lpC=m_lpMdi->findWindow(szName);
	if(lpC){
		if(lpC->type()==KVI_WND_TYPE_CHAN)return ((KviChanWnd *)lpC);
		else return 0;
	} else return 0;
	_debug_leavetrace("getChanWnd");
}
//============ getQueryWnd ============//
KviQueryWnd * KviServerParser::getQueryWnd(const char *szName)
{
	_debug_entertrace("getQueryWnd");
	KviMdiChild *lpC=m_lpMdi->findWindow(szName);
	if(lpC){
		if(lpC->type()==KVI_WND_TYPE_QUERY)return ((KviQueryWnd *)lpC);
		else return 0;
	} else return 0;
	_debug_leavetrace("getQueryWnd");
}
//============ isMe ============//
bool KviServerParser::isMe(QString &szNick)
{
	_debug_entertrace("isMe");
	const char *pD=szNick.data();
	if(pD){
		if(((*pD)=='@')||((*pD)=='+')){
			pD++;
			if(!(*pD))return false;
		}
		return (!strcasecmp(pD,m_lpFrm->m_lpGlb->szNick.data()));
	}
	return false;
	_debug_leavetrace("isMe");
}
//============ skipWord ============//
const char * KviServerParser::skipWord(QString &szString)
{
	_debug_entertrace("skipWord");
	char *cr=szString.data();
	while((*cr) && (*cr != ' '))cr++;
	if(*cr){
		while((*cr) && (*cr==' '))cr++;
	}
	return cr;
	_debug_leavetrace("skipWord");
}
//============ skipUpToTrailing ===========//
const char * KviServerParser::skipUpToTrailing(QString &szString)
{
	_debug_entertrace("skipUpToTrailing");
	char *cr=szString.data();
	while((*cr) && ((*cr)!= ':'))cr++;
	if(*cr){
		while((*cr) && ((*cr)==':'))cr++;
	}
	return cr;
	_debug_leavetrace("skipUpToTrailing");
}
//============ parseNumericError ============//
void KviServerParser::parseNumericError(QString &,int nCommand,QString &szParams)
{
	_debug_entertrace("parseNumericError");
	switch(nCommand){
		case ERR_NONICKNAMEGIVEN:
		case ERR_ERRONEUSNICKNAME:
		case ERR_NICKNAMEINUSE:
		case ERR_NICKCOLLISION:
			handleNickError(nCommand,szParams);
			break;
		default:
			if(m_lpEventManager->lpEvent[KVI_Event_OnUnhandledServerError]->bEnabled){
				QString szPara;
				szPara.setNum(nCommand);
				szPara+=" ";
				szPara+=szParams.data();
				if(m_lpUserParser->executeEvent(topWin(),
						m_lpEventManager->lpEvent[KVI_Event_OnUnhandledServerError],
						szPara)!=KVI_EventErr_Halt)
					topWin()->doFmtOutput(KVI_OUT_RPLERR,"[SERVER ERROR %d] %s",nCommand,szParams.data());
			} else topWin()->doFmtOutput(KVI_OUT_RPLERR,"[SERVER REPLY %d] %s",nCommand,szParams.data());
			break;
	}
//#define ERR_NOSUCHNICK	     401
//#define ERR_NOSUCHSERVER     402
//#define ERR_NOSUCHCHANNEL    403
//#define ERR_CANNOTSENDTOCHAN 404
//#define ERR_TOOMANYCHANNELS  405
//#define ERR_WASNOSUCHNICK    406
//#define ERR_TOOMANYTARGETS   407
//#define ERR_NOORIGIN	     409

//#define ERR_NORECIPIENT	     411
//#define ERR_NOTEXTTOSEND     412
//#define ERR_NOTOPLEVEL	     413
//#define ERR_WILDTOPLEVEL     414

//#define ERR_QUERYTOOLONG     416	/* Undernet extension */

//#define ERR_UNKNOWNCOMMAND   421
//#define ERR_NOMOTD	     422
//#define ERR_NOADMININFO	     423
/*      ERR_FILEERROR	     424           removed from RFC1459 */

//#define ERR_BANNICKCHANGE    437	/* Undernet extension */
//#define ERR_NICKTOOFAST	     438	/* Undernet extension */
//#define ERR_TARGETTOOFAST    439	/* Undernet extension */

//#define ERR_USERNOTINCHANNEL 441
//#define ERR_NOTONCHANNEL     442
//#define ERR_USERONCHANNEL    443
/*      ERR_NOLOGIN  	     444           removed from RFC1459 */
/*      ERR_SUMMONDISABLED   445           removed from RFC1459 */
/*      ERR_USERSDISABLED    446	   removed from RFC1459 */

//#define ERR_NOTREGISTERED    451

//#define ERR_NEEDMOREPARAMS   461
//#define ERR_ALREADYREGISTRED 462
//#define ERR_NOPERMFORHOST    463
//#define ERR_PASSWDMISMATCH   464
//#define ERR_YOUREBANNEDCREEP 465
//#define ERR_YOUWILLBEBANNED  466
//#define ERR_KEYSET	     467	/* Undernet extension */
//#define ERR_INVALIDUSERNAME  468	/* Undernet extension */

//#define ERR_CHANNELISFULL    471
//#define ERR_UNKNOWNMODE	     472
//#define ERR_INVITEONLYCHAN   473
//#define ERR_BANNEDFROMCHAN   474
//#define ERR_BADCHANNELKEY    475
//#define ERR_BADCHANMASK	     476	/* Undernet extension */
//#define ERR_MODELESS         477	/* Extension to RFC1459 */
//#define ERR_BANLISTFULL	     478	/* Undernet extension */

//#define ERR_NOPRIVILEGES     481
//#define ERR_CHANOPRIVSNEEDED 482
//#define ERR_CANTKILLSERVER   483
//#define ERR_ISCHANSERVICE    484	/* Undernet extension */

//#define ERR_NOOPERHOST	     491

//#define ERR_UMODEUNKNOWNFLAG 501
//#define ERR_USERSDONTMATCH   502

//#define ERR_SILELISTFULL     511	/* Undernet extension */

//#define ERR_NOSUCHGLINE	     512	/* Undernet extension */
//#define ERR_BADPING	     513	/* Undernet extension */
	_debug_leavetrace("parseNumericError");
}
//============ handleNickError ============//
void KviServerParser::handleNickError(int nCommand,QString &szParams)
{
	_debug_entertrace("handleNickError");
	if(m_lpFrm->m_bLoggedIn){	//Notify the error
		switch(nCommand){
			case ERR_NONICKNAMEGIVEN:
				topWin()->doFmtOutput(KVI_OUT_RPLERR,"[SERVER ERROR %d] No nickname given.",nCommand);
				break;
			default:
				topWin()->doFmtOutput(KVI_OUT_RPLERR,"[SERVER ERROR %d] %s",nCommand,skipWord(szParams));
				break;
		}
	} else {					//Change the nick
		if((!(m_lpFrm->m_bUsingAltNick))&&(!(m_lpUsr->szAltNick.isEmpty()))){
			m_lpSock->sendFmtData("NICK %s",m_lpUsr->szAltNick.data());
			m_lpFrm->m_lpGlb->szNick=m_lpUsr->szAltNick.data();
			topWin()->doFmtOutput(KVI_OUT_INTERNAL,"Login nickname refused by server, trying alternate nick %s",
					m_lpUsr->szAltNick.data());
			m_lpFrm->m_bUsingAltNick=true;
		} else {
			QString szRandom=m_lpUsr->szNick.copy();
			if(szRandom[0]>'x' || szRandom[0]<'a')szRandom[0]='a';
			else szRandom[0]=(szRandom[0]+1);
			m_lpSock->sendFmtData("NICK %s",szRandom.data());
			m_lpFrm->m_lpGlb->szNick=szRandom.data();
			topWin()->doFmtOutput(KVI_OUT_INTERNAL,"Login nickname refused by server, trying random alternate nick %s",
					szRandom.data());
		}
	}
	_debug_leavetrace("handleNickError");
}
//============ parseNumericReply ============//
void KviServerParser::parseNumericReply(QString &szPrefix,int nCommand,QString &szParams)
{
	_debug_entertrace("parseNumericReply");
	switch(nCommand){
		case RPL_MOTDSTART:
		case RPL_MOTD:
		case RPL_MOTD2:
		case RPL_ENDOFMOTD:
			m_lpFrm->m_bLoggedIn=true;
			m_lpCons->doOutput(KVI_OUT_MOTD,skipUpToTrailing(szParams));
			break;
		case RPL_NAMREPLY:
		case RPL_ENDOFNAMES:
			handleNameReply(nCommand,szParams);
			break;
		case RPL_NOTOPIC:
		case RPL_TOPIC:
		case RPL_TOPICWHOTIME:
			handleTopicReply(nCommand,szParams);
			break;
		case RPL_CHANNELMODEIS:
			handleModeReply(szPrefix,szParams);
			break;
		case RPL_USERHOST:
			handleUserHost(szParams);
			break;
		case RPL_UNAWAY: // added by RAD Kade 1  
			topWin()->doOutput(KVI_OUT_BACK,skipUpToTrailing(szParams)); 
			setStatus("You are now %s","back."); 
			break; 
		case RPL_NOWAWAY: // added by RAD Kade 1 
			topWin()->doOutput(KVI_OUT_AWAY,skipUpToTrailing(szParams)); 
			setStatus("You are now %s","away."); 
			break; 
		case RPL_WHOISUSER:
		case RPL_WHOISSERVER:
		case RPL_WHOISOPERATOR:
		case RPL_WHOISIDLE:
		case RPL_ENDOFWHOIS:
		case RPL_WHOISCHANNELS:
		case RPL_ENDOFWHOWAS:
		case RPL_WHOWASUSER:
		case RPL_AWAY:
			handleWhois(szPrefix,nCommand,szParams);
			break;
		case RPL_YOURHOST:
		case RPL_CREATED:
		case RPL_MYINFO:
		case RPL_MAP:			//I dunno...I have never seen the following ones...
		case RPL_MAPMORE:		//
		case RPL_MAPEND:		//
		case RPL_SNOMASK:		//
		case RPL_STATMEMTOT:	//
		case RPL_STATMEM:		//'Till here...
		case RPL_INFOSTART:
		case RPL_INFO:
		case RPL_ENDOFINFO:
		case RPL_YOUREOPER:		//This one is here just because it uses the same syntax...
		case RPL_LUSERCLIENT:
		case RPL_LUSERME:
		case RPL_LUSERLOCAL:
		case RPL_LUSERGLOBAL:
			m_lpFrm->m_bLoggedIn=true;
			m_lpCons->doOutput(KVI_OUT_INFO,skipUpToTrailing(szParams));
			break;
		case RPL_LUSEROP:
		case RPL_LUSERUNKNOWN:
		case RPL_LUSERCHANNELS:
			m_lpFrm->m_bLoggedIn=true;
			m_lpCons->doOutput(KVI_OUT_INFO,skipWord(szParams));
			break;
		case RPL_WELCOME:
			m_lpFrm->m_bLoggedIn=true;
			{
				//extract the last token from the Welcome message.
				//it should be our mask
				szParams=szParams.stripWhiteSpace();
				const char *theP=strrchr(szParams.data(),' ');
				m_lpFrm->m_lpGlb->szMaskFromServer=QString(theP);
				int idx=m_lpFrm->m_lpGlb->szMaskFromServer.find('!');
				if(idx==-1)m_lpCons->doOutput(KVI_OUT_ERROR,"The server missed to send us our mask."\
									"\nThe $_mymask and $_myaddress identifers may contain incorrect values."\
									"\nThis will be fixed when you join the first channel.");
 				else {
					KviNick nk(m_lpFrm->m_lpGlb->szMaskFromServer);
					if(nk.szNick != m_lpFrm->m_lpGlb->szNick){
						m_lpCons->doFmtOutput(KVI_OUT_INTERNAL,"The server changed our nick to %s",nk.szNick.data());
						m_lpFrm->m_lpGlb->szNick=nk.szNick.copy();
					}
					m_lpFrm->m_lpGlb->szHostName=nk.szAddress.copy();
				}
			}
			m_lpCons->doOutput(KVI_OUT_INFO,skipUpToTrailing(szParams));
			if(m_lpEventManager->lpEvent[KVI_Event_OnLoggedIn]->bEnabled){
				QString szPara=m_lpFrm->m_lpGlb->szNick+" "+m_lpFrm->m_lpGlb->szUserName+" "+m_lpFrm->m_lpGlb->szRealName;
				m_lpUserParser->executeEvent(m_lpCons,m_lpEventManager->lpEvent[KVI_Event_OnLoggedIn],szPara);
			}
			//set the user modes
			{
				QString szMode="";
				if(m_lpUsr->bool_wallops)szMode+='w';
				if(m_lpUsr->bool_servernotices)szMode+='s';
				if(m_lpUsr->bool_invisible)szMode+='i';
				if(!szMode.isEmpty())m_lpSock->sendFmtData("MODE %s %s",m_lpFrm->m_lpGlb->szNick.data(),szMode.data());
			}
			m_lpFrm->startNotifyList();
			break;
		case RPL_BANLIST:
		case RPL_ENDOFBANLIST:
			handleBanList(nCommand,szParams);
			break;
		case RPL_ISON:
			handleIsOn(szParams);
			break;
		case RPL_CREATIONTIME:
			handleCreationTime(szParams);
			break;
		case RPL_LINKS:
		case RPL_ENDOFLINKS:
			handleLinks(nCommand,szParams);
			break;
		default:
			if(m_lpEventManager->lpEvent[KVI_Event_OnUnhandledServerReply]->bEnabled){
				QString szPara;
				szPara.setNum(nCommand);
				szPara+=" ";
				szPara+=szParams.data();
				if(m_lpUserParser->executeEvent(topWin(),
						m_lpEventManager->lpEvent[KVI_Event_OnUnhandledServerReply],
						szPara)!=KVI_EventErr_Halt)
					topWin()->doFmtOutput(KVI_OUT_SERVER,"[SERVER REPLY %d] %s",nCommand,szParams.data());
			} else topWin()->doFmtOutput(KVI_OUT_SERVER,"[SERVER REPLY %d] %s",nCommand,szParams.data());
			break;
	}
//#define RPL_NONE	     300
//#define RPL_TEXT	     304
//#define RPL_USERIP	     307	/* Undernet extension */
//#define RPL_LISTSTART	     321
//#define RPL_LIST	     322
//#define RPL_LISTEND	     323
//#define RPL_LISTUSAGE	     334	/* Undernet extension */
//#define RPL_INVITING	     341
///*      RPL_SUMMONING        342	   removed from RFC1459 */
//#define RPL_VERSION	     351
//
//#define RPL_WHOREPLY	     352	/* See also RPL_ENDOFWHO */
//#define RPL_WHOSPCRPL        354	/* Undernet extension,
//#define RPL_KILLDONE	     361
//#define RPL_CLOSING	     362
//#define RPL_CLOSEEND	     363

//#define RPL_ENDOFWHOWAS	     369
//#define RPL_REHASHING	     382
//#define RPL_MYPORTIS	     384
//#define RPL_NOTOPERANYMORE   385	/* Extension to RFC1459 */
//#define RPL_TIME	     391
//#define RPL_TRACELINK	     200
//#define RPL_TRACECONNECTING  201
//#define RPL_TRACEHANDSHAKE   202
//#define RPL_TRACEUNKNOWN     203
//#define RPL_TRACEOPERATOR    204
//#define RPL_TRACEUSER	     205
//#define RPL_TRACESERVER	     206
//#define RPL_TRACENEWTYPE     208
//#define RPL_TRACECLASS	     209
//#define RPL_STATSLINKINFO    211
//#define RPL_STATSCOMMANDS    212
//#define RPL_STATSCLINE	     213
//#define RPL_STATSNLINE	     214
//#define RPL_STATSILINE	     215
//#define RPL_STATSKLINE	     216
//#define RPL_STATSPLINE	     217	/* Undernet extenstion */
//#define RPL_STATSYLINE	     218
//#define RPL_ENDOFSTATS	     219	/* See also RPL_STATSDLINE */
//#define RPL_UMODEIS	     221
//#define RPL_SERVICEINFO	     231
//#define RPL_ENDOFSERVICES    232
//#define RPL_SERVICE	     233
//#define RPL_SERVLIST	     234
//#define RPL_SERVLISTEND	     235
//#define RPL_STATSLLINE	     241
//#define RPL_STATSUPTIME	     242
//#define RPL_STATSOLINE	     243
//#define RPL_STATSHLINE	     244
///*      RPL_STATSSLINE	     245	   Reserved */
//#define RPL_STATSTLINE	     246	/* Undernet extension */
//#define RPL_STATSGLINE	     247	/* Undernet extension */
//#define RPL_STATSULINE	     248	/* Undernet extension */
//#define RPL_STATSDEBUG	     249	/* Extension to RFC1459 */
//#define RPL_STATSCONN	     250	/* Undernet extension */
//#define RPL_ADMINME	     256
//#define RPL_ADMINLOC1	     257
//#define RPL_ADMINLOC2	     258
//#define RPL_ADMINEMAIL	     259
//#define RPL_TRACELOG	     261
//#define RPL_TRACEPING	     262	/* Extension to RFC1459 */
//#define RPL_SILELIST	     271	/* Undernet extension */
//#define RPL_ENDOFSILELIST    272	/* Undernet extension */
//#define RPL_STATSDLINE	     275	/* Undernet extension */
//#define RPL_GLIST	     280	/* Undernet extension */
//#define RPL_ENDOFGLIST	     281	/* Undernet extension */
	_debug_leavetrace("parseNumericReply");
}
//============ handleLinks ============//
void KviServerParser::handleLinks(int nCommand,QString &szParams)
{
	_debug_entertrace("handleLinks");
	KviIrcToken par(szParams);
	QString myNick=par.nextWord();
	bool doit=false;
	if(m_lpFrm->m_lpLinksDialog){
		if(!m_lpFrm->m_lpLinksDialog->m_bDone)doit=true;
	}
	if(doit){
		if(nCommand==RPL_LINKS){
			QString szHost=par.nextWord();
			QString szParenthost=par.nextWord();
			par.stripUpToTrailing();
			m_lpFrm->m_lpLinksDialog->insertLink(szHost.data(),szParenthost.data(),par.szString.data());
		} else m_lpFrm->m_lpLinksDialog->endOfLinks();
	} else {
		if(nCommand==RPL_LINKS){
			QString szHost=par.nextWord();
			QString szParenthost=par.nextWord();
			par.stripUpToTrailing();
			m_lpCons->doFmtOutput(KVI_OUT_SERVER,
				"[LINKS REPLY] : host : %s linked to %s [ %s ]",
				szHost.data(),szParenthost.data(),par.szString.data()); 	
		}
	}
	_debug_leavetrace("handleLinks");
}
//============ handleIsOn ============//
void KviServerParser::handleIsOn(QString &szParams)
{
	_debug_entertrace("handleIsOn");
	KviIrcToken par(szParams);
	par.stripUpToTrailing();
	QStrIList lst(true);
	lst.setAutoDelete(true);
	while(!par.szString.isEmpty()){
		QString szNewNick=par.nextWord();
		lst.append(szNewNick.data());
	}
	const char *szNk=0;
	int curIdx=0;
	for(szNk=m_lpOpt->lpNotifyList->first();szNk;szNk=m_lpOpt->lpNotifyList->next()){
		int idx=lst.find(szNk);
		if(idx != -1){
			lst.remove(idx);
			int *lpInt=m_lpFrm->m_lpIsOnList->at(curIdx);
			if(lpInt){
				if(!(*lpInt)){
					if(m_lpOpt->bNotifyListInActive)topWin()->doFmtOutput(KVI_OUT_WHO,"%s is on irc",szNk);
					else m_lpCons->doFmtOutput(KVI_OUT_WHO,"%s is on irc",szNk);
					(*lpInt)=1;
					m_lpCons->m_lpNotifyList->nickJoin(szNk);
				}
			}
		} else {
			int *lpInt=m_lpFrm->m_lpIsOnList->at(curIdx);
			if(lpInt){
				if((*lpInt)){
					if(m_lpOpt->bNotifyListInActive)topWin()->doFmtOutput(KVI_OUT_WHO,"%s has left irc",szNk);
					else m_lpCons->doFmtOutput(KVI_OUT_WHO,"%s has left irc",szNk);
					m_lpCons->m_lpNotifyList->nickPart(szNk);
					(*lpInt)=0;
				}
			}
		}
		curIdx++;
	}
	while(!lst.isEmpty())lst.removeLast();
	_debug_leavetrace("handleIsOn");
}
//============ handleCreationTime ============//
void KviServerParser::handleCreationTime(QString &szParams)
{
	_debug_entertrace("handleCreationTime");
	KviIrcToken par(szParams);
	QString channel=par.nextChannel();
	QString szTime=par.szString.stripWhiteSpace();
	KviChanWnd *lpWnd=getChanWnd(channel.data());
	if(lpWnd){
		bool bOK;
		uint crTime=szTime.toUInt(&bOK);
		if(bOK){
			QDateTime dt;
			dt.setTime_t(crTime);
			szTime=dt.toString();
			lpWnd->doFmtOutput(KVI_OUT_TIME,"Channel was created on %s",szTime.data());
		}
	}
	_debug_leavetrace("handleCreationTime");
}
//============ handleTopicReply ============//
void KviServerParser::handleTopicReply(int nCommand,QString &szParams)
{
	_debug_entertrace("handleTopicReply");
	KviIrcToken par(szParams);
	QString channel=par.nextChannel();
	par.stripUpToTrailing(); //contains the topic
	KviChanWnd *lpWnd=getChanWnd(channel.data());
	if(!lpWnd){
		topWin()->doFmtOutput(KVI_OUT_TOPIC,"[TOPIC REPLY %d] : %s : %s",nCommand,channel.data(),par.szString.data());
	} else {
		if(nCommand==RPL_NOTOPIC){
			lpWnd->setTopic("[ No topic is set ]");
			return;
		}
		if(nCommand==RPL_TOPIC)lpWnd->setTopic(par.szString.data());
		else {
			QString topSetBy=par.nextWord();
			QString szTopTime=par.szString.stripWhiteSpace();
			bool bOK;
			uint topicTime=szTopTime.toUInt(&bOK);
			if(bOK){
				QDateTime dt;
				dt.setTime_t(topicTime);
				szTopTime=dt.toString();
				lpWnd->doFmtOutput(KVI_OUT_TOPIC,"Topic was set by %s on %s",topSetBy.data(),szTopTime.data());
			} else lpWnd->doFmtOutput(KVI_OUT_TOPIC,"Topic was set by %s",topSetBy.data());
		}
	}
	_debug_leavetrace("handleTopicReply");
}
//============ handleModeReply ============//
void KviServerParser::handleModeReply(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handleModeReply");
	KviIrcToken par(szParams);
	par.nextWord();
	handleMode(szPrefix,par.szString);
	_debug_leavetrace("handleModeReply");
}
//============ handleUserHost ============//
void KviServerParser::handleUserHost(QString &szParams)
{
	_debug_entertrace("handleUserHost");
	KviIrcToken par(szParams);
	par.stripUpToTrailing();
	QString szUH;
	QString szNK;
	QString szUN=KVI_STR_NULL;
	while(!par.szString.isEmpty()){
		szUH=par.nextWord();
		if(!szUH.isEmpty()){
			int idx=szUH.find('=');
			if(idx > 0){
				szNK=szUH.left(idx);
				szUH.remove(0,idx+1);
				if(szNK[idx-1]=='*')szNK.remove(idx-1,1);
				if(!szUH.isEmpty()){
					if((szUH[0]=='+')||(szUH[0]=='-'))szUH.remove(0,1);
					idx=szUH.find('@');
					if(idx != -1){
						szUN=szUH.left(idx);
						szUH.remove(0,idx+1);
					}
					KviNick nk;
					nk.szNick=szNK.copy();
					nk.szAddress=szUH.copy();
					nk.szUserName=szUN.copy();
					KviMdiChild *lpC;
					for(lpC=m_lpMdi->m_lpChildList->first();lpC;lpC=m_lpMdi->m_lpChildList->next()){
						if(lpC->type()==KVI_WND_TYPE_CHAN){
							KviChanWnd *lpWnd=(KviChanWnd *)lpC;
							lpWnd->updateUser(nk); //if is there it will be updated...
						}
					}
				}		
			}
		}
	}
	_debug_leavetrace("handleUserHost");
}
//============ handleBanList ============//
void KviServerParser::handleBanList(int nCommand,QString &szParams)
{
	_debug_entertrace("handleBanList");
	KviIrcToken par(szParams);
	QString channel=par.nextChannel();
	KviChanWnd *lpChan=getChanWnd(channel.data());
	if(!lpChan){
		topWin()->doFmtOutput(KVI_OUT_SERVER,"[BANLIST REPLY %d] : %s : %s",nCommand,channel.data(),szParams.data());
//		m_lpCons->doFmtOutput(KVI_OUT_ERROR,"[INTERNAL ERROR (BANLIST REPLY PARSING)] : Can't find channel %s",channel.data());
		return;
	}
	if(nCommand==RPL_BANLIST){
		// Fixed by RAD Kade 1
		if(!lpChan->m_bHasBanList)lpChan->changeBanList(par.nextWord(),par.nextWord(),true); 	
		//if(!lpChan->m_bHasBanList)lpChan->changeBanList(par.szString.data(),0,true);
		else lpChan->doFmtOutput(KVI_OUT_SERVER,"[BANLIST REPLY %d] : %s : %s",nCommand,channel.data(),szParams.data());		
	} else lpChan->m_bHasBanList=true;
	_debug_leavetrace("handleBanList");
}
//============ handleNameReply ============//
void KviServerParser::handleNameReply(int nCommand,QString &szParams)
{
	_debug_entertrace("handleNameReply");
	KviIrcToken par(szParams);
	QString channel=par.nextChannel();

	KviChanWnd *lpChan=getChanWnd(channel.data());
	if(!lpChan){
		topWin()->doFmtOutput(KVI_OUT_SERVER,"[NAMES REPLY %d] : %s : %s",nCommand,channel.data(),szParams.data());
	//	m_lpCons->doFmtOutput(KVI_OUT_ERROR,"[INTERNAL ERROR (NAME REPLY PARSING)] : Can't find channel %s",channel.data());
		return;
	}
	if(nCommand==RPL_ENDOFNAMES){
		if(lpChan->m_bHasAllNames)lpChan->doFmtOutput(KVI_OUT_SERVER,"[NAMES_REPLY %d] : %s : End of names list",nCommand,channel.data());
		else {
			lpChan->m_bHasAllNames=true;
			if(m_lpEventManager->lpEvent[KVI_Event_OnMeJoin]->bEnabled){
				QString szPara=channel.data();
				m_lpUserParser->executeEvent(lpChan,m_lpEventManager->lpEvent[KVI_Event_OnMeJoin],szPara);
			}
			if(lpChan->m_iTotal==1){ 	//useful on efnet (see irc.nijenrode.nl)
										//that don't send our nick in names reply
										//so we need to op ourself
				if(!lpChan->m_bIsMeOp){
					lpChan->nickOp(m_lpFrm->m_lpGlb->szNick.data(),true);
					lpChan->m_bIsMeOp=true;
					lpChan->setLabelsPalette();
				}
			}
			if(m_lpOpt->bEnableUserListUpdate){
				lpChan->startUpdateUserList();
				lpChan->doOutput(KVI_OUT_INTERNAL,"Starting compilation of the internal users list");
			}
		}
	} else {
		par.stripUpToTrailing();
		if(lpChan->m_bHasAllNames)lpChan->doFmtOutput(KVI_OUT_SERVER,"[NAMES REPLY %d] : %s : %s",nCommand,channel.data(),szParams.data());
		else {
			QString szNick;
			lpChan->m_bAutoUpdateUsers=false;
			lpChan->m_lpListBox->setAutoUpdate(false);
			while(!par.szString.isEmpty()){
				szNick=par.nextWord();
				if(!szNick.isEmpty()){
					if(!isMe(szNick))lpChan->nickJoin(szNick);
//					else {
//						if(szNick[0]=='@'){
//							szNick.remove(0,1);
//							lpChan->nickOp(szNick,true); //Useful on EFNet where the server (see irc.nienrode.nl)
//														 //does not send the MODE +o message in empty channels
//						}
//					}
				}
			}
			lpChan->m_bAutoUpdateUsers=true;
			lpChan->m_lpListBox->setAutoUpdate(true);
			lpChan->m_lpListBox->repaint();
			lpChan->updateUsersLabel();
			//Now we have a list of names here...
		}	
	}
	_debug_leavetrace("handleNameReply");
}
//============ parseLiteral ============//
void KviServerParser::parseLiteral(QString &szPrefix,QString &szCommand,QString &szParams)
{
	_debug_entertrace("parseLiteral");
	QString szCmd(szCommand.upper());
	if(szCommand.isEmpty())return;
	for (int i=0;cmdTab[i].szName!=0;i++){
		if (!strcmp(szCommand.data(),cmdTab[i].szName)){
	 	   (this->*(cmdTab[i].proc))(szPrefix,szParams);
			return;
		}
	}
	topWin()->doFmtOutput(KVI_OUT_SERVER,"[%s][%s] %s",szPrefix.data(),szCommand.data(),szParams.data()); 
	_debug_leavetrace("parseLiteral");
}
//============ handlePing ============//
void KviServerParser::handlePing(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handlePing");
	if(szPrefix.isEmpty())szPrefix=m_lpFrm->m_lpGlb->szServerName;
	m_lpCons->doFmtOutput(KVI_OUT_PING,"Ping request from %s [PING %s] ... PONG!",szPrefix.data(),szParams.data());
	m_lpSock->sendFmtData("PONG %s",szParams.data());
	setStatus("Received PING from %s",szPrefix.data());
	_debug_leavetrace("handlePing");
}
//============ handleError ============//
void KviServerParser::handleError(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handleError");
	if(szPrefix.isEmpty())szPrefix=m_lpFrm->m_lpGlb->szServerName;
	m_lpCons->doFmtOutput(KVI_OUT_RPLERR,"[ERROR from %s] : %s",szPrefix.data(),szParams.data());
	setStatus("Received ERROR from %s",szPrefix.data());
	_debug_leavetrace("handleError");
}
//============ handleJoin ============//
void KviServerParser::handleJoin(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handleJoin");
	KviNick joiner(szPrefix);
	QString channel(skipUpToTrailing(szParams));
	QString szMask;
	joiner.mask(szMask,1);
	if(isMe(joiner.szNick)){
		joiner.mask(m_lpFrm->m_lpGlb->szMaskFromServer,0);
		m_lpFrm->m_lpGlb->szHostName=joiner.szAddress.copy();
		KviChanWnd * lpChan=m_lpFrm->createChanWnd(channel.data());	
		lpChan->nickJoin(joiner);
		m_lpSock->sendFmtData("MODE %s",channel.data());
		lpChan->doFmtOutput(KVI_OUT_JOIN,"You have joined %s [you're known as %s!%s]",channel.data(),
					joiner.szNick.data(),szMask.data());
		setStatus("Joined %s",channel.data());
	} else {
		KviChanWnd *lpChan=getChanWnd(channel.data());
		if(lpChan){
			lpChan->nickJoin(joiner);
			if(m_lpEventManager->lpEvent[KVI_Event_OnJoin]->bEnabled){
				QString szPara=channel+" "+joiner.szNick+" "+joiner.szUserName+" "+joiner.szAddress;
				m_lpUserParser->executeEvent(lpChan,m_lpEventManager->lpEvent[KVI_Event_OnJoin],szPara);
			}
			lpChan->doFmtOutput(KVI_OUT_JOIN,"%s [%s] has joined %s",joiner.szNick.data(),szMask.data(),channel.data());
			if(m_lpOpt->bCloneScanOnJoin)lpChan->doSingleCloneScan(&joiner);	
		} else {
			m_lpCons->doFmtOutput(KVI_OUT_ERROR,"[INTERNAL ERROR] : JOIN message from [%s] to an inexisting channel %s",
							joiner.szNick.data(),channel.data());
		}
	}	
	_debug_leavetrace("handleJoin");
}
//============ handlePart ============//
void KviServerParser::handlePart(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handlePart");
	KviNick parter(szPrefix);
	KviIrcToken par(szParams);
	QString channel=par.nextChannel();
	par.stripUpToTrailing();
	if(par.szString.isEmpty())par.szString="Signoff";
	if(isMe(parter.szNick)){
		m_lpMdi->removeWindow(channel.data());
		m_lpCons->doFmtOutput(KVI_OUT_PART,"You have left channel %s [%s]",channel.data(),par.szString.data());
		if(m_lpEventManager->lpEvent[KVI_Event_OnMePart]->bEnabled){
			QString szPara=channel.copy();
			m_lpUserParser->executeEvent(m_lpCons,m_lpEventManager->lpEvent[KVI_Event_OnMePart],szPara);
		}	
		setStatus("Left %s",channel.data());
		_debug_leavetrace("handlePart...chan window removed");
		return;
	}
	KviChanWnd *lpWnd=getChanWnd(channel.data());
	if(!lpWnd){
		m_lpCons->doFmtOutput(KVI_OUT_ERROR,"[INTERNAL ERROR (PART PARSING)] : Can't find channel %s",channel.data());
		return;
	} else {
		if(lpWnd->getNick(parter))lpWnd->nickPart(parter);
		QString szmask;
		parter.mask(szmask,1);
		if(m_lpEventManager->lpEvent[KVI_Event_OnPart]->bEnabled){
			QString szPara=channel+" "+parter.szNick+" "+parter.szUserName+" "+parter.szAddress;
			m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnPart],szPara);
		}
		lpWnd->doFmtOutput(KVI_OUT_PART,"%s [%s] has left %s [%s]",parter.szNick.data()
				,szmask.data(),channel.data(),par.szString.data());
	}
	_debug_leavetrace("handlePart");
}
//============ handleTopic ============//
void KviServerParser::handleTopic(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handleTopic");
	KviNick topiker(szPrefix); //nick that changes topic
	KviIrcToken par(szParams);
	QString channel=par.nextChannel();
	par.stripUpToTrailing();
//	if(par.szString.isEmpty())szParams="[ No topic is set ]";
	KviChanWnd *lpWnd=getChanWnd(channel.data());
	if(!lpWnd)m_lpCons->doFmtOutput(KVI_OUT_ERROR,"[INTERNAL ERROR (TOPIC PARSING)] Can't find channel %s",channel.data());
	else {	
		QString szmask;
		topiker.mask(szmask,1);
		if(par.szString.isEmpty()){
			lpWnd->doFmtOutput(KVI_OUT_TOPIC,"%s [%s] has unset the topic",topiker.szNick.data(),szmask.data()); 
			lpWnd->setTopic("[ No topic is set ]"); 
		} else { 
			lpWnd->doFmtOutput(KVI_OUT_TOPIC,"%s [%s] changes topic to %s",topiker.szNick.data(),szmask.data(),par.szString.data());
			lpWnd->setTopic(par.szString.data());
		}
		lpWnd->updateUser(topiker);
		if(m_lpEventManager->lpEvent[KVI_Event_OnTopicChange]->bEnabled){
			QString szPara=channel+" "+topiker.szNick+" "+par.szString;
			m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnTopicChange],szPara);
		}	
		setStatus("Topic change for %s",channel.data());
	}
	_debug_leavetrace("handleTopic");
}
//============ handleKick ============//
void KviServerParser::handleKick(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handleKick");
	KviNick kicker(szPrefix);
	KviIrcToken par(szParams);
	QString channel=par.nextWord();
	QString kicked=par.nextWord();
	par.stripUpToTrailing();

	if(par.szString.isEmpty())par.szString=" *** No reason specified *** ";
	QString szmask;
	kicker.mask(szmask,1);

	if(isMe(kicked)){	
		m_lpMdi->removeWindow(channel.data());
		if(m_lpEventManager->lpEvent[KVI_Event_OnMeKick]->bEnabled){
			QString szPara=channel+" "+kicker.szNick+" "+par.szString;
			m_lpUserParser->executeEvent(m_lpCons,m_lpEventManager->lpEvent[KVI_Event_OnMeKick],szPara);
		}	
		topWin()->doFmtOutput(KVI_OUT_KICK,"You have been kicked off %s by %s [%s] [%s]",
				channel.data(),kicker.szNick.data(),szmask.data(),par.szString.data());
		setStatus("Kicked from %s",channel.data());
		if(m_lpOpt->bAutoRejoinOnKick){
			m_lpCons->doFmtOutput(KVI_OUT_INTERNAL,"Attempting to rejoin %s....",channel.data());
			m_lpSock->sendFmtData("JOIN %s",channel.data());
		}
		_debug_leavetrace("handleKick");
		return;
	}

	KviChanWnd *lpWnd=getChanWnd(channel.data());
	if(!lpWnd)m_lpCons->doFmtOutput(KVI_OUT_ERROR,"[INTERNAL ERROR (KICK PARSING)] : Can't find channel %s",channel.data());
	else {
		if(lpWnd->getNick(kicked))lpWnd->nickPart(kicked);
		lpWnd->updateUser(kicker); //just to be fun...
		if(m_lpEventManager->lpEvent[KVI_Event_OnKick]->bEnabled){
			QString szPara=channel+" "+kicker.szNick+" "+kicked+" "+par.szString;
			m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnKick],szPara);
		}
		if(m_lpEventManager->lpEvent[KVI_Event_OnMassKick]->bEnabled){
			if(lpWnd->checkMassKick(kicker.szNick)){
				QString szPara=lpWnd->name();
				szPara+=' ';
				szPara+=kicker.szNick.data();
				m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnMassKick],szPara);
			}
		}
		lpWnd->doFmtOutput(KVI_OUT_KICK,"%s was kicked from %s by %s [%s] [%s]",kicked.data(),channel.data(),
					kicker.szNick.data(),szmask.data(),par.szString.data());
	}
	_debug_leavetrace("handleKick");
}
//============ handleNick ============//
void KviServerParser::handleNick(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handleNick");
	KviNick nicker(szPrefix); //nick that changes nick
	int idx=szParams.find(':');
	if(idx != -1)szParams.remove(0,idx+1);
	szParams=szParams.stripWhiteSpace();

	if(isMe(nicker.szNick)){
		m_lpFrm->m_lpGlb->szNick=szParams.data();
		KviNick srv(m_lpFrm->m_lpGlb->szMaskFromServer);
		srv.szNick=szParams.data();
		srv.mask(m_lpFrm->m_lpGlb->szMaskFromServer);
		m_lpCons->doFmtOutput(KVI_OUT_NICK,"You are now known as %s",szParams.data());
		setStatus("Changed nick to %s",szParams.data());
	}
	if(m_lpEventManager->lpEvent[KVI_Event_OnNick]->bEnabled){
		QString szPara=nicker.szNick+" "+szParams;
		m_lpUserParser->executeEvent(m_lpCons,m_lpEventManager->lpEvent[KVI_Event_OnNick],szPara);
	}	
	int nWinds=m_lpMdi->m_lpChildList->count();
	if(nWinds){
		KviMdiChild *lpChild=0;
		for(lpChild=m_lpMdi->m_lpChildList->first();lpChild;lpChild=m_lpMdi->m_lpChildList->next()){
			if(lpChild->type()==KVI_WND_TYPE_CHAN){
				KviChanWnd *lpWnd=(KviChanWnd *)lpChild;
				KviNick *lpN=lpWnd->getNick(nicker);
				if(lpN){
					KviNick kNewNick(*lpN);
					kNewNick.szNick=szParams.copy();
					kNewNick.szAddress=nicker.szAddress.copy();
					kNewNick.szUserName=nicker.szUserName.copy();
					lpWnd->nickPart(nicker);
					//debug("Window : %s (class : %s)",lpWnd->name(),lpWnd->className());
					lpWnd->nickJoin(kNewNick);
					QString szmask;
					nicker.mask(szmask,1);
					lpWnd->doFmtOutput(KVI_OUT_NICK,
						"%s [%s] is now known as %s",nicker.szNick.data(),szmask.data(),szParams.data());
				}
			}
			QString szWindowName(lpChild->name());
			if((lpChild->type()==KVI_WND_TYPE_QUERY)&&(szWindowName.lower()==nicker.szNick.lower())){
				lpChild->setName(szParams.data());
				((KviQueryWnd *)lpChild)->m_lpInput->m_szParentName=szParams.copy();
				QString szmask;
				nicker.mask(szmask);
				lpChild->doFmtOutput(KVI_OUT_NICK,
					"%s [%s] is now known as %s",nicker.szNick.data(),szmask.data(),szParams.data());
				//nicker.szNick=szParams.copy();
			}
		}
	}
	_debug_leavetrace("handleNick");
}
//============ handlePrivmsg ============//
void KviServerParser::handlePrivmsg(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handlePrivmsg");
	KviNick talker(szPrefix); //from...
	KviIrcToken par(szParams);
	QString target=par.nextWord();
	par.stripUpToTrailing();
	if(par.szString.isEmpty())par.szString="( *** empty privmsg from server *** )";
	//First check for CTCPs
	if((par.szString[0]==0x01)&&(par.szString[par.szString.length()-1]==0x01)){
		m_lpCTCP->handleCTCPRequest(talker,target,&par);
		return;
	}
	if(isMe(target)){
		KviQueryWnd *lpWnd=getQueryWnd(talker.szNick.data());
		QString szMask;
		talker.mask(szMask);
		if(!lpWnd){
			lpWnd=m_lpFrm->createQueryWnd(talker.szNick.data());
			lpWnd->doFmtOutput(KVI_OUT_INTERNAL,"Private conversation request by %s",szMask.data());
			setStatus("Started private conversation with %s",talker.szNick.data());
		}
		QString szNicko;
//		if(m_lpOpt->bInverseNick)szNicko="<\x016";
//		else szNicko="<";
//		szNicko+=talker.szNick;
//		if(m_lpOpt->bInverseNick)szNicko+="\x0f>";
//		else szNicko+=">";
//		if(szNicko.length()<12){
//			szNicko.setExpand(11,(m_lpOpt->bPrefixMsg ? ':' : ' '));
//		} else szNicko+=(m_lpOpt->bPrefixMsg ? ':' : ' ');
		m_lpFrm->formatNickOutputString(szNicko,talker.szNick);
		if(m_lpEventManager->lpEvent[KVI_Event_OnMePrivmsg]->bEnabled){
				QString szPara=talker.szNick.data();
				szPara+=' ';
				szPara+=par.szString.data();
				m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnMePrivmsg],szPara);
		}
		lpWnd->doFmtOutput(KVI_OUT_NONE,"%s%s",szNicko.data(),par.szString.data());
		_debug_leavetrace("handlePrivMsg");
		return;
	}
	KviChanWnd *lpWnd=getChanWnd(target.data());
	if(!lpWnd)m_lpCons->doFmtOutput(KVI_OUT_SERVER,"[PRIVMSG to %s from %s] :%s",target.data(),talker.szNick.data(),par.szString.data());
	else {
		QString szNicko;
		m_lpFrm->formatNickOutputString(szNicko,talker.szNick);

//		if(m_lpOpt->bInverseNick)szNicko="<\x016";
//		else szNicko="<";
//		szNicko+=talker.szNick;
//		if(m_lpOpt->bInverseNick)szNicko+="\x0f>";
//		else szNicko+=">";
//		if(szNicko.length()<12){
//			szNicko.setExpand(11,(m_lpOpt->bPrefixMsg ? ':' : ' '));
//		} else szNicko+=(m_lpOpt->bPrefixMsg ? ':' : ' ');
		if(m_lpEventManager->lpEvent[KVI_Event_OnChannelPrivmsg]->bEnabled){
				QString szPara=target.data();
				szPara+=' ';
				szPara+=talker.szNick.data();
				szPara+=' ';
				szPara+=par.szString.data();
				m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnChannelPrivmsg],szPara);
		}
		lpWnd->doFmtOutput(KVI_OUT_NONE,"%s%s",szNicko.data(),par.szString.data());
		lpWnd->updateUser(talker); //Just for fun...
		if(m_lpEventManager->lpEvent[KVI_Event_OnPublicFlood]->bEnabled){
			if(lpWnd->checkPublicFlood(talker.szNick)){
				QString szPara=lpWnd->name();
				szPara+=' ';
				szPara+=talker.szNick.data();
				m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnPublicFlood],szPara);
			}
		}
	}
	_debug_leavetrace("handlePrivmsg");
}
//============ handleNotice ============//
void KviServerParser::handleNotice(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handleNotice");
	KviNick talker(szPrefix); //from...
	KviIrcToken par(szParams);
	QString target=par.nextWord();
	par.stripUpToTrailing();
	if(par.szString.isEmpty())par.szString="( *** empty notice from server *** )";
	//First check for CTCPs
	if((par.szString[0]==0x01)&&(par.szString[par.szString.length()-1]==0x01)){
		m_lpCTCP->handleCTCPReply(talker,target,&par);
		return;
	}
	bool bServerNotice=(talker.szNick.find('.') != -1);
	if(isMe(target)){
		KviMdiChild *lpWnd=topWin();
		QString szMask;
		talker.mask(szMask);
		QString szNicko;
		m_lpFrm->formatNickOutputString(szNicko,talker.szNick);
//
//		if(m_lpOpt->bInverseNick)szNicko="<\x016";
//		else szNicko="<";
//		szNicko+=talker.szNick;
//		if(m_lpOpt->bInverseNick)szNicko+="\x0f>";
//		else szNicko+=">";
//		if(szNicko.length()<12){
//			szNicko.setExpand(11,(m_lpOpt->bPrefixMsg ? ':' : ' '));
//		} else szNicko+=(m_lpOpt->bPrefixMsg ? ':' : ' ');
		if(bServerNotice){
			if(m_lpEventManager->lpEvent[KVI_Event_OnServerNotice]->bEnabled){
					QString szPara=talker.szNick.data();
					szPara+=' ';
					szPara+=par.szString.data();
					m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnServerNotice],szPara);
			}	
		} else {
			if(m_lpEventManager->lpEvent[KVI_Event_OnMeNotice]->bEnabled){
					QString szPara=talker.szNick.data();
					szPara+=' ';
					szPara+=par.szString.data();
					m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnMeNotice],szPara);
			}
		}
		lpWnd->doFmtOutput(KVI_OUT_NOTICE,"%s%s",szNicko.data(),par.szString.data());
		_debug_leavetrace("handleNotice");
		return;
	}
	KviChanWnd *lpWnd=getChanWnd(target.data());

	if(!lpWnd)m_lpCons->doFmtOutput(KVI_OUT_SERVER,"[NOTICE to %s from %s] :%s",target.data(),talker.szNick.data(),par.szString.data());
	else {
		QString szNicko;
		m_lpFrm->formatNickOutputString(szNicko,talker.szNick);

//		if(m_lpOpt->bInverseNick)szNicko="<\x016";
//		else szNicko="<";
//		szNicko+=talker.szNick;
//		if(m_lpOpt->bInverseNick)szNicko+="\x0f>";
//		else szNicko+=">";
//		if(szNicko.length()<12){
//			szNicko.setExpand(11,(m_lpOpt->bPrefixMsg ? ':' : ' '));
//		} else szNicko+=(m_lpOpt->bPrefixMsg ? ':' : ' ');
		if(bServerNotice){
			if(m_lpEventManager->lpEvent[KVI_Event_OnServerNotice]->bEnabled){
						QString szPara=talker.szNick.data();
						szPara+=' ';
						szPara+=par.szString.data();
						m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnServerNotice],szPara);
			}	
			if(m_lpEventManager->lpEvent[KVI_Event_OnChannelNotice]->bEnabled){
					QString szPara=target.data();
					szPara+=' ';
					szPara+=talker.szNick.data();
					szPara+=' ';
					szPara+=par.szString.data();
					m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnChannelNotice],szPara);
			}
		}
		lpWnd->doFmtOutput(KVI_OUT_NOTICE,"%s%s",szNicko.data(),par.szString.data());
		lpWnd->updateUser(talker); //Just for fun...
	}
	_debug_leavetrace("handleNotice");
}
//============ handleMode ============//
void KviServerParser::handleMode(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handleMode");
	debug("Mode change : %s",szParams.data());
	KviNick source(szPrefix);
	KviIrcToken par(szParams);
	if((par.szString[0]=='#')||(par.szString[0]=='&')){ //CHANNEL MODES
		QString channel	=par.nextWord();
		QString modes	=par.nextWord();
		par.szString.stripWhiteSpace();
		par.szString.simplifyWhiteSpace();
		int idx=par.szString.find(':');
		while(idx != -1){
			par.szString.remove(idx,1);
			idx=par.szString.find(':');
		}
		//We should newer get a ':' in the params now...
		KviChanWnd *lpW=getChanWnd(channel.data());
		if(!lpW){
			m_lpCons->doFmtOutput(KVI_OUT_ERROR,"[INTERNAL ERROR (MODE PARSING)] : Can't find channel %s",channel.data());
			return;
		}
		lpW->updateUser(source);
		if(modes.isEmpty()){
			m_lpCons->doOutput(KVI_OUT_ERROR,"[INTERNAL ERROR] : Empty mode sequence.");
			return;
		}
		QString szMask;
		source.mask(szMask,1);		
		{ /////////BLOCK RECOGNIZE MODES
			//Now szModes contains something as -o+b Mimmo@*.molello.com
			char *lpC=modes.data();
			if((*lpC)==' ')lpC++;
			if(!(*lpC)){
				m_lpCons->doOutput(KVI_OUT_ERROR,"[INTERNAL ERROR] : Empty mode sequence.");
				return;
			}
			//OK we have a + , - ora a mode letter here
			bool bModeAdd=true;
			QString parameter;
			while((*lpC)&&(*lpC != ' ')){
				switch(*lpC){
				case '+':
					bModeAdd=true;
					break;
				case '-':
					bModeAdd=false;
					break;
				case 'o':
				case 'b':
				case 'v':
					parameter=par.nextWord();
					handleChanUserMode(lpW,source,(*lpC),bModeAdd,parameter);
					break;
				case 'k':
				case 'l':
					if(bModeAdd){
						parameter=par.nextWord();
						if(parameter.isEmpty())parameter="[No parameter specified]";
						lpW->doFmtOutput(KVI_OUT_CHANMODE,"%s (%s) sets mode +%c %s for %s",
							source.szNick.data(),szMask.data(),(*lpC),parameter.data(),channel.data());
					} else {
						lpW->doFmtOutput(KVI_OUT_CHANMODE,"%s (%s) sets mode -%c for %s",
							source.szNick.data(),szMask.data(),(*lpC),channel.data());
					}
					lpW->setMode((*lpC),bModeAdd);
					break;
				default:
					lpW->doFmtOutput(KVI_OUT_CHANMODE,"%s [%s] sets mode %c%c for %s",source.szNick.data(),
							szMask.data(),(bModeAdd ? '+' : '-'),(*lpC),channel.data());
					lpW->setMode((*lpC),bModeAdd);
					break;
				}
				lpC++;
			}
			
		}
	} else {
		//Nick modes 
		QString szNicko;
		szNicko=par.nextWord();
		if(szNicko.isEmpty())szNicko=m_lpFrm->m_lpGlb->szNick;
		topWin()->doFmtOutput(KVI_OUT_SERVER,"%s sets mode %s for %s"
					,source.szNick.data(),par.szString.data(),szNicko.data());
	}
	_debug_leavetrace("handleMode");
}
//============ handleChanUserMode ============//
void KviServerParser::handleChanUserMode(KviChanWnd *lpWnd,KviNick &source,char mode,bool bAdd,QString &szParam)
{
	_debug_entertrace("handleChanUserMode");
	QString szMask;
	source.mask(szMask,1);
	int type=KVI_OUT_CHANMODE;
	if(mode != 'b'){
		KviNick *lpN=lpWnd->getNick(szParam);
		if(!lpN){
			m_lpCons->doFmtOutput(KVI_OUT_ERROR,"[INTERNAL ERROR] : Can't find %s on %s",szParam.data(),lpWnd->name());
			return;
		}
	}
	QString szWhat;
	int even;
	bool meTarget=false;
	switch(mode){
		case 'o':
			type=(bAdd ? KVI_OUT_OP        : KVI_OUT_DEOP);
			if(isMe(szParam)){
				meTarget=true;
				even=(bAdd ? KVI_Event_OnMeOp  : KVI_Event_OnMeDeop);
			} else {
				even=(bAdd ? KVI_Event_OnOp    : KVI_Event_OnDeop);
			}
			szWhat="operator status";
			lpWnd->nickOp(szParam.data(),bAdd);
			if(bAdd){
				if(m_lpEventManager->lpEvent[KVI_Event_OnMassOp]->bEnabled){
					if(lpWnd->checkMassOp(source.szNick)){
						QString szPara=lpWnd->name();
						szPara+=' ';
						szPara+=source.szNick.data();
						m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnMassOp],szPara);
					}
				}
			} else {
				if(m_lpEventManager->lpEvent[KVI_Event_OnMassDeop]->bEnabled){
					if(lpWnd->checkMassDeop(source.szNick)){
						QString szPara=lpWnd->name();
						szPara+=' ';
						szPara+=source.szNick.data();
						m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnMassDeop],szPara);
					}
				}
			}
			break;
		case 'b':
			type=(bAdd ? KVI_OUT_BAN       : KVI_OUT_UNBAN);
			{
				KviNick *myNick=lpWnd->getNick(m_lpFrm->m_lpGlb->szNick.data());
				if(myNick->matches(szParam)){
					meTarget=true;
					even=(bAdd ? KVI_Event_OnMeBan : KVI_Event_OnMeUnban);
				} else {
					even=(bAdd ? KVI_Event_OnBan   : KVI_Event_OnUnban);
				}
				QString szNewMask;
				source.mask(szNewMask,0);
				//lpWnd->changeBanList(szParam.data(),szNewMask.data(),bAdd);
				// Fixed by RAD Kade 1
			    lpWnd->changeBanList(szNewMask.data(),szParam.data(),bAdd); 		
				//check mass ban
				if(bAdd){
					if(m_lpEventManager->lpEvent[KVI_Event_OnMassBan]->bEnabled){
						if(lpWnd->checkMassBan(source.szNick)){
							QString szPara=lpWnd->name();
							szPara+=' ';
							szPara+=source.szNick.data();
							m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[KVI_Event_OnMassBan],szPara);
						}
					}
				}
			}
			break;
		case 'v':
			type=(bAdd ? KVI_OUT_VOICE     : KVI_OUT_DEVOICE);
			if(isMe(szParam)){
				meTarget=true;
				even=(bAdd ? KVI_Event_OnMeVoice : KVI_Event_OnMeDevoice);
			} else {
				even=(bAdd ? KVI_Event_OnVoice : KVI_Event_OnDevoice);
			}
			szWhat="voice status";
			lpWnd->nickVoice(szParam.data(),bAdd);
			break;
		default:
			_debug_fatal("handleUnknownMode...we should NEWER arrive here.");
			break;
	}
	if(m_lpEventManager->lpEvent[even]->bEnabled){
		QString szPara=lpWnd->name();
		if(meTarget){
			szPara+=" "+source.szNick;
			if(mode=='b')szPara+=" "+szParam;
		} else {
			szPara+=" "+source.szNick+" "+szParam;
		}
		m_lpUserParser->executeEvent(lpWnd,m_lpEventManager->lpEvent[even],szPara);
	}
	lpWnd->doFmtOutput(type,"%s [%s] sets mode %c%c %s",source.szNick.data(),
				szMask.data(), (bAdd ? '+':'-') , mode , szParam.data());
	if(meTarget&&((mode=='o')||(mode=='v'))){
		QString szStat;
		szStat.sprintf("%s %s on %s",(bAdd ? "Gained" : "Lost"),szWhat.data(),lpWnd->name());
		m_lpFrm->setStatusText(szStat.data());
		if(mode=='o'){
			lpWnd->m_bIsMeOp=bAdd;
			lpWnd->setLabelsPalette(); //show that we can change the topic etc...
		}
	}
	_debug_leavetrace("handleChanUserMode");
}
//============ handleQuit ============//
void KviServerParser::handleQuit(QString &szPrefix,QString &szParams)
{
	_debug_entertrace("handleQuit");
	KviNick quitter(szPrefix); //nick that quits
	KviIrcToken par(szParams);
	par.stripUpToTrailing();
	if(par.szString.isEmpty())par.szString="( *** I Quit *** )";
	if(m_lpEventManager->lpEvent[KVI_Event_OnQuit]->bEnabled){
		QString szPara=quitter.szNick+" "+par.szString;
		m_lpUserParser->executeEvent(m_lpCons,m_lpEventManager->lpEvent[KVI_Event_OnQuit],szPara);
	}
	int nWinds=m_lpMdi->m_lpChildList->count();
	if(nWinds){
		KviMdiChild *lpChild;
		for(lpChild=m_lpMdi->m_lpChildList->first();lpChild;lpChild=m_lpMdi->m_lpChildList->next()){
			if(lpChild->type()==KVI_WND_TYPE_CHAN){
				KviChanWnd *lpWnd=(KviChanWnd *)lpChild;
				if(lpWnd->getNick(quitter)){
					lpWnd->nickPart(quitter);
					QString szmask;
					quitter.mask(szmask,1);
					lpWnd->doFmtOutput(KVI_OUT_QUIT,
						"%s [%s] has quit irc. [%s]",quitter.szNick.data(),
						szmask.data(),par.szString.data());
				}
			}
			QString szWindowName(lpChild->name());
			if((lpChild->type()==KVI_WND_TYPE_QUERY)&&(szWindowName.lower()==quitter.szNick.lower())){
				QString szmask;
				quitter.mask(szmask,1);
				lpChild->doFmtOutput(KVI_OUT_QUIT,"%s [%s] is has quit irc [%s]",quitter.szNick.data(),
										szmask.data(),szParams.data());
			}
		}
	}
	_debug_leavetrace("handleQuit");
}
void KviServerParser::handleWhois(QString &szPrefix,int nCommand,QString &szParams){
	KviIrcToken par(szParams);
	QString szNick=par.nextWord(); //Cut our nickname
	szNick=par.nextWord();         //get the whois target nick
	KviNick nk;
	KviMdiChild *lpC=0;
	QString sz1,sz2;
	switch(nCommand){
	case RPL_WHOISUSER:
	case RPL_WHOWASUSER:
		sz1=par.nextWord(); //user
		sz2=par.nextWord(); //host
		topWin()->doFmtOutput(KVI_OUT_WHO,"%c%s%c %s           : %c%s!%s@%s",KVI_TEXT_BOLD,
				szNick.data(),KVI_TEXT_BOLD,((nCommand==RPL_WHOISUSER) ? "is " : "was"),
							KVI_TEXT_BOLD,szNick.data(),sz1.data(),sz2.data());
		par.stripUpToTrailing();
		topWin()->doFmtOutput(KVI_OUT_WHO,"%c%s%c real name     : %s",KVI_TEXT_BOLD,
				szNick.data(),KVI_TEXT_BOLD,par.szString.data());
		//This one should be safe...(nick+user+host)
		sz1.sprintf("%s!%s@%s",szNick.data(),sz1.data(),sz2.data());
		nk.fromString(sz1);
		for(lpC=m_lpMdi->m_lpChildList->first();lpC;lpC=m_lpMdi->m_lpChildList->next()){
			if(lpC->type()==KVI_WND_TYPE_CHAN)((KviChanWnd *)lpC)->updateUser(nk);
		}
		break;
	case RPL_WHOISSERVER:
		sz1=par.nextWord(); //server
		par.stripUpToTrailing(); //server info
		topWin()->doFmtOutput(KVI_OUT_WHO,"%c%s%c using server  : %s [%s]",KVI_TEXT_BOLD,
				szNick.data(),KVI_TEXT_BOLD,sz1.data(),par.szString.data());
		break;
	case RPL_WHOISOPERATOR:
		topWin()->doFmtOutput(KVI_OUT_WHO,"%c%s%c status        : %cIRC Operator",
				KVI_TEXT_BOLD,szNick.data(),KVI_TEXT_BOLD,KVI_TEXT_UNDERLINE);
		break;
	case RPL_WHOISIDLE:
		sz1=par.nextWord(); //seconds idle
		topWin()->doFmtOutput(KVI_OUT_WHO,"%c%s%c idle time     : %s seconds",KVI_TEXT_BOLD,
				szNick.data(),KVI_TEXT_BOLD,sz1.data());
		break;
	case RPL_WHOISCHANNELS:
		par.stripUpToTrailing(); //channels
		topWin()->doFmtOutput(KVI_OUT_WHO,"%c%s%c channels      : %s",KVI_TEXT_BOLD,
				szNick.data(),KVI_TEXT_BOLD,par.szString.data());
		break;
	case RPL_AWAY: //Added by RAD Kade 1
		par.stripUpToTrailing(); //Away message
		topWin()->doFmtOutput(KVI_OUT_WHO,"%c%s%c is away       : %s",KVI_TEXT_BOLD,
				szNick.data(),KVI_TEXT_BOLD,par.szString.data());
		break;
	case RPL_ENDOFWHOIS:
		topWin()->doFmtOutput(KVI_OUT_WHO,"%c%s%c whois from    : %s",KVI_TEXT_BOLD,
				szNick.data(),KVI_TEXT_BOLD,szPrefix.data());
		break;
	case RPL_ENDOFWHOWAS:
		topWin()->doFmtOutput(KVI_OUT_WHO,"%c%s%c whowas from   : %s",KVI_TEXT_BOLD,
				szNick.data(),KVI_TEXT_BOLD,szPrefix.data());	
		break;
	}
}

