//
//   This file is part of the KVIrc+ irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//
#include "kvi_app.h"

KviSrv::KviSrv(){
	readSettings();
}
KviSrv::~KviSrv(){
	saveSettings();
}
void KviSrv::readSettings(){

//	QString szFileName(QDir::homeDirPath());
//	szFileName+="/kvirc/Config/kvi.server.conf";

	QString szFileName(getKVIrcHomeDirectory("Config/kvi.server.conf"));
	KSimpleConfig cfg(szFileName.data());

	cfg.setGroup("__SERVERS__");

//	cfg.readListEntry("serverList",serverList);

	while(!serverList.isEmpty())serverList.removeLast();

	int cnt=cfg.readNumEntry("serverEntries",0);
	
	QString szEntry;
	QString szNumbr;

	for(int i=0;i<cnt;i++){
		szEntry="srv";
		szNumbr.setNum(i);
		szEntry+=szNumbr;
		serverList.append(cfg.readEntry(szEntry.data(),"__Empty__"));
	}

	if(serverList.isEmpty()){
		serverList.append("IRCNet:irc.tin.it:6667:Resolve at connect::[Telecom Italia Net] Rome:Italy");
		serverList.append("IRCNet:irc.fun.uni.net:6667:Resolve at connect::[Salerno university IRC Server] Salerno:Italy");
		serverList.append("DALNet:irc.dal.net:6667:Resolve at connect::[DALNet main server]");
	}

	cfg.setGroup("__OPTIONS__");

	cacheLookup              = cfg.readNumEntry("cacheLookup",true);
	iTimeout                 = cfg.readNumEntry("iTimeout",30);
	reconnectOnDisconnect    = cfg.readNumEntry("reconnectOnDisconnect",true);
	reconnectOnConnectFailed = cfg.readNumEntry("reconnectOnConnectFailed",true);
	bUseProxy                = cfg.readNumEntry("bUseProxy",false);
	szProxyPort              = cfg.readEntry("szProxyPort","1080");
	szProxyAddress           = cfg.readEntry("szProxyAddress","fire.localdomain");
	szProxyIp                = cfg.readEntry("szProxyIp","");
	szProxyUsername          = cfg.readEntry("szProxyUsername","");
	szProxyPassword          = cfg.readEntry("szProxyPassword","");
}
void KviSrv::saveSettings(){
//
//	QString szFileName(QDir::homeDirPath());
//	szFileName+="/kvirc/Config/kvi.server.conf";
	QString szFileName(getKVIrcHomeDirectory("Config/kvi.server.conf"));
	KSimpleConfig cfg(szFileName.data());

	cfg.setGroup("__SERVERS__");

//	cfg.writeEntry("serverList",serverList);

	int cnt=serverList.count();

	cfg.writeEntry("serverEntries",cnt);
	
	QString szEntry;
	QString szNumbr;

	for(int i=0;i<cnt;i++){
		szEntry="srv";
		szNumbr.setNum(i);
		szEntry+=szNumbr;
		cfg.writeEntry(szEntry.data(),serverList.at(i));
	}

	cfg.setGroup("__OPTIONS__");

	cfg.writeEntry("cacheLookup",cacheLookup);
	cfg.writeEntry("iTimeout",iTimeout);
	cfg.writeEntry("reconnectOnDisconnect",reconnectOnDisconnect);
	cfg.writeEntry("reconnectOnConnectFailed",reconnectOnConnectFailed);
	cfg.writeEntry("bUseProxy",bUseProxy);
	cfg.writeEntry("szProxyPort",szProxyPort);
	cfg.writeEntry("szProxyAddress",szProxyAddress);
	cfg.writeEntry("szProxyIp",szProxyIp);
	cfg.writeEntry("szProxyUsername",szProxyUsername);
	cfg.writeEntry("szProxyPassword",szProxyPassword);

	cfg.sync();

}
//============ readFromMircServersIni ============//
void KviSrv::readFromMircServersIni(const char *_patch)
{
	_debug_entertrace("readFromMircServersIni");
	QFile serversIni(_patch);
	if(!serversIni.exists()){
		KMsgBox::message(0,"File I/O Error","Can't open the specified file for reading",KMsgBox::EXCLAMATION);
		return;
	}

	KSimpleConfig cfg(_patch,true);
	cfg.setGroup("servers");
	int curidx=0;
	QString szIdx;
	szIdx.setNum(curidx);
	QString szKey="n";
	szKey+=szIdx;
	while(cfg.hasKey(szKey.data())){
		QString szEntry=cfg.readEntry(szKey.data());
		QString szNet;
		QString szDes;
		QString szSrv;
		QString szPrt;
		int idx=szEntry.find("SERVER");
		if(idx != -1){
			szDes=szEntry.left(idx);
			szEntry.remove(0,idx+1);
			idx=szEntry.find(':');
			if(idx != -1){
				szEntry.remove(0,idx+1);
				idx=szEntry.find(':');
				if(idx != -1){
					szSrv=szEntry.left(idx);
					szEntry.remove(0,idx+1);
				}
				idx=szEntry.find("GROUP");
				szNet="UnknownNet";
				if(idx != -1){
					szPrt=szEntry.left(idx);
					idx=szPrt.find(',');
					if(idx != -1){ //only one prt allowed
						szPrt=szPrt.left(idx);
					}
					idx=szEntry.find(':');
					if(idx != -1){
						szEntry.remove(0,idx+1);
						if(!szEntry.isEmpty())szNet=szEntry.copy();
					}
				} else szPrt="6667";
				QString szFinal=szNet+":"+szSrv+":"+szPrt+":Resolve at connect::"+szDes;
				serverList.append(szFinal.data());
			}
		}
		szKey="n";
		curidx++;
		szIdx.setNum(curidx);
		szKey+=szIdx;
	}
	_debug_leavetrace("readFromMircServersIni");
}
