//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1998 Szymon Stefanek (stefanek@tin.it)
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Library General Public
//   License as published by the Free Software Foundation; either
//   version 2 of the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//   Library General Public License for more details.
//
//   You should have received a copy of the GNU Library General Public License
//   along with this program; see the file COPYING.  If not, write to
//   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//   Boston, MA 02111-1307, USA.
//

//============ parseIf ============//
int KviUserParser::parseIf(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseIf");
	if(!(**pC))return (_KVI_CMD_parseIf|_KVI_ERR_MissingExpression);
	if((**pC) != '(')return (_KVI_CMD_parseIf|_KVI_ERR_UnexpectedChar);
	const char *rghPtr=(*pC);
	int match=1;
	while(match){
		(rghPtr)++;
		if(!(*rghPtr))return (_KVI_CMD_parseIf|_KVI_ERR_UnexpectedEndInExpr);
		if(*rghPtr=='(')match++;
		if(*rghPtr==')')match--;
	}
	(rghPtr)++;
	if(!(*rghPtr))return (_KVI_CMD_parseIf|_KVI_ERR_UnexpectedEndInExpr);
	(*pC)++; //skip the first (
	QString szExpr((*pC),(rghPtr-(*pC)));
	(*pC)=rghPtr;
	//m_lpIdentifiers->substitute(lpC,szExpr,pP);
	//Copy the then piece
	//szExpr= ( something )
	_macro_skipSenselessChars((*pC));
	QString szThen;
	if(!(**pC))return (_KVI_CMD_parseIf|_KVI_ERR_MissingCommand);
	if(**pC==KVI_CHAR_BEGIN){ //strip up to matching brace
		rghPtr=(*pC);
		match=1;
		while(match){
			(rghPtr)++;
			if(!(*rghPtr))return (_KVI_CMD_parseIf|_KVI_ERR_UnexpectedEndInCmd);
			if(*rghPtr==KVI_CHAR_BEGIN)match++;
			if(*rghPtr==KVI_CHAR_END)match--;
		}
		(rghPtr)++;
		if(!(*rghPtr))return (_KVI_CMD_parseIf|_KVI_ERR_UnexpectedEndInCmd);
		szThen=QString((*pC),(rghPtr-(*pC))+1);
		if(*rghPtr !=';')return (_KVI_CMD_parseIf|_KVI_ERR_MissingSeparator);
		(*pC)=rghPtr;
	} else { //strip up to ;
		rghPtr=(*pC);
		_macro_skipUpToEndOfCmd(rghPtr);
		szThen=QString((*pC),(rghPtr-(*pC))+1);
		(*pC)=rghPtr;
	}
	QString szElse="";
	rghPtr=(*pC);
	if(rghPtr){
		rghPtr++;
		_macro_skipSenselessChars(rghPtr);
		if(!strncasecmp("else",rghPtr,4)){
			rghPtr+=4;
			_macro_skipSenselessChars(rghPtr);
			if(!*rghPtr)return (_KVI_CMD_parseIf|_KVI_ERR_MissingCommand);
			(*pC)=rghPtr;
			if(**pC==KVI_CHAR_BEGIN){ //strip up to matching brace
				rghPtr=(*pC);
				match=1;
				while(match){
					(rghPtr)++;
					if(!(*rghPtr))return (_KVI_CMD_parseIf|_KVI_ERR_UnexpectedEndInCmd);
					if(*rghPtr==KVI_CHAR_BEGIN)match++;
					if(*rghPtr==KVI_CHAR_END)match--;
				}
				(rghPtr)++;
				if(!(*rghPtr))return (_KVI_CMD_parseIf|_KVI_ERR_UnexpectedEndInCmd);
				szElse=QString((*pC),(rghPtr-(*pC))+1);
				if(*rghPtr !=';')return (_KVI_CMD_parseIf|_KVI_ERR_MissingSeparator);
				(*pC)=rghPtr;
			} else { //strip up to ;
				rghPtr=(*pC);
				_macro_skipUpToEndOfCmd(rghPtr);
				szElse=QString((*pC),(rghPtr-(*pC))+1);
				(*pC)=rghPtr;
			}	
		}
	}
	const char *pBool=szExpr.data();
	bool bEvalToTrue;
	int numErr=evaluateBoolean(lpC,&pBool,&bEvalToTrue,pP,false);
	if(numErr & _KVI_ERR_ErrorMask)return numErr;
	if(bEvalToTrue){
		const char *pCmd=szThen.data();
		return parseUserCommand(lpC,&pCmd,pP);
	} else {
		if(!szElse.isEmpty()){
			const char *pCmd=szElse.data();
			return parseUserCommand(lpC,&pCmd,pP);
		} else return (_KVI_CMD_parseIf|_KVI_ERR_Success);
	}
	_debug_leavetrace("parseIf");
}
//============ parseWhile ============//
int KviUserParser::parseWhile(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseWhile");
	if(!(**pC))return (_KVI_CMD_parseWhile|_KVI_ERR_MissingExpression);
	if((**pC) != '(')return (_KVI_CMD_parseWhile|_KVI_ERR_UnexpectedChar);
	const char *rghPtr=(*pC);
	int match=1;
	while(match){
		(rghPtr)++;
		if(!(*rghPtr))return (_KVI_CMD_parseWhile|_KVI_ERR_UnexpectedEndInExpr);
		if(*rghPtr=='(')match++;
		if(*rghPtr==')')match--;
	}
	(rghPtr)++;
	if(!(*rghPtr))return (_KVI_CMD_parseWhile|_KVI_ERR_UnexpectedEndInExpr);
	(*pC)++; //skip the first (
	QString szExpr((*pC),(rghPtr-(*pC)));
	(*pC)=rghPtr;
	//Got the expression
	_macro_skipSenselessChars((*pC));
	QString szThen;
	if(!(**pC))return (_KVI_CMD_parseWhile|_KVI_ERR_MissingCommand);
	if(**pC==KVI_CHAR_BEGIN){ //strip up to matching brace
		rghPtr=(*pC);
		match=1;
		while(match){
			(rghPtr)++;
			if(!(*rghPtr))return (_KVI_CMD_parseWhile|_KVI_ERR_UnexpectedEndInCmd);
			if(*rghPtr==KVI_CHAR_BEGIN)match++;
			if(*rghPtr==KVI_CHAR_END)match--;
		}
		(rghPtr)++;
		if(!(*rghPtr))return (_KVI_CMD_parseWhile|_KVI_ERR_UnexpectedEndInCmd);
		szThen=QString((*pC),(rghPtr-(*pC))+1);
		if(*rghPtr !=';')return (_KVI_CMD_parseWhile|_KVI_ERR_MissingSeparator);
		(*pC)=rghPtr;
	} else { //strip up to ;
		rghPtr=(*pC);
		_macro_skipUpToEndOfCmd(rghPtr);
		szThen=QString((*pC),(rghPtr-(*pC))+1);
		(*pC)=rghPtr;
	}
	bool bEvalToTrue=true;
	while(bEvalToTrue){
		const char *pBool=szExpr.data();
		int numErr=evaluateBoolean(lpC,&pBool,&bEvalToTrue,pP,false);
		if(numErr & _KVI_ERR_ErrorMask)return numErr;
		if(bEvalToTrue){
			const char *pCmd=szThen.data();
			numErr=parseUserCommand(lpC,&pCmd,pP);
			if(numErr & _KVI_ERR_ErrorMask)return numErr;
		}
	}
	_debug_leavetrace("parseWhile");
	return (_KVI_CMD_parseWhile|_KVI_ERR_Success);
}
//============ parseJoin ============//
int KviUserParser::parseJoin(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseJoin");
	if(!(**pC))return (_KVI_CMD_parseJoin|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseJoin|_KVI_ERR_MissingChannel);
	if(m_lpSock->sendFmtData("JOIN %s",szCmd.data()))return (_KVI_CMD_parseJoin|_KVI_ERR_NotConnected);
	(*pC)=rghPtr;
	_debug_leavetrace("parseJoin");
	return (_KVI_CMD_parseJoin|_KVI_ERR_Success);
}
//============ parseChat ============//
int KviUserParser::parseChat(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseChat");
	if(!(**pC))return (_KVI_CMD_parseChat|_KVI_ERR_MissingTarget);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseChat|_KVI_ERR_MissingTarget);
	QString szTrg="";
	int idx=0;
	while(!szCmd.isEmpty()){
		idx=szCmd.find(',');
		if(idx==-1){ szTrg=szCmd;szCmd=""; }
		else { szTrg=szCmd.left(idx);szCmd.remove(0,idx+1); }
		szTrg=szTrg.stripWhiteSpace();	
		QString szChatName="[DCC-CHAT]:-"+szTrg;
		if(m_lpMdi->findWindow(szChatName.data())||(szTrg==m_szLastDCCTarget)){
			lpC->doFmtOutput(KVI_OUT_ERROR,"[DCC CHAT] : A previous DCC CHAT to %s already exists",szTrg.data());
		} else {
			m_szLastDCCTarget=szTrg.copy();
			KviChatWnd *lpW=m_lpFrm->createChatWnd(szChatName.data());
			lpW->requestChat(szTrg.data());
		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseChat");
	return (_KVI_CMD_parseChat|_KVI_ERR_Success);
}
//============ parseQuery ============//
int KviUserParser::parseQuery(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseQuery");
	if(!(**pC))return (_KVI_CMD_parseQuery|_KVI_ERR_MissingTarget);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseQuery|_KVI_ERR_MissingTarget);
	if(!m_lpSock->m_bConnected)return (_KVI_CMD_parseQuery|_KVI_ERR_NotConnected);
	QString szTrg;
	while(!szCmd.isEmpty()){
		int idx=szCmd.find(',');
		if(idx==-1){ szTrg=szCmd; szCmd=""; }
		else { szTrg=szCmd.left(idx); szCmd.remove(0,idx+1); }
		szTrg=szTrg.stripWhiteSpace();
		if(m_lpMdi->findWindow(szTrg.data())||(szTrg==m_szLastQueryTarget)){
			lpC->doFmtOutput(KVI_OUT_ERROR,"[QUERY] : A previous QUERY to %s already exists",szTrg.data());
		} else {
			m_szLastQueryTarget=szTrg.copy();
			KviQueryWnd *lpW=m_lpFrm->createQueryWnd(szTrg.data());
			lpW->doFmtOutput(KVI_OUT_INTERNAL,"Starting conversation with %s",szTrg.data());	
		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseQuery");
	return (_KVI_CMD_parseQuery|_KVI_ERR_Success);
}
//============ parsePart ============//
int KviUserParser::parsePart(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parsePart");
	if(!(**pC))return (_KVI_CMD_parsePart|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parsePart|_KVI_ERR_MissingChannel);
	if((szCmd[0]!='#')&&(szCmd[0]!='&'))return (_KVI_CMD_parsePart|_KVI_ERR_MissingChannel);
	int idx=szCmd.find(' ');
	if(idx!=-1)szCmd.insert(idx+1,':');
	else {
		if(!m_lpOpt->szPartMsg.isEmpty()){
			szCmd+=" :";szCmd+=m_lpOpt->szPartMsg;m_lpIdentifiers->substitute(lpC,szCmd,pP);
		}
	}
	if(m_lpSock->sendFmtData("PART %s",szCmd.data()))return (_KVI_CMD_parsePart|_KVI_ERR_NotConnected);
	(*pC)=rghPtr;
	_debug_leavetrace("parsePart");
	return (_KVI_CMD_parsePart|_KVI_ERR_Success);
}
//============ parseRaw ============//
int KviUserParser::parseRaw(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseRaw");
	if(!(**pC))return (_KVI_CMD_parseRaw|_KVI_ERR_MissingData);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseRaw|_KVI_ERR_MissingData);
	if(m_lpSock->sendData(szCmd.data()))return (_KVI_CMD_parseRaw|_KVI_ERR_NotConnected);
	m_lpCons->doFmtOutput(KVI_OUT_RAW,"[SENDING TO SERVER] : %s",szCmd.data());
	(*pC)=rghPtr;
	_debug_leavetrace("parseRaw");
	return (_KVI_CMD_parseRaw|_KVI_ERR_Success);
}
//============ parseNick ============//
int KviUserParser::parseNick(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseNick");
	if(!(**pC))return (_KVI_CMD_parseNick|_KVI_ERR_MissingNick);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseNick|_KVI_ERR_MissingNick);
	if(m_lpSock->sendFmtData("NICK %s",szCmd.data()))return (_KVI_CMD_parseNick|_KVI_ERR_NotConnected);
	(*pC)=rghPtr;
	_debug_leavetrace("parseNick");
	return (_KVI_CMD_parseNick|_KVI_ERR_Success);
}
//============ parseWhois ============//
int KviUserParser::parseWhois(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseWhois");
	if(!(**pC))return (_KVI_CMD_parseWhois|_KVI_ERR_MissingNick);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseWhois|_KVI_ERR_MissingNick);
	if(szCmd.length()>3){
		if((szCmd[0]=='-')&&(szCmd[1]=='i')&&(szCmd[2]==' ')){
			szCmd.remove(0,3);
			szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())return (_KVI_CMD_parseWhois|_KVI_ERR_MissingNick);
			int idxComma=szCmd.find(',');
			QString szFirstNick="";
			if(idxComma != -1){ szFirstNick=szCmd.left(idxComma); }
			else { szFirstNick=szCmd.copy(); }
			szCmd.prepend(" ");
			szCmd.prepend(szFirstNick.data());
		}
	}
	if(m_lpSock->sendFmtData("WHOIS %s",szCmd.data()))return (_KVI_CMD_parseWhois|_KVI_ERR_NotConnected);
	(*pC)=rghPtr;
	_debug_leavetrace("parseWhois");
	return (_KVI_CMD_parseWhois|_KVI_ERR_Success);
}
//============ parseSet ============//
int KviUserParser::parseSet(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseSet");
	if(!(**pC))return (_KVI_CMD_parseSet|_KVI_ERR_MissingVariable);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
	szCmd=szCmd.stripWhiteSpace();
	if(szCmd.length()<2)return (_KVI_CMD_parseSet|_KVI_ERR_MissingVariable);
	if(szCmd[0] != '$')return (_KVI_CMD_parseSet|_KVI_ERR_ExpectedVariable);
	if((szCmd[1] == '_') || (szCmd[1] == '%'))return (_KVI_CMD_parseSet|_KVI_ERR_CantSetSystemVar);
	QString szVariable;
	int idx=szCmd.find(' ');
	if(idx!=-1){
		szVariable=szCmd.left(idx);szCmd.remove(0,idx+1);
		m_lpIdentifiers->substitute(lpC,szCmd,pP);szCmd=szCmd.stripWhiteSpace();
		if(szCmd.find(szVariable.data())!= -1)return (_KVI_CMD_parseSet|_KVI_ERR_InfiniteRecursion);
		if(szCmd.isEmpty())return (_KVI_CMD_parseSet|_KVI_ERR_MissingValue);
	} else return (_KVI_CMD_parseSet|_KVI_ERR_MissingValue);
	if((szVariable[1]>='0')&&(szVariable[1]<='9')&&(szVariable.length()==2)){
		if(!pP)return (_KVI_CMD_parseSet|_KVI_ERR_ThisIsNotAnAlias);
		szVariable.remove(0,1);
		int index=szVariable.toInt();
		pP->param[index]=szCmd.copy();
	} else {
		KviVariable *lpV=0;
		for(lpV=m_lpVarList->first();lpV;lpV=m_lpVarList->next()){
			if(!strncasecmp(lpV->szName.data(),szVariable.data(),strlen(lpV->szName.data()))){
				lpV->szValue=szCmd.copy();
				(*pC)=rghPtr;
				return (_KVI_CMD_parseSet|_KVI_ERR_Success);
			}
		}
		lpV=new KviVariable;
		lpV->szName=szVariable.copy();
		lpV->szValue=szCmd.copy();
		m_lpVarList->append(lpV);
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseSet");
	return (_KVI_CMD_parseSet|_KVI_ERR_Success);	
}
//============ parseUnset ============//
int KviUserParser::parseUnset(KviMdiChild *,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseUnset");
	if(!(**pC))return (_KVI_CMD_parseUnset|_KVI_ERR_MissingVariable);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
	szCmd=szCmd.stripWhiteSpace();
	if(szCmd.length()<2)return (_KVI_CMD_parseUnset|_KVI_ERR_MissingVariable);
	if(szCmd[0] != '$')return (_KVI_CMD_parseUnset|_KVI_ERR_ExpectedVariable);
	if((szCmd[1] == '_')||(szCmd[1]=='%'))return (_KVI_CMD_parseSet|_KVI_ERR_CantSetSystemVar);
	int idx=szCmd.find(' ');
	if(idx!=-1)return (_KVI_CMD_parseUnset|_KVI_ERR_CantUnsetTwo);
	if((szCmd[1]>='0')&&(szCmd[1]<='9')&&(szCmd.length()==2)){
		if(!pP)return (_KVI_CMD_parseUnset|_KVI_ERR_ThisIsNotAnAlias);
		szCmd.remove(0,1);
		int index=szCmd.toInt();
		pP->param[index]=KVI_STR_NULL;
	} else {
		KviVariable *lpV=0;
		for(lpV=m_lpVarList->first();lpV;lpV=m_lpVarList->next()){
			if(!strncasecmp(lpV->szName.data(),szCmd.data(),strlen(lpV->szName.data()))){
				m_lpVarList->removeRef(lpV);
				(*pC)=rghPtr;		
				return (_KVI_CMD_parseUnset|_KVI_ERR_Success);
			}
		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseUnset");
	return (_KVI_CMD_parseUnset|_KVI_ERR_Success);
}
//============ parseEcho ============//
int KviUserParser::parseEcho(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseEcho");
	if(!(**pC))return (_KVI_CMD_parseEcho|_KVI_ERR_MissingWindow);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseEcho|_KVI_ERR_MissingWindow);
	QString szWindow;
	int idx=szCmd.find(' ');
	if(idx!=-1){
		szWindow=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
		if(szCmd.isEmpty())return (_KVI_CMD_parseEcho|_KVI_ERR_MissingText);
	} else return (_KVI_CMD_parseEcho|_KVI_ERR_MissingText);
	KviMdiChild *lpW=m_lpMdi->findWindow(szWindow.data());
	if(lpW){
		lpW->doOutput(KVI_OUT_ECHO,szCmd.data());
		(*pC)=rghPtr;	
		return (_KVI_CMD_parseEcho|_KVI_ERR_Success);
	} else return (_KVI_CMD_parseEcho|_KVI_ERR_WindowNotFound);
	_debug_leavetrace("parseEcho");
}
//============ parseCtcp ============//
int KviUserParser::parseCtcp(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseCtcp");
	if(!(**pC))return (_KVI_CMD_parseCtcp|_KVI_ERR_MissingCtcpType);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseCtcp|_KVI_ERR_MissingCtcpType);
	QString szType="";
	int idx=szCmd.find(' ');
	if(idx!=-1){ //search fot the nick after a space...
		szType=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
		if(szCmd.isEmpty())return (_KVI_CMD_parseCtcp|_KVI_ERR_MissingNick);
	} else return (_KVI_CMD_parseCtcp|_KVI_ERR_MissingNick);
	szType=szType.upper();
	QString szTarget="";
	//szType=CTCP szCmd now contains the target list and optionally the ctcp arguments
	idx=szCmd.find(' '); //search for the params at the end
	if(idx!=-1){ //found params...szTaget=up to space,szCmd=params
		szTarget=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
	} else { //no params,szTarget=szCmd , szCmd=nothing
		szTarget=szCmd.copy();
		szCmd="";
	}
	QString szTrg;
	while(!szTarget.isEmpty()){
		idx=szTarget.find(',');
		if(idx==-1){szTrg=szTarget.copy();szTarget="";}
		else {szTrg=szTarget.left(idx); szTarget.remove(0,idx+1);}
		if(szType != "PING"){
			if(szCmd.isEmpty()){
				if(m_lpSock->sendFmtData("PRIVMSG %s :%c%s%c",szTrg.data(),0x01,szType.data(),0x01))return (_KVI_CMD_parseCtcp|_KVI_ERR_NotConnected);
			} else {
				if(m_lpSock->sendFmtData("PRIVMSG %s :%c%s %s%c",szTrg.data(),0x01,szType.data(),szCmd.data(),0x01))return (_KVI_CMD_parseCtcp|_KVI_ERR_NotConnected);
			}
		} else {
			if(m_lpSock->sendFmtData("PRIVMSG %s :%cPING %u%c",
				szTrg.data(),0x01,(uint)time(0),0x01))return(_KVI_CMD_parseCtcp|_KVI_ERR_NotConnected);
 		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseCtcp");
	return (_KVI_CMD_parseCtcp|_KVI_ERR_Success);
}
//============ parseMe ============//
int KviUserParser::parseMe(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseMe");
	if(!(**pC))return (_KVI_CMD_parseMe|_KVI_ERR_MissingText);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseMe|_KVI_ERR_MissingText);
	if((lpC->type()==KVI_WND_TYPE_CHAN)||(lpC->type()==KVI_WND_TYPE_QUERY)){
		if(m_lpSock->sendFmtData("PRIVMSG %s :%cACTION %s%c",lpC->name(),0x01,szCmd.data(),0x01))return (_KVI_CMD_parseMe|_KVI_ERR_NotConnected);
		lpC->doFmtOutput(KVI_OUT_ACTION,"%s %s",m_lpGlb->szNick.data(),szCmd.data());
	} else if(lpC->type()==KVI_WND_TYPE_CHAT){
		if(!((KviChatWnd *)lpC)->m_bConnected)return (_KVI_CMD_parseMe|_KVI_ERR_NotConnected);
		QString szDCCCmd;
		szDCCCmd+=0x01;
		szDCCCmd+="ACTION ";
		szDCCCmd+=szCmd.data();
		szDCCCmd+=0x01;
		lpC->sendData(szDCCCmd.data());
		lpC->doFmtOutput(KVI_OUT_ACTION,"%s %s",m_lpGlb->szNick.data(),szCmd.data());
	} else return (_KVI_CMD_parseMe|_KVI_ERR_NonChannelAction);
	(*pC)=rghPtr;
	_debug_leavetrace("parseMe");
	return (_KVI_CMD_parseMe|_KVI_ERR_Success);
}
//============ parseNotice ============//
int KviUserParser::parseNotice(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseNotice");
	if(!(**pC))return (_KVI_CMD_parseNotice|_KVI_ERR_MissingTarget);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
	szCmd=szCmd.stripWhiteSpace();
	if(szCmd.isEmpty())return (_KVI_CMD_parseNotice|_KVI_ERR_MissingTarget);
	QString szTarget;
	int idx=szCmd.find(' ');
	if(idx!=-1){
		szTarget=szCmd.left(idx);
		m_lpIdentifiers->substitute(lpC,szTarget,pP);
		szTarget=szTarget.stripWhiteSpace();
		szCmd.remove(0,idx+1);
		szCmd=szCmd.stripWhiteSpace();
		if(szCmd.isEmpty())return (_KVI_CMD_parseNotice|_KVI_ERR_MissingText);
	} else return (_KVI_CMD_parseNotice|_KVI_ERR_MissingText);
	QString szTrg;
	while(!szTarget.isEmpty()){
		idx=szTarget.find(',');
		if(idx==-1){ szTrg=szTarget; szTarget="";}
		else { szTrg=szTarget.left(idx); szTarget.remove(0,idx+1); }
		KviMdiChild *lpW=m_lpMdi->findWindow(szTrg.data());
		m_lpIdentifiers->substitute(lpC,szCmd,pP);
		if(m_lpSock->sendFmtData("NOTICE %s :%s",szTrg.data(),szCmd.data()))return (_KVI_CMD_parseNotice|_KVI_ERR_NotConnected);		
		if(lpW){
			if((lpW->type()==KVI_WND_TYPE_QUERY)||(lpW->type()==KVI_WND_TYPE_CHAN)){
				QString szNicko;
				m_lpFrm->formatNickOutputString(szNicko,m_lpGlb->szNick);
//				szNicko+=m_lpGlb->szNick.data();
//				szNicko+=">";
//				if(szNicko.length()<12){
//					szNicko.setExpand(11,(m_lpOpt->bPrefixMsg ? ':' : ' '));
//				} else szNicko+=(m_lpOpt->bPrefixMsg ? ':' : ' ');
				lpW->doFmtOutput(KVI_OUT_NOTICE,"%s%s",szNicko.data(),szCmd.data());
			} else {
				lpC->doFmtOutput(KVI_OUT_NOTICE,"[>> %s >>] : %s",szTrg.data(),szCmd.data());	
			}
		} else {
			lpC->doFmtOutput(KVI_OUT_NOTICE,"[>> %s >>] : %s",szTrg.data(),szCmd.data());	
		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseNotice");
	return (_KVI_CMD_parseNotice|_KVI_ERR_Success);
}
//============ parseSay ============//
int KviUserParser::parseSay(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseSay");
	if(!(**pC))return (_KVI_CMD_parseSay|_KVI_ERR_MissingWindow);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
	szCmd=szCmd.stripWhiteSpace();
	if(szCmd.isEmpty())return (_KVI_CMD_parseSay|_KVI_ERR_MissingWindow);
	QString szTarget;
	int idx=szCmd.find(' ');
	if(idx!=-1){
		szTarget=szCmd.left(idx);
		m_lpIdentifiers->substitute(lpC,szTarget,pP);
		szTarget=szTarget.stripWhiteSpace();
		szCmd.remove(0,idx+1);
		szCmd=szCmd.stripWhiteSpace();
		if(szCmd.isEmpty())return (_KVI_CMD_parseSay|_KVI_ERR_MissingText);
	} else return (_KVI_CMD_parseSay|_KVI_ERR_MissingText);
	QString szTrg;
	while(!szTarget.isEmpty()){
		idx=szTarget.find(',');
		if(idx==-1){ szTrg=szTarget; szTarget="";}
		else { szTrg=szTarget.left(idx); szTarget.remove(0,idx+1); }
		KviMdiChild *lpW=m_lpMdi->findWindow(szTrg.data());
		m_lpIdentifiers->substitute(lpC,szCmd,pP);
		if(lpW){
			if( ((lpW->type())==KVI_WND_TYPE_CHAT)||((lpW->type())==KVI_WND_TYPE_CHAN)||
				((lpW->type())==KVI_WND_TYPE_QUERY) )parseUserInput(lpW,lpW->type(),lpW->name(),szCmd);
			else {
				if(m_lpSock->sendFmtData("PRIVMSG %s :%s",szTrg.data(),szCmd.data()))return (_KVI_CMD_parseSay|_KVI_ERR_NotConnected);
				lpC->doFmtOutput(KVI_OUT_PRIVMSG,"[>> %s >>] : %s",szTrg.data(),szCmd.data());		
			}
		} else {
			if(m_lpSock->sendFmtData("PRIVMSG %s :%s",szTrg.data(),szCmd.data()))return (_KVI_CMD_parseSay|_KVI_ERR_NotConnected);	
			lpC->doFmtOutput(KVI_OUT_PRIVMSG,"[>> %s >>] : %s",szTrg.data(),szCmd.data());	
		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseSay");
	return (_KVI_CMD_parseSay|_KVI_ERR_Success);
}
//============ parseTopic ============//
int KviUserParser::parseTopic(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseTopic");
	if(!(**pC))return (_KVI_CMD_parseTopic|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseTopic|_KVI_ERR_MissingChannel);
	QString szChannel;
	if((szCmd[0]=='#')||(szCmd[0]=='&')){
		int idx=szCmd.find(' ');
		if(idx != -1){
			szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())szCmd=" ";
		} else {szChannel=szCmd;szCmd=" ";}
	} else {
		if(lpC->type()==KVI_WND_TYPE_CHAN)szChannel=lpC->name();
		else return (_KVI_CMD_parseTopic|_KVI_ERR_NotOnChannel);
	}
	if(m_lpSock->sendFmtData("TOPIC %s :%s",szChannel.data(),szCmd.data()))return (_KVI_CMD_parseTopic|_KVI_ERR_NotConnected);
	(*pC)=rghPtr;
	_debug_leavetrace("parseTopic");
	return (_KVI_CMD_parseTopic|_KVI_ERR_Success);
}
//============ parseMode ============//
int KviUserParser::parseMode(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseMode");
	if(!(**pC))return (_KVI_CMD_parseMode|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
	m_lpIdentifiers->substitute(lpC,szCmd,pP);szCmd=szCmd.stripWhiteSpace();
	if(szCmd.isEmpty())return (_KVI_CMD_parseMode|_KVI_ERR_MissingChannel);
	QString szChannel;
	int idx=szCmd.find(' ');
	if(idx != -1){
		szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
		if(szCmd.isEmpty())return (_KVI_CMD_parseMode|_KVI_ERR_MissingMode);
	} else return (_KVI_CMD_parseMode|_KVI_ERR_MissingMode);
	if(m_lpSock->sendFmtData("MODE %s %s",szChannel.data(),szCmd.data()))return (_KVI_CMD_parseMode|_KVI_ERR_NotConnected);
	(*pC)=rghPtr;
	_debug_leavetrace("parseMode");
	return (_KVI_CMD_parseMode|_KVI_ERR_Success);
}
//============ parseKick ============//
int KviUserParser::parseKick(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseKick");
	if(!(**pC))return (_KVI_CMD_parseKick|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseKick|_KVI_ERR_MissingChannel);
	QString szChannel;
	int idx=0;
	if((szCmd[0]=='#')||(szCmd[0]=='&')){
		idx=szCmd.find(' ');
		if(idx != -1){
			szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())return (_KVI_CMD_parseKick|_KVI_ERR_MissingNick);
		} else return (_KVI_CMD_parseKick|_KVI_ERR_MissingNick);
	} else {
		if(lpC->type()==KVI_WND_TYPE_CHAN)szChannel=lpC->name();
		else return (_KVI_CMD_parseKick|_KVI_ERR_NotOnChannel);
	}
	QString szTrg;
	idx=szCmd.find(' ');
	if(idx != -1){
		szTrg=szCmd.left(idx);
		szCmd.remove(0,idx+1);
		szCmd=szCmd.stripWhiteSpace();
		if(szCmd.isEmpty()){
			if(m_lpOpt->szKickMsg.isEmpty())szCmd="Bye";
			else {szCmd=m_lpOpt->szKickMsg;m_lpIdentifiers->substitute(lpC,szCmd,pP);}
		}
	} else {
		szTrg=szCmd.copy();
		if(m_lpOpt->szKickMsg.isEmpty())szCmd="Bye";
		else { szCmd=m_lpOpt->szKickMsg;m_lpIdentifiers->substitute(lpC,szCmd,pP);}
	}
	QString szTarget;
	while(!szTrg.isEmpty()){
		idx=szTrg.find(',');
		if(idx==-1){ szTarget=szTrg; szTrg=""; }
		else { szTarget=szTrg.left(idx); szTrg.remove(0,idx+1); }
		if(m_lpSock->sendFmtData("KICK %s %s :%s",szChannel.data(),szTarget.data(),szCmd.data()))return (_KVI_CMD_parseKick|_KVI_ERR_NotConnected);
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseKick");
	return (_KVI_CMD_parseKick|_KVI_ERR_Success);
}
//============ parseOp ============//
int KviUserParser::parseOp(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseOp");
	if(!(**pC))return (_KVI_CMD_parseOp|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseOp|_KVI_ERR_MissingChannel);
	QString szChannel;
	if((szCmd[0]=='#')||(szCmd[0]=='&')){
		int idx=szCmd.find(' ');
		if(idx != -1){
			szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())return (_KVI_CMD_parseOp|_KVI_ERR_MissingNick);
		} else return (_KVI_CMD_parseOp|_KVI_ERR_MissingNick);
	} else {
		if(lpC->type()==KVI_WND_TYPE_CHAN)szChannel=lpC->name();
		else return (_KVI_CMD_parseOp|_KVI_ERR_NotOnChannel);
	}
	int numError=executeMode(szChannel,szCmd,true,'o');
	if(numError & _KVI_ERR_ErrorMask)return numError;
	(*pC)=rghPtr;
	_debug_leavetrace("parseOp");
	return (_KVI_CMD_parseOp|_KVI_ERR_Success);
}
//============ parseDeop ============//
int KviUserParser::parseDeop(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseDeop");
	if(!(**pC))return (_KVI_CMD_parseDeop|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseDeop|_KVI_ERR_MissingChannel);
	QString szChannel;
	if((szCmd[0]=='#')||(szCmd[0]=='&')){
		int idx=szCmd.find(' ');
		if(idx != -1){
			szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())return (_KVI_CMD_parseDeop|_KVI_ERR_MissingNick);
		} else return (_KVI_CMD_parseDeop|_KVI_ERR_MissingNick);
	} else {
		if(lpC->type()==KVI_WND_TYPE_CHAN)szChannel=lpC->name();
		else return (_KVI_CMD_parseDeop|_KVI_ERR_NotOnChannel);
	}
	int numError=executeMode(szChannel,szCmd,false,'o');
	if(numError & _KVI_ERR_ErrorMask)return numError;
	(*pC)=rghPtr;
	_debug_leavetrace("parseDeop");
	return (_KVI_CMD_parseDeop|_KVI_ERR_Success);
}
//============ parseBan ============//
int KviUserParser::parseBan(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseBan");
	if(!(**pC))return (_KVI_CMD_parseBan|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseBan|_KVI_ERR_MissingChannel);
	QString szChannel;
	if((szCmd[0]=='#')||(szCmd[0]=='&')){
		int idx=szCmd.find(' ');
		if(idx != -1){
			szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())return (_KVI_CMD_parseBan|_KVI_ERR_MissingNick);
		} else return (_KVI_CMD_parseBan|_KVI_ERR_MissingNick);
	} else {
		if(lpC->type()==KVI_WND_TYPE_CHAN)szChannel=lpC->name();
		else return (_KVI_CMD_parseBan|_KVI_ERR_NotOnChannel);
	}
	int numError=executeMode(szChannel,szCmd,true,'b');
	if(numError & _KVI_ERR_ErrorMask)return numError;
	(*pC)=rghPtr;
	_debug_leavetrace("parseBan");
	return (_KVI_CMD_parseBan|_KVI_ERR_Success);
}
//============ parseUnban ============//
int KviUserParser::parseUnban(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseUnban");
	if(!(**pC))return (_KVI_CMD_parseUnban|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseUnban|_KVI_ERR_MissingChannel);
	QString szChannel;
	if((szCmd[0]=='#')||(szCmd[0]=='&')){
		int idx=szCmd.find(' ');
		if(idx != -1){
			szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())return (_KVI_CMD_parseUnban|_KVI_ERR_MissingNick);
		} else return (_KVI_CMD_parseUnban|_KVI_ERR_MissingNick);
	} else {
		if(lpC->type()==KVI_WND_TYPE_CHAN)szChannel=lpC->name();
		else return (_KVI_CMD_parseUnban|_KVI_ERR_NotOnChannel);
	}
	int numError=executeMode(szChannel,szCmd,false,'b');
	if(numError & _KVI_ERR_ErrorMask)return numError;
	(*pC)=rghPtr;
	_debug_leavetrace("parseUnban");
	return (_KVI_CMD_parseUnban|_KVI_ERR_Success);
}
//============ parseVoice ============//
int KviUserParser::parseVoice(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseVoice");
	if(!(**pC))return (_KVI_CMD_parseVoice|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseVoice|_KVI_ERR_MissingChannel);
	QString szChannel;
	if((szCmd[0]=='#')||(szCmd[0]=='&')){
		int idx=szCmd.find(' ');
		if(idx != -1){
			szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())return (_KVI_CMD_parseVoice|_KVI_ERR_MissingNick);
		} else return (_KVI_CMD_parseVoice|_KVI_ERR_MissingNick);
	} else {
		if(lpC->type()==KVI_WND_TYPE_CHAN)szChannel=lpC->name();
		else return (_KVI_CMD_parseVoice|_KVI_ERR_NotOnChannel);
	}
	int numError=executeMode(szChannel,szCmd,true,'v');
	if(numError & _KVI_ERR_ErrorMask)return numError;
	(*pC)=rghPtr;
	_debug_leavetrace("parseVoice");
	return (_KVI_CMD_parseVoice|_KVI_ERR_Success);
}
//============ parseDevoice ============//
int KviUserParser::parseDevoice(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseDevoice");
	if(!(**pC))return (_KVI_CMD_parseDevoice|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseDevoice|_KVI_ERR_MissingChannel);
	QString szChannel;
	if((szCmd[0]=='#')||(szCmd[0]=='&')){
		int idx=szCmd.find(' ');
		if(idx != -1){
			szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())return (_KVI_CMD_parseDevoice|_KVI_ERR_MissingNick);
		} else return (_KVI_CMD_parseDevoice|_KVI_ERR_MissingNick);
	} else {
		if(lpC->type()==KVI_WND_TYPE_CHAN)szChannel=lpC->name();
		else return (_KVI_CMD_parseDevoice|_KVI_ERR_NotOnChannel);
	}
	int numError=executeMode(szChannel,szCmd,false,'v');
	if(numError & _KVI_ERR_ErrorMask)return numError;
	(*pC)=rghPtr;
	_debug_leavetrace("parseDevoice");
	return (_KVI_CMD_parseDevoice|_KVI_ERR_Success);
}
//============ parseHelp ============//
int KviUserParser::parseHelp(KviMdiChild *lpC,const char ** pC,KviAliasParams *)
{
	_debug_entertrace("parseHelp");
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
	(*pC)=rghPtr;
	QString szTrg;
	szCmd=szCmd.stripWhiteSpace();
	m_lpHelp->doHelp(lpC,szCmd);
	_debug_leavetrace("parseHelp");
	return (_KVI_CMD_parseHelp|_KVI_ERR_Success);
}
//============ parseSend ============//
int KviUserParser::parseSend(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseSend");
	if(!(**pC))return (_KVI_CMD_parseSend|_KVI_ERR_MissingTarget);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseSend|_KVI_ERR_MissingTarget);
	QString szTargets;
	int idx=szCmd.find(' ');
	if(idx != -1){
		szTargets=szCmd.left(idx);
		szCmd.remove(0,idx+1);
	} else {
		szTargets=szCmd.copy();
		szCmd="";
	}
	szCmd=szCmd.stripWhiteSpace();
	QString szTrg="";
	while(!szTargets.isEmpty()){
		int idx=szTargets.find(',');
		if(idx==-1){ szTrg=szTargets.copy(); szTargets=""; }
		else { szTrg=szTargets.left(idx); szTargets.remove(0,idx+1); }
		szTrg=szTrg.stripWhiteSpace();
		if(szCmd.isEmpty()){
			QString szSnd="Choose a file to send to ";
			szSnd+=szTrg;
			m_lpFrm->m_bFocusManagingEnabled=false;
			szCmd=QString(KFileDialog::getOpenFileName(0,0,m_lpFrm,szSnd.data()));
			m_lpFrm->m_bFocusManagingEnabled=true;
		}
		if(!szCmd.isEmpty()){
			QString szSendName="[DCC-SEND]:-"+szTrg+" "+szCmd;
			if(m_lpMdi->findWindow(szSendName.data())||(szSendName==m_szLastSendTarget)){
				lpC->doFmtOutput(KVI_OUT_ERROR,"[DCC SEND] : A previous DCC SEND with the same file name to %s already exists",szTrg.data());
			} else {
				m_szLastSendTarget=szSendName.copy();
				KviSendWnd *lpW=m_lpFrm->createSendWnd(szSendName.data(),m_lpOpt->bMinimizeDCCSend);
				lpW->requestSend(szTrg.data(),szCmd.data());
			}
		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseSend");
	return (_KVI_CMD_parseSend|_KVI_ERR_Success);
}
//============ parseKick ============//
int KviUserParser::parseKban(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseKban");
	if(!(**pC))return (_KVI_CMD_parseKban|_KVI_ERR_MissingChannel);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseKban|_KVI_ERR_MissingChannel);
	QString szChannel;
	int idx=0;
	if((szCmd[0]=='#')||(szCmd[0]=='&')){
		idx=szCmd.find(' ');
		if(idx != -1){
			szChannel=szCmd.left(idx);szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();
			if(szCmd.isEmpty())return (_KVI_CMD_parseKban|_KVI_ERR_MissingNick);
		} else return (_KVI_CMD_parseKban|_KVI_ERR_MissingNick);
	} else {
		if(lpC->type()==KVI_WND_TYPE_CHAN)szChannel=lpC->name();
		else return (_KVI_CMD_parseKban|_KVI_ERR_NotOnChannel);
	}
	QString szTrg;
	idx=szCmd.find(' ');
	if(idx != -1){
		szTrg=szCmd.left(idx);
		szCmd.remove(0,idx+1);
		szCmd=szCmd.stripWhiteSpace();
		if(szCmd.isEmpty()){
			if(m_lpOpt->szKickMsg.isEmpty())szCmd="Bye";
			else {szCmd=m_lpOpt->szKickMsg;m_lpIdentifiers->substitute(lpC,szCmd,pP);}
		}
	} else {
		szTrg=szCmd.copy();
		if(m_lpOpt->szKickMsg.isEmpty())szCmd="Bye";
		else { szCmd=m_lpOpt->szKickMsg;m_lpIdentifiers->substitute(lpC,szCmd,pP);}
	}
	QString szTarget;
	while(!szTrg.isEmpty()){
		idx=szTrg.find(',');
		if(idx==-1){ szTarget=szTrg; szTrg=""; }
		else { szTarget=szTrg.left(idx); szTrg.remove(0,idx+1); }
		if(m_lpSock->sendFmtData("KICK %s %s :%s",szChannel.data(),szTarget.data(),szCmd.data()))return (_KVI_CMD_parseKban|_KVI_ERR_NotConnected);
		m_lpSock->sendFmtData("MODE %s +b %s",szChannel.data(),szTarget.data());
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseKban");
	return (_KVI_CMD_parseKban|_KVI_ERR_Success);
}
//============ parseWhowas ============//
int KviUserParser::parseWhowas(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseWhowas");
	if(!(**pC))return (_KVI_CMD_parseWhowas|_KVI_ERR_MissingNick);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseWhowas|_KVI_ERR_MissingNick);
	int idx=szCmd.find(' ');
	QString szTrg;
	if(idx == -1){ szTrg=szCmd.copy(); szCmd=""; }
	else { szTrg=szCmd.left(idx); szCmd.remove(0,idx+1); }
	szCmd=szCmd.stripWhiteSpace();
	QString szT;
	while(!szTrg.isEmpty()){
		idx=szTrg.find(',');
		if(idx==-1){ szT=szTrg.copy(); szTrg=""; }
		else { szT=szTrg.left(idx); szTrg.remove(0,idx+1); }
		if(szCmd.isEmpty()){
			if(m_lpSock->sendFmtData("WHOWAS %s",szT.data()))return (_KVI_CMD_parseWhowas|_KVI_ERR_NotConnected);
		} else {
			if(m_lpSock->sendFmtData("WHOWAS %s %s",szT.data(),szCmd.data()))return (_KVI_CMD_parseWhowas|_KVI_ERR_NotConnected);
		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseWhowas");
	return (_KVI_CMD_parseWhowas|_KVI_ERR_Success);
}
//============ parseUserlist ============//
int KviUserParser::parseUserlist(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseUserlist");
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd="";
	if(**pC){
		szCmd=QString((*pC),(rghPtr-(*pC))+1);
		m_lpIdentifiers->substitute(lpC,szCmd,pP);szCmd=szCmd.stripWhiteSpace();
	}
	QString szChan="";
	if(szCmd.isEmpty()){
		if(lpC->type() != KVI_WND_TYPE_CHAN)return (_KVI_CMD_parseUserlist|_KVI_ERR_NotOnChannel);
		else szChan=lpC->name();
	} else {
		if(szCmd[0]=='#' || szCmd[0]=='&'){
			int idx=szCmd.find(' ');
			if(idx != -1){ szChan=szCmd.left(idx); szCmd.remove(0,idx+1); }
			else { szChan=szCmd.copy(); szCmd=""; }
		} else {
			if(lpC->type() != KVI_WND_TYPE_CHAN)return (_KVI_CMD_parseUserlist|_KVI_ERR_NotOnChannel);
			else szChan=lpC->name();
		}
	}
	QString szTrg;
	KviChanWnd *lpWnd=0;
	KviMdiChild *lpChild=0;
	if(!szChan.isEmpty()){
		lpChild=m_lpMdi->findWindow(szChan.data());
		if(!lpChild)return (_KVI_CMD_parseUserlist|_KVI_ERR_WindowNotFound);
		if(lpChild->type() != KVI_WND_TYPE_CHAN)return (_KVI_CMD_parseUserlist|_KVI_ERR_NotOnChannel);
		lpWnd=(KviChanWnd *)lpChild;
	} else return (_KVI_CMD_parseUserlist|_KVI_ERR_NotOnChannel);
	if(!szCmd.isEmpty()){
		while(!szCmd.isEmpty()){
			int idx=szCmd.find(',');
			if(idx==-1){ szTrg=szCmd; szCmd=""; }
			else { szTrg=szCmd.left(idx); szCmd.remove(0,idx+1); }
			szTrg=szTrg.stripWhiteSpace();
			KviNick *lpN=lpWnd->getNick(szTrg.data());
			if(lpN){
				QString szWho="[ USER : ";
				szWho+=szTrg;
				if(szWho.length()<19)szWho.setExpand(18,' ');
				szWho+=" ] is ";
				szWho+=lpN->szNick;
				szWho+='!';
				szWho+=lpN->szUserName;
				szWho+='@';
				szWho+=lpN->szAddress;
				lpC->doOutput(KVI_OUT_WHO,szWho.data());
			} else {
				lpC->doFmtOutput(KVI_OUT_ERROR,"User %s not found on %s",szTrg.data(),szChan.data());
			}
		}
	} else {
		lpWnd->doUserList();
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseUserlist");
	return (_KVI_CMD_parseUserlist|_KVI_ERR_Success);
}
//============ parseClonescan ============//
int KviUserParser::parseClonescan(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseClonescan");
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd="";
	if(**pC){
		szCmd=QString((*pC),(rghPtr-(*pC))+1);
		m_lpIdentifiers->substitute(lpC,szCmd,pP);szCmd=szCmd.stripWhiteSpace();
	}
	QString szChan="";
	if(szCmd.isEmpty()){
		if(lpC->type() != KVI_WND_TYPE_CHAN)return (_KVI_CMD_parseClonescan|_KVI_ERR_NotOnChannel);
		else szChan=lpC->name();
	} else {
		if(szCmd[0]=='#' || szCmd[0]=='&'){
			int idx=szCmd.find(' ');
			if(idx != -1){ szChan=szCmd.left(idx); szCmd.remove(0,idx+1); }
			else { szChan=szCmd.copy(); szCmd=""; }
		} else {
			if(lpC->type() != KVI_WND_TYPE_CHAN)return (_KVI_CMD_parseClonescan|_KVI_ERR_NotOnChannel);
			else szChan=lpC->name();
		}
	}
	QString szTrg;
	KviChanWnd *lpWnd=0;
	KviMdiChild *lpChild=0;
	if(!szChan.isEmpty()){
		lpChild=m_lpMdi->findWindow(szChan.data());
		if(!lpChild)return (_KVI_CMD_parseClonescan|_KVI_ERR_WindowNotFound);
		if(lpChild->type() != KVI_WND_TYPE_CHAN)return (_KVI_CMD_parseClonescan|_KVI_ERR_NotOnChannel);
		lpWnd=(KviChanWnd *)lpChild;
	} else return (_KVI_CMD_parseClonescan|_KVI_ERR_NotOnChannel);
	bool bFound=false;
	if(!szCmd.isEmpty()){
		while(!szCmd.isEmpty()){
			int idx=szCmd.find(',');
			if(idx==-1){ szTrg=szCmd; szCmd=""; }
			else { szTrg=szCmd.left(idx); szCmd.remove(0,idx+1); }
			szTrg=szTrg.stripWhiteSpace();
			KviNick *lpN=lpWnd->getNick(szTrg.data());
			bFound=lpWnd->doSingleCloneScan(lpN);
		}
		if(!bFound)lpC->doOutput(KVI_OUT_CLONE,"CLONESCAN TERMINATED : No clones found");
	} else {
		lpWnd->doGlobalCloneScan();
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseClonescan");
	return (_KVI_CMD_parseClonescan|_KVI_ERR_Success);
}
//============ parseTimer ============//
// timer start|stop|list|killall <name> 
int KviUserParser::parseTimer(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseTimer");
	if(!(**pC))return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerName);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
	m_lpIdentifiers->substitute(lpC,szCmd,pP);
	szCmd=szCmd.stripWhiteSpace();
	if(szCmd.isEmpty())return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerName);
	int idx=szCmd.find(' ');
	QString szOperation;
	if(idx != -1){ szOperation=szCmd.left(idx); szCmd.remove(0,idx+1); }
	else { szOperation=szCmd; szCmd=""; }
	if(!strncasecmp(szOperation.data(),"start",5)){
		if(szCmd.isEmpty())return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerName);
		int idx=szCmd.find(' ');
		if(idx != -1){
			QString szTimerName=szCmd.left(idx);
			szCmd.remove(0,idx+1);
			szCmd=szCmd.stripWhiteSpace();
			idx=szCmd.find(' ');
			if(idx != -1){
				QString szTimeout=szCmd.left(idx);
				szCmd.remove(0,idx+1);
				szCmd=szCmd.stripWhiteSpace();
				bool bOK=false;
				uint iTimeout=szTimeout.toUInt(&bOK);
				if(bOK){
					if(iTimeout<100)return (_KVI_CMD_parseTimer|_KVI_ERR_TooSmallTimeout);
					if(szCmd.isEmpty())return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerCommand);
					szCmd+=';'; //terminate the command...need it (?)
					m_lpTimerManager->startNamedTimer(szTimerName.data(),szCmd.data(),lpC,iTimeout);
				} else return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerTime);
			} else return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerCommand);
		} else return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerTime);	
	} else if(!strncasecmp(szOperation.data(),"shot",4)){
		if(szCmd.isEmpty())return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerTime);
		int idx=szCmd.find(' ');
		if(idx != -1){
			QString szTimeout=szCmd.left(idx);
			szCmd.remove(0,idx+1);
			szCmd=szCmd.stripWhiteSpace();
			bool bOK=false;
			uint iTimeout=szTimeout.toUInt(&bOK);
			if(bOK){
				if(iTimeout<100)return (_KVI_CMD_parseTimer|_KVI_ERR_TooSmallTimeout);
				if(szCmd.isEmpty())return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerCommand);
				szCmd+=';'; //terminate the command...need it (?)
				m_lpTimerManager->startNamedTimer("_SINGLE SHOT_",szCmd.data(),lpC,iTimeout);
			} else return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerTime);
		} else return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerCommand);	
	} else if(!strncasecmp(szOperation.data(),"stop",4)){
		szCmd=szCmd.stripWhiteSpace();
		if(szCmd.isEmpty())return (_KVI_CMD_parseTimer|_KVI_ERR_MissingTimerName);
		m_lpTimerManager->stopTimer(szCmd.data());
	} else if(!strncasecmp(szOperation.data(),"list",4)){
		KviTimerStruct *lpT=0;
		if(m_lpTimerManager->m_lpTimerList->isEmpty()){
			lpC->doOutput(KVI_OUT_TIME,"Timer list : No active timers.");
		} else {
			for(lpT=m_lpTimerManager->m_lpTimerList->first();lpT;lpT=m_lpTimerManager->m_lpTimerList->next()){
				lpC->doFmtOutput(KVI_OUT_TIME,"Timer : [%s] Timeout : %d msec. Window : %s Command : %s",
						lpT->szName.data(),lpT->interval,lpT->lpWindow->name(),lpT->szCommand.data());
			}
		}
	} else if(!strncasecmp(szOperation.data(),"kill",4)){
		m_lpTimerManager->stopAllTimers();
	} else return (_KVI_CMD_parseTimer|_KVI_ERR_UnknownTimerCmd);
	(*pC)=rghPtr;
	_debug_leavetrace("parseTimer");
	return (_KVI_CMD_parseTimer|_KVI_ERR_Success);
}
//============ parseRun ============//
int KviUserParser::parseRun(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseRun");
	if(!(**pC))return (_KVI_CMD_parseRun|_KVI_ERR_MissingProcName);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseRun|_KVI_ERR_MissingProcName);
	m_lpProcessManager->runProcess(szCmd.data());
	(*pC)=rghPtr;
	_debug_leavetrace("parseRun");
	return (_KVI_CMD_parseRun|_KVI_ERR_Success);
}
//============ parseExec ============//
int KviUserParser::parseExec(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseExec");
	if(!(**pC))return (_KVI_CMD_parseExec|_KVI_ERR_MissingProcName);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseExec|_KVI_ERR_MissingProcName);
	m_lpProcessManager->execProcess(szCmd.data());
	(*pC)=rghPtr;
	_debug_leavetrace("parseExec");
	return (_KVI_CMD_parseExec|_KVI_ERR_Success);
}
//============ parseProclist ============//
int KviUserParser::parseProclist(KviMdiChild *lpC,const char ** pC,KviAliasParams *)
{
	_debug_entertrace("parseProclist");
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	KviProcParams *lpP=0;
	if(m_lpProcessManager->m_lpProcList->isEmpty()){
		lpC->doFmtOutput(KVI_OUT_STDIN,"Proclist : No processes running.");
	} else {
		for(lpP=m_lpProcessManager->m_lpProcList->first();lpP;lpP=m_lpProcessManager->m_lpProcList->next()){
			lpC->doFmtOutput(KVI_OUT_STDIN,"Pid : %d  Commandline : %s",lpP->lpProc->getPid(),lpP->szCmdLine.data());
		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseProclist");
	return (_KVI_CMD_parseProclist|_KVI_ERR_Success);
}
//============ parseKillproc ============//
int KviUserParser::parseKillproc(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseKillproc");
	if(!(**pC))return (_KVI_CMD_parseKillproc|_KVI_ERR_MissingProcPid);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseKillproc|_KVI_ERR_MissingProcPid);
	if(!strcasecmp(szCmd.data(),"all")){
		m_lpProcessManager->killAll();
	} else {
		bool bOK=false;
		int aPid=szCmd.toInt(&bOK);
		if(!bOK)return (_KVI_CMD_parseKillproc|_KVI_ERR_MissingProcPid);
		m_lpProcessManager->killProcess(aPid);
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseKillproc");
	return (_KVI_CMD_parseKillproc|_KVI_ERR_Success);
}
//============ parseWriteproc ============//
int KviUserParser::parseWriteproc(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseWriteproc");
	if(!(**pC))return (_KVI_CMD_parseWriteproc|_KVI_ERR_MissingProcPid);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseWriteproc|_KVI_ERR_MissingProcPid);
	int idx=szCmd.find(' ');
	QString szPid;
	if(idx==-1){ szPid=szCmd.copy(); szCmd=""; }
	else { szPid=szCmd.left(idx); szCmd.remove(0,idx+1); szCmd=szCmd.stripWhiteSpace(); }
	bool bOK=false;
	int aPid=szPid.toInt(&bOK);
	if(!bOK)return (_KVI_CMD_parseWriteproc|_KVI_ERR_MissingProcPid);
	//Append a LF to the end of the string...maybe
	//Make it optional for next version
	szCmd+='\n';
	m_lpProcessManager->writeProcess(aPid,szCmd.data());
	(*pC)=rghPtr;
	_debug_leavetrace("parseWriteproc");
	return (_KVI_CMD_parseWriteproc|_KVI_ERR_Success);
}
//============ parsePlay ============//
int KviUserParser::parsePlay(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parsePlay");
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty()){
		QString szSnd="Choose a file to play";
		QString szHome(getKVIrcHomeDirectory("Audio"));
		m_lpFrm->m_bFocusManagingEnabled=false;
		szCmd=QString(KFileDialog::getOpenFileName(szHome.data(),0,m_lpFrm,szSnd.data()));
		m_lpFrm->m_bFocusManagingEnabled=true;
		if(szCmd.isEmpty())return (_KVI_CMD_parsePlay|_KVI_ERR_Success);
	}
	switch(m_lpFrm->m_lpAudio->playSound(szCmd.data())){
		case KVI_AUDIO_FILE_NOT_FOUND:
			lpC->doFmtOutput(KVI_OUT_ERROR,"[PLAY] : Sound file %s not found",szCmd.data());
			break;
		case KVI_AUDIO_NO_SOUND_SERVER:
			lpC->doOutput(KVI_OUT_ERROR,"[PLAY] : Sound server not active or not ready.");
			break;
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parsePlay");
	return (_KVI_CMD_parsePlay|_KVI_ERR_Success);
}
//============ parseHalt ============//
int KviUserParser::parseHalt(KviMdiChild *,const char **,KviAliasParams *)
{
	_debug_entertrace("parseHalt");
	_debug_leavetrace("parseHalt");
	return (_KVI_CMD_parseHalt|_KVI_ERR_Halt);
}
//============ parseServer ============//
int KviUserParser::parseServer(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseServer");
	if(!(**pC))return (_KVI_CMD_parseServer|_KVI_ERR_MissingServer);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseServer|_KVI_ERR_MissingServer);
	int idx=szCmd.find(' ');
	QString szServer;
	int iPort=6667;
	if(idx!=-1){
		szServer=szCmd.left(idx);
		szCmd.remove(0,idx+1);
		szCmd=szCmd.stripWhiteSpace();
		bool bOK=false;
		iPort=szCmd.toInt(&bOK);
		if(!bOK)iPort=6667;
	} else szServer=szCmd;
	m_lpFrm->hopServer(szServer,iPort);
	(*pC)=rghPtr;
	_debug_leavetrace("parseServer");
	return (_KVI_CMD_parseServer|_KVI_ERR_Success);
}
//============ parseAway ============//
int KviUserParser::parseAway(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseAway");
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
	szCmd=szCmd.stripWhiteSpace();
	if(szCmd.isEmpty())szCmd=m_lpOpt->szAwayMsg.stripWhiteSpace();
	if(szCmd.isEmpty())szCmd="Away since $_date $_time";
	m_lpIdentifiers->substitute(lpC,szCmd,pP);
	if(m_lpSock->sendFmtData("AWAY :%s",szCmd.data()))return (_KVI_CMD_parseAway|_KVI_ERR_NotConnected);
	(*pC)=rghPtr;
	_debug_leavetrace("parseAway");
	return (_KVI_CMD_parseAway|_KVI_ERR_Success);
}
//============ parseBack ============//
int KviUserParser::parseBack(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseBack");
	_macro_skipUpToEndOfCmd((*pC));
	if(m_lpSock->sendData("AWAY"))return (_KVI_CMD_parseBack|_KVI_ERR_NotConnected);
	_debug_leavetrace("parseBack");
	return (_KVI_CMD_parseBack|_KVI_ERR_Success);
}
//============ parseWindow ============//
//WINDOW [open|close|hide|show] <@name> noinput hidden noclose size=x,y,w,h background=file.ext
// images=true|false timestamp=true|false font=font_name,size
int KviUserParser::parseWindow(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseWindow");
	if(!(**pC))return (_KVI_CMD_parseWindow|_KVI_ERR_MissingWindow);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseWindow|_KVI_ERR_MissingWindow);
	m_lpIdentifiers->substitute(lpC,szCmd,pP);
	int idx=szCmd.find(' ');
	QString szCommand="";
	if(idx != -1){ szCommand=szCmd.left(idx); szCmd.remove(0,idx+1);szCmd=szCmd.stripWhiteSpace();}
	else return (_KVI_CMD_parseWindow|_KVI_ERR_MissingWindow);
	if(szCmd.isEmpty())return (_KVI_CMD_parseWindow|_KVI_ERR_MissingWindow);
	if(!strncasecmp(szCommand.data(),"open",4)){
		if(szCmd[0] != '*')return (_KVI_CMD_parseWindow|_KVI_ERR_WrongWindowName);
		//check if the name contains spaces (and flags...)
		idx=szCmd.find(' ');
		QString szWinName="";
		if(idx != -1){ szWinName=szCmd.left(idx); szCmd.remove(0,idx);szCmd=szCmd.stripWhiteSpace(); }
		else { szWinName=szCmd.copy(); szCmd=""; }
		KviMdiChild *lpX=m_lpMdi->findWindow(szWinName.data());
		if(!lpX){ //create it
			bool hasInp=true;
			bool hidden=false;
			bool hasClo=true;
			QRect geom(0,0,100,100);
			bool gotSize=false;
			QString szBack="";
			KviUserWnd *lpW=0;
			if(szCmd.isEmpty()){
				lpW=m_lpFrm->createUserWnd(szWinName.data(),hidden,hasInp);
			} else {
				//check for flags.
				int spIndex=szCmd.find(' ');
				QString szArg="";
				QString szTimestamp="";
				QString szImages="";
				uint fontSize=0;
				QString szFont;
				while(!szCmd.isEmpty()){
					if(spIndex==-1){ szArg=szCmd.copy(); szCmd=""; }
					else { szArg=szCmd.left(spIndex); szCmd.remove(0,spIndex+1); szCmd=szCmd.stripWhiteSpace(); }
					if(!strcasecmp("noclose",szArg.data()))hasClo=false;
					else if(!strcasecmp("noinput",szArg.data()))hasInp=false;
					else if(!strcasecmp("hidden",szArg.data()))hidden=true;
					else if(!strncasecmp("size=",szArg.data(),5)){
						gotSize=true;
						int idxEq=szArg.find('=');
						QString szRect=szArg.right(szArg.length()-(idxEq+1));
						int g[4];
						g[0]=0;
						g[1]=0;
						g[2]=200;
						g[3]=200;
						sscanf(szRect.data(),"%d,%d,%d,%d",&g[0],&g[1],&g[2],&g[3]);
						geom.setRect(g[0],g[1],g[2],g[3]);
					} else if(!strncasecmp("background=",szArg.data(),11)){
						int idxEq=szArg.find('=');
						if(idxEq != -1)szBack=szArg.right(szArg.length()-(idxEq+1));
					} else if(!strncasecmp("timestamp=",szArg.data(),10)){
						int idxEq=szArg.find('=');
						if(idxEq != -1)szTimestamp=szArg.right(szArg.length()-(idxEq+1));		
					} else if(!strncasecmp("images=",szArg.data(),7)){
						int idxEq=szArg.find('=');
						if(idxEq != -1)szImages=szArg.right(szArg.length()-(idxEq+1));
					} else if(!strncasecmp("font=",szArg.data(),5)){
						int idxEq=szArg.find('=');
						if(idxEq != -1){
							QString szFontTmp=szArg.right(szArg.length()-(idxEq+1));
							int idx=szFontTmp.find(',');
							if(idx != -1){
								szFont=szFontTmp.left(idx);
								szFontTmp.remove(0,idx+1);
								bool bOk=false;
								fontSize=szFontTmp.toUInt(&bOk);
								if(!bOk)fontSize=12;
							} else {
								fontSize=12;
								szFont=szFontTmp.copy();
							}
						}
					}
					spIndex=szCmd.find(' ');
				}
				//debug("Back = %s",szBack.data());
				//create the wnd
				const char *szB=( szBack.isEmpty() ? 0 : szBack.data());
				QRect *ptrToGeom= (gotSize ? (&geom) : 0);
				lpW=m_lpFrm->createUserWnd(szWinName.data(),hidden,hasInp,szB,ptrToGeom);
				lpW->enableClose(hasClo);
				if(!szTimestamp.isEmpty()){
					if(!strncasecmp(szTimestamp.data(),"true",4))lpW->m_lpOutput->m_bTimestamp=true;
					else if(!strncasecmp(szTimestamp.data(),"false",5))lpW->m_lpOutput->m_bTimestamp=false;
				}
				if(!szImages.isEmpty()){
					if(!strncasecmp(szImages.data(),"true",4))lpW->m_lpOutput->m_bShowPixmaps=true;
					else if(!strncasecmp(szImages.data(),"false",5))lpW->m_lpOutput->m_bShowPixmaps=false;
				}
				if(!szFont.isEmpty()){
					QFont fnt(szFont.data(),fontSize);
					lpW->m_lpOutput->setFont(fnt);
				}
			}
		}
	} else if(!strncasecmp(szCommand.data(),"close",5)){
		if(szCmd[0] != '*')return (_KVI_CMD_parseWindow|_KVI_ERR_WrongWindowName);
		KviMdiChild *lpW=m_lpMdi->findWindow(szCmd.data());
		if(!lpW)return (_KVI_CMD_parseWindow|_KVI_ERR_WindowNotFound);
		lpW->closeSlot();
	} else if(!strncasecmp(szCommand.data(),"hide",4)){
		KviMdiChild *lpW=m_lpMdi->findWindow(szCmd.data());
		if(!lpW)return (_KVI_CMD_parseWindow|_KVI_ERR_WindowNotFound);
		if(!lpW->m_bMinimized)lpW->minimizeWindow(); //minimize
		m_lpMdi->focusTopChild();
	} else if(!strncasecmp(szCommand.data(),"show",4)){
		KviMdiChild *lpW=m_lpMdi->findWindow(szCmd.data());
		if(!lpW)return (_KVI_CMD_parseWindow|_KVI_ERR_WindowNotFound);
		if(lpW->m_bMinimized)lpW->minimizeWindow(); //restore
		lpW->raise();
		lpW->setFocus();
	} else if(!strncasecmp(szCommand.data(),"maximize",8)){
		KviMdiChild *lpW=m_lpMdi->findWindow(szCmd.data());
		if(!lpW)return (_KVI_CMD_parseWindow|_KVI_ERR_WindowNotFound);
		if(!lpW->m_bMaximized)lpW->maximizeWindow();
		lpW->raise();
		lpW->setFocus();
	} else if(!strncasecmp(szCommand.data(),"restore",7)){
		KviMdiChild *lpW=m_lpMdi->findWindow(szCmd.data());
		if(!lpW)return (_KVI_CMD_parseWindow|_KVI_ERR_WindowNotFound);
		if(lpW->m_bMaximized)lpW->maximizeWindow();
		lpW->raise();
		lpW->setFocus();
	} else if(!strncasecmp(szCommand.data(),"setimage",8)){
		QString szWinName="";
		if(szCmd.isEmpty())return (_KVI_CMD_parseWindow|_KVI_ERR_MissingWindow);
		idx=szCmd.find(' ');
		if(idx != -1){ szWinName=szCmd.left(idx); szCmd.remove(0,idx);szCmd=szCmd.stripWhiteSpace(); }
		else return (_KVI_CMD_parseWindow|_KVI_ERR_MissingFileName);
		KviMdiChild *lpW=m_lpMdi->findWindow(szWinName.data());
		if(!lpW)return (_KVI_CMD_parseWindow|_KVI_ERR_WindowNotFound);
		if(lpW->m_lpOutput){
			lpW->m_lpOutput->setUserBackground(szCmd.data());
			lpW->m_lpOutput->repaint(false);
		}
	} else return (_KVI_CMD_parseWindow|_KVI_ERR_UnknownWindowCommand);
	(*pC)=rghPtr;
	_debug_leavetrace("parseWindow");
	return (_KVI_CMD_parseWindow|_KVI_ERR_Success);
}
//============ parseForeach ============//
int KviUserParser::parseForeach(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseForeach");
	if(!(**pC))return (_KVI_CMD_parseForeach|_KVI_ERR_MissingExpression);
	if((**pC) != '(')return (_KVI_CMD_parseForeach|_KVI_ERR_UnexpectedChar);
	const char *rghPtr=(*pC);
	int match=1;
	while(match){
		(rghPtr)++;
		if(!(*rghPtr))return (_KVI_CMD_parseForeach|_KVI_ERR_UnexpectedEndInExpr);
		if(*rghPtr=='(')match++;
		if(*rghPtr==')')match--;
	}
	(rghPtr)++;
	if(!(*rghPtr))return (_KVI_CMD_parseForeach|_KVI_ERR_UnexpectedEndInExpr);
	(*pC)++; //skip the first (
	QString szExpr((*pC),(rghPtr-(*pC)));
	(*pC)=rghPtr;
	//Got the expression
	_macro_skipSenselessChars((*pC));
	QString szThen;
	if(!(**pC))return (_KVI_CMD_parseForeach|_KVI_ERR_MissingCommand);
	if(**pC==KVI_CHAR_BEGIN){ //strip up to matching brace
		rghPtr=(*pC);
		match=1;
		while(match){
			(rghPtr)++;
			if(!(*rghPtr))return (_KVI_CMD_parseForeach|_KVI_ERR_UnexpectedEndInCmd);
			if(*rghPtr==KVI_CHAR_BEGIN)match++;
			if(*rghPtr==KVI_CHAR_END)match--;
		}
		(rghPtr)++;
		if(!(*rghPtr))return (_KVI_CMD_parseForeach|_KVI_ERR_UnexpectedEndInCmd);
		szThen=QString((*pC),(rghPtr-(*pC))+1);
		if(*rghPtr !=';')return (_KVI_CMD_parseForeach|_KVI_ERR_MissingSeparator);
		(*pC)=rghPtr;
	} else { //strip up to ;
		rghPtr=(*pC);
		_macro_skipUpToEndOfCmd(rghPtr);
		szThen=QString((*pC),(rghPtr-(*pC))+1);
		(*pC)=rghPtr;
	}
	////Now evaluate the parameters passed
	QString szVar;
	int idx=szExpr.find(',');
	if(idx > 1){
		szVar=szExpr.left(idx);szExpr.remove(0,idx+1);
	} else return (_KVI_CMD_parseForeach|_KVI_ERR_MissingVariable);

	KviVariable *lpX=0;
	int varIdx=0;
	if((szVar[1]>='0')&&(szVar[1]<='9')&&(szVar.length()==2)){
		if(!pP)return (_KVI_CMD_parseForeach|_KVI_ERR_ThisIsNotAnAlias);
		szVar.remove(0,1);
		varIdx=szVar.toInt();
	} else {
		KviVariable *lpV=0;
		for(lpV=m_lpVarList->first();((lpV) && (!lpX));lpV=m_lpVarList->next()){
			if(!strncasecmp(lpV->szName.data(),szVar.data(),strlen(lpV->szName.data()))){
				lpX=lpV;
			}
		}
		if(!lpX){
			lpX=new KviVariable;
			lpX->szName=szVar.copy();
			m_lpVarList->append(lpX);
		}
	}
	//now lpX points to the var to set or is 0 --> varIdx is the index to the parameter to store
	m_lpIdentifiers->substitute(lpC,szExpr,pP);
	QString * pStr=( lpX ? (&(lpX->szValue)) : (&(pP->param[varIdx])) );
	while(!szExpr.isEmpty()){
		int cmdIdx=szExpr.find(',');
		if(cmdIdx != -1){ (*pStr)=szExpr.left(cmdIdx); szExpr.remove(0,cmdIdx+1); }
		else { (*pStr)=szExpr.copy(); szExpr=""; }
		const char *pCmd=szThen.data();
		int numErr=parseUserCommand(lpC,&pCmd,pP);
		if(numErr & _KVI_ERR_ErrorMask)return numErr;
		if(lpX){ //we were using a variable
			if(m_lpVarList->findRef(lpX) == -1){ //someone deleted the variable
				//recreate it
				lpX=new KviVariable;
				lpX->szName=szVar.copy();
				m_lpVarList->append(lpX);
				pStr = (&lpX->szValue);
			}
		}
	}
	if(lpX)m_lpVarList->removeRef(lpX);
	_debug_leavetrace("parseForeach");
	return (_KVI_CMD_parseForeach|_KVI_ERR_Success);
}
//============ parseDns ============//
int KviUserParser::parseDns(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseDns");
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
	szCmd=szCmd.stripWhiteSpace();
	if(szCmd.isEmpty())return (_KVI_CMD_parseDns|_KVI_ERR_NoHostToResolve);
	m_lpIdentifiers->substitute(lpC,szCmd,pP);
	if(!m_lpFrm->m_lpDns->queryDns(szCmd.data()))return (_KVI_CMD_parseDns|_KVI_ERR_CantStartResolver);
	(*pC)=rghPtr;
	_debug_leavetrace("parseDns");
	return (_KVI_CMD_parseDns|_KVI_ERR_Success);
}
//============ parseAscii ============//
int KviUserParser::parseAscii(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseAscii");
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	int timeout=1000;
	KviMdiChild *lpWnd=lpC;
	QString szFile="";
	if(szCmd.isEmpty()){
		QString szSnd="Choose a file to play";
		QString szHome(getKVIrcHomeDirectory("Ascii"));
		m_lpFrm->m_bFocusManagingEnabled=false;
		szFile=QString(KFileDialog::getOpenFileName(szHome.data(),0,m_lpFrm,szSnd.data()));
		m_lpFrm->m_bFocusManagingEnabled=true;
		if(szFile.isEmpty())return (_KVI_CMD_parseAscii|_KVI_ERR_Success);
	} else { //at least file name
		int idx=szCmd.find(' ');
		if(idx==-1){
			szFile=szCmd.copy();
		} else {
			szFile=szCmd.left(idx);
			szCmd.remove(0,idx+1);
			szCmd=szCmd.stripWhiteSpace();
			QString szWindow;
			idx=szCmd.find(' ');
			if(idx==-1){
				szWindow=szCmd.copy();
			} else {
				szWindow=szCmd.left(idx);
				szCmd.remove(0,idx+1);
				szCmd=szCmd.stripWhiteSpace();
				bool bOk=false;
				timeout=szCmd.toUInt(&bOk);
				if(!bOk)timeout=1000;
			}
			KviMdiChild *lpX=m_lpFrm->m_lpMdi->findWindow(szWindow.data());
			if(lpX)lpWnd=lpX;
			else {
				lpC->doFmtOutput(KVI_OUT_ERROR,"[ASCII] : Can't play file : window %s not found",szWindow.data());
				(*pC)=rghPtr;
				_debug_leavetrace("parseAscii");
				return (_KVI_CMD_parseAscii|_KVI_ERR_Success);
			}
		}
	}
	if(!strncasecmp(szFile.data(),"-stop",5))m_lpFrm->m_lpAsciiPlayer->stopAll();
	else {
		if(!m_lpFrm->m_lpAsciiPlayer->playAscii(szFile.data(),lpWnd,timeout)){
			lpC->doFmtOutput(KVI_OUT_ERROR,"[ASCII] : Can't play file : file %s not found",szFile.data());
		}
	}
	(*pC)=rghPtr;
	_debug_leavetrace("parseAscii");
	return (_KVI_CMD_parseAscii|_KVI_ERR_Success);
}
//============ parseCache ============//
//cache load section key variable
//cache store section key data
//cache clear
//cache remove section key
//cache remove section
//cache flush
int KviUserParser::parseCache(KviMdiChild *lpC,const char ** pC,KviAliasParams *pP)
{
	_debug_entertrace("parseCache");
	if(!(**pC))return (_KVI_CMD_parseCache|_KVI_ERR_MissingOperation);
	const char *rghPtr=(*pC);
	_macro_skipUpToEndOfCmd(rghPtr);
	QString szCmd((*pC),(rghPtr-(*pC))+1);
//	m_lpIdentifiers->substitute(lpC,szCmd,pP);
	szCmd=szCmd.stripWhiteSpace();
//	_macro_getCommandParameters(szCmd,(*pC),rghPtr,lpC,pP);
	if(szCmd.isEmpty())return (_KVI_CMD_parseCache|_KVI_ERR_MissingOperation);
	if(!strncasecmp(szCmd.data(),"load",4)){
		KviIrcToken token(szCmd);
		QString szDum=token.nextWord();
		QString szSec=token.nextWord();
		QString szKey=token.nextWord();
		QString szVar=token.nextWord();
		if(szSec.isEmpty() || szKey.isEmpty() || (szVar.length()<2))return (_KVI_CMD_parseCache|_KVI_ERR_MissingKey);
		if(szVar[0] != '$')return (_KVI_CMD_parseCache|_KVI_ERR_MissingKey);
		if((szVar[1] == '%')||(szVar[1]=='_'))return (_KVI_CMD_parseCache|_KVI_ERR_CantSetSystemVar);
		QString szVal=0;
		szVal=m_lpCache->getKey(szSec.data(),szKey.data());
		if(szVal.isEmpty())szVal=KVI_STR_NULL;
		if((szVar[1]>='0')&&(szVar[1]<='9')&&(szVar.length()==2)){
			if(!pP)return (_KVI_CMD_parseCache|_KVI_ERR_ThisIsNotAnAlias);
			szVar.remove(0,1);
			int index=szVar.toInt();
			pP->param[index]=szVal.copy();
		} else {
			KviVariable *lpV=0;
			for(lpV=m_lpVarList->first();lpV;lpV=m_lpVarList->next()){
				if(!strncasecmp(lpV->szName.data(),szVar.data(),strlen(lpV->szName.data()))){
					lpV->szValue=szVal.copy();
					(*pC)=rghPtr;
					return (_KVI_CMD_parseCache|_KVI_ERR_Success);
				}
			}
			lpV=new KviVariable;
			lpV->szName=szVar.copy();
			lpV->szValue=szVal.copy();
			m_lpVarList->append(lpV);
		}
	} else if(!strncasecmp(szCmd.data(),"store",5)){
		m_lpIdentifiers->substitute(lpC,szCmd,pP);
		KviIrcToken token(szCmd);
		QString szDum=token.nextWord();
		QString szSec=token.nextWord();
		QString szKey=token.nextWord();
		if(szSec.isEmpty() || szKey.isEmpty() || token.szString.isEmpty())return (_KVI_CMD_parseCache|_KVI_ERR_MissingKey);
		m_lpCache->insertKey(szSec.data(),szKey.data(),token.szString.data());
	} else if(!strncasecmp(szCmd.data(),"clear",5)){
		m_lpCache->clearCache();
	} else if(!strncasecmp(szCmd.data(),"show",4)){
		m_lpCache->echoToWindow(lpC);
	} else if(!strncasecmp(szCmd.data(),"flush",5)){
		if(!m_lpCache->writeCache()){
			KMsgBox::message(m_lpFrm,"File I/O error","Unable to flush cache.");
		}
	} else if(!strncasecmp(szCmd.data(),"remove",6)){
		m_lpIdentifiers->substitute(lpC,szCmd,pP);
		KviIrcToken token(szCmd);
		QString szDum=token.nextWord();
		QString szSec=token.nextWord();
		QString szKey=token.nextWord();
		if(szSec.isEmpty())return (_KVI_CMD_parseCache|_KVI_ERR_MissingKey);
		if(szKey.isEmpty()){
			m_lpCache->removeSection(szSec.data());
		} else {
			m_lpCache->removeKey(szSec.data(),szKey.data());
		}
	} else return (_KVI_CMD_parseCache|_KVI_ERR_UnknownCacheOperation);
	(*pC)=rghPtr;
	_debug_leavetrace("parseCache");
	return (_KVI_CMD_parseCache|_KVI_ERR_Success);
}
